<?php
namespace aosi{
	/*
	izlazna vrijednost false bilo koje metode oznacava neuspjeh, a true ili vrijednost > 0 oznacava uspjeh
	nakon izvrsenja bilo koje metode, ako se dogodila greska poruka o gresci moze se dobiti metodom $obj->err_message(), a kod greske metodom  $obj->err_code();
	
	
	*/
	class Aosi_core extends Simple{
		
		private $ldap;
		
		function __construct() {
			$this->ldap=new Ldap();
		}
		/*Funkcija primarno služi autentikaciji korisnika. 
		Uz rezultat binda (autentikaciju) provjerava status korisnika u smislu politike obavezne promjene zaporke
		Ako korisnik jos nije sam promijenio zaporku, nakon uspjesne autentikacije metoda $obj->result() vraca array:
			'authn_left'=>broj preostalih pokusaja autentikacije prije zakljucavanja acc-a,
			'pwd_expires_on'=>vrijeme isteka zaporke u formatu 'd.m.Y. H:i:s',
			'is_exipred'=>je li zaporka istekla 1|0,
			'time_left'=> ako zaporka nije istekla, za koliko vremena istice (formatirano)
			'time_left_s'=> ako zaporka nije istekla, za koliko vremena istice (u sekundama)
		Ulazne vrijednosti:
			$uid= uid korisnika u imeniku
			$password=base64 encodiran password,
			$base=base dn
		Izlazna vrijednost: 
			false: autentikacija neuspjesna
			true: autentikacija uspjesna
		*/
		public function userBind($uid,$password,$base){
			list($uid,$base)=$this->clean($uid,$base);
			if($this->ldap->userBind($uid,$password,$base)){
				$this->set_result_obj($this->ldap);
				\aosi\Logger::write('Autentikacija uspjesna dn:' . $uid .','. $base ,6);
				return true;
			} else {
				$this->set_error_obj($this->ldap);
				\aosi\Logger::write($this->err_message() . ' dn:' . $uid .','. $base . " code:" . $this->err_code(),6);
				return false;
			}
		}
		
		/*Funkcija primarno služi autentikaciji / autorizaciji administratora imenika. 
		Uz rezultat binda (autentikaciju) provjerava ima li osoba administratorska prava.
		Ulazne vrijednosti:
		$uid= uid korisnika u imeniku
		$password=base64 encodiran password,
		$base=base dn
		Izlazna vrijednost: 
		false: autentikacija neuspjesna
		0 - Korisnik: autentikacija uspjesna, ali korisnik nema pravo administracije imenika, 
		>0 - Administrator: 1 - Administator definiran u konfiguracijskoj datoteci, 2 - Administrator definiran u imeniku, 3 - Administrator iz imenika s ograničenjem
		Ako je izlazna vrijednost 3 (administrator s ogranicenjem), onda metoda result ovog objekta vraca hash s ogranicenjima ['naziv_atributa'=>['vrijednost1','vrijednost2',..],['']=>['',''..]
		*/
		public function ldapBind($uid,$password,$base){
			list($uid,$base)=$this->clean($uid,$base);
			$b=$this->ldap->ldapBind($uid,$password,$base);
			$this->set_error_obj($this->ldap);
			if($b===false){
				\aosi\Logger::write($this->err_message() . ' dn:' . $uid .','. $base . " code:" . $this->err_code(),6);
				return false;
			}
			switch($b){
				case 0:
					$this->set_err_code(-15);
					$this->set_err_message('User uid=' . $uid .','. $base . ' does not have admin privileges!');
					\aosi\Logger::write('Autentikacija uspjesna ali korisnik nije administrator dn: uid=' . $uid .','. $base ,6);
					break;
				case 3:
					$this->set_result($this->ldap->result());
				case 1:
				case 2:
					\aosi\Logger::write('Autentikacija uspjesna dn:' . $uid .','. $base . " adminType=$b",6);
					break;
				default:
					return false;
			}
			return $b;
		}
		
		/*Funkcija vraca zapise koji zadovoljavaju uvjete pretrage zadane ldap filterom i omogucuje paginaciju
		Zapisi su uvijek sortirani po dn-u
		po izvrsenju funkcije metoda $obj->result(); vraca array u kojem je ['count'] ukupan broj zapisa koji zadovoljavaju uvjete pretrage
		Ulazni podaci:
			$uid: uid korisnika u imeniku
			$password: base64 enkodiran password korisnika iz imenika
			$base: base dn korisnika
			$filter: ldap filter
			$attributes: array atributa koji se dohvacaju
			$from: redni broj zapisa od kojeg se treba popis prikazati
			$size: broj zapisa koji se prikazuju
		Izlaz:
			array (u obliku koji vraca funkcija ldap_result_to_array) zapisa koji zadovoljavaju uvjete pretrage
			false ili 0 ako nema nijednog zapisa koji zadovoljava uvjete pretrage
		*/
		public function ldapList($uid,$password,$base,$filter,$attributes,$from,$size){
			list($uid,$base)=$this->clean($uid,$base);
			$attributes=$this->chk_filter_attrs($filter,$attributes);
			if(!is_array($attributes)){
				return false;
			}
			$ret=$this->ldap->ldapList($uid,$password,$base,$filter,$attributes,$from,$size);
			$this->set_error_obj($this->ldap);
			$this->set_result_obj($this->ldap);
			return $ret;
		}
		/*Funkcija sluzi provjeri postoji li korisnik u imeniku u ime administratora. 
		Ulazne vrijednosti:
			$username: uid administratora u imeniku
			$password: base64 enkodiran password administratora iz imenika
			$base: base dn korisnika
			$uid: uid korisnika za kojeg se radi provjera
		Izlazna vrijednost: 
			false: nema rezultata
			array (u obliku koji vraca funkcija ldap_result_to_array) koji sadrži samo jedan korisnički atribut: uid
			
		Po izvrsenju metode funkcija $obj->result() vraca array:
			'locked'=>true | false
			'authn_left'=>broj preostalih pokusaja autentikacije prije zakljucavanja acc-a,
			'pwd_expires_on'=>vrijeme isteka zaporke u formatu 'd.m.Y. H:i:s',
			'is_exipred'=>je li zaporka istekla 1|0,
			'time_left'=> ako zaporka nije istekla, za koliko vremena istice (formatirano)
			'time_left_s'=> ako zaporka nije istekla, za koliko vremena istice (u sekundama)
		*/
		
		public function ldapUserExists($username,$password,$base,$uid){
			list($username,$base)=$this->clean($username,$base);
			$ret=$this->ldap->ldapUserExists($username,$password,$base,$uid);
			$this->set_error_obj($this->ldap);
			$this->set_result_obj($this->ldap);
			return $ret;
		}
		
		
		/* 
		Funkcija vraca zapise koji zadovoljavaju uvjete pretrage zadane ldap filterom. Za binarne atribute vraca md5 hash njihove vrijednosti.
		Zapisi su uvijek sortirani po dn-u
		Ulazni podaci:
			$uid: uid korisnika u imeniku
			$password: base64 enkodiran password korisnika iz imenika
			$base: base dn korisnika
			$filter: ldap filter
			$attributes: array atributa koji se dohvacaju
		Izlaz:
			array (u obliku koji vraca funkcija ldap_result_to_array) zapisa koji zadovoljavaju uvjete pretrage
			false ili 0 ako nema nijednog zapisa koji zadovoljava uvjete pretrage
		*/
		public function ldapSearch($uid,$password,$base,$filter,$attributes){
			list($uid,$base)=$this->clean($uid,$base);
			$attributes=$this->chk_filter_attrs($filter,$attributes);
			if(!is_array($attributes)){
				return false;
			}
			$ret=$this->ldap->ldapSearch($uid,$password,$base,$filter,$attributes);
			$this->set_error_obj($this->ldap);
			return $ret;
		}
		
		/* 
		
		todo: premjestiti pwdpolicy među atribute
		
		Funkcija vraca zapise koji zadovoljavaju uvjete pretrage zadane ldap filterom. Za binarne atribute vraca md5 hash njihove vrijednosti.
		
		Ulazni podaci:
			$uid: uid korisnika u imeniku
			$password: base64 enkodiran password korisnika iz imenika
			$base: base dn korisnika
			$filter: ldap filter
			$attributes: array atributa koji se dohvacaju
		Izlaz:
			array (u obliku koji vraca funkcija ldap_result_to_array) zapisa koji zadovoljavaju uvjete pretrage. 
				Ako korisnik jos nije sam promijenio zaporku, u njegovom zapisu u rezultatu dodat će se još jedan atribut pwdpolicysubentry ciji je sadrzaj 
				array:['locked=>true|false,'authn_left'=>broj preostalih pokusaja autentikacije prije zakljucavanja acc-a,'pwd_expires_on'=>vrijeme isteka 
				zaporke u formatu 'd.m.Y. H:i:s','is_exipred'=>je li zaporka istekla 1|0,'time_left'=> ako zaporka nije istekla, za koliko vremena istice (formatirano),'time_left_s'=> ako zaporka nije istekla, za koliko vremena istice (u sekundama)]
			false ili 0 ako nema nijednog zapisa koji zadovoljava uvjete pretrage
		*/
		public function userSearch($uid,$password,$base,$filter,$attributes){
			list($uid,$base)=$this->clean($uid,$base);
			$attributes=$this->chk_filter_attrs($filter,$attributes);
			if(!is_array($attributes)){
				return false;
			}
			$ret=$this->ldap->userSearch($uid,$password,$base,$filter,$attributes);
			$this->set_error_obj($this->ldap);
			return $ret;
		}
		
		/*Funkcija vraca zapise koji zadovoljavaju uvjete pretrage zadane ldap filterom. Za binarne atribute vraca md5 hash njihove vrijednosti.
		Binda se na imenik anonimno.
		Ulazni podaci:
			$base: base dn pretrage
			$filter: ldap filter
			$attributes: array atributa koji se dohvacaju
		Izlaz:
			array (u obliku koji vraca funkcija ldap_result_to_array) zapisa koji zadovoljavaju uvjete pretrage
			false ili 0 ako nema nijednog zapisa koji zadovoljava uvjete pretrage
		*/
		public function anonSearch($base,$filter,$attributes){
			$attributes=$this->chk_filter_attrs($filter,$attributes);
			if(!is_array($attributes)){
				return false;
			}
			$ret=$this->ldap->anonSearch($base,$filter,$attributes);
			$this->set_error_obj($this->ldap);
			return $ret;
		}
		
		/*Funkcija služi dohvatu informacija o administratoru imenika
		Ulazne vrijednosti:
		$user= uid korisnika (koji postavlja upit) u imeniku
		$password=base64 encodiran password,
		$base=base dn
		$uid= uid korisnika na kojeg se odnosi upit
		Izlazna vrijednost: 
		false: autentikacija neuspjesna ili korisnik na kojeg se upit odnosi ne postoji u imeniku
		0 - Korisnik: Korisnik postoji, ali nema pravo administracije imenika, 
		>0 - Administrator: 1 - Administator definiran u konfiguracijskoj datoteci, 2 - Administrator definiran u imeniku, 3 - Administrator iz imenika s ograničenjem
		Ako je izlazna vrijednost 3 (administrator s ogranicenjem), onda metoda result ovog objekta vraca hash s ogranicenjima ['naziv_atributa'=>['vrijednost1','vrijednost2',..],['']=>['',''..]
		*/
		public function ldapAdminInfo($user,$password,$base,$uid){
			list($user,$base)=$this->clean($user,$base);
			$ret=$this->ldap->ldapAdminInfo($user,$password,$base,$uid);
			$this->set_error_obj($this->ldap);
			$this->set_result_obj($this->ldap);
			return $ret;
		}
		
		/*Funkcija služi dohvatu informacija o administratoru imenika
		Ulazne vrijednosti:
		$user= uid korisnika (koji postavlja upit) u imeniku
		$password=base64 encodiran password,
		$base=base dn
		Izlazna vrijednost: 
			false: operacija neuspjesna 
			true: operacija uspjesna
		ako je izlaz true, onda metoda result ovog objekta vraca array s podacima administratorima
		obj->result() vraca array: ['dn1'=>['uid'=>vrijednsot atributa uid,'cn'=>vrijednsot atributa cn,'hredupersonrole'=>vrijednsot atributa hredupersonrole,'admin_type'=>1|2|3,'limits'=>['naziv_atributa1'=>['vrijednost1','vrijednost2',..],'naziv_atributa2'=>['vrijednost1','vrijednost2',..],['']=>['',''..]]],'dn2'=>['admin_type'=>1|2|3]]]
		admin_type: 1 - Administator definiran u konfiguracijskoj datoteci, 2 - Administrator definiran u imeniku, 3 - Administrator iz imenika s ograničenjem
		Ako je admin_type 3 (administrator s ogranicenjem), član niza 'limits' sadrzi hash s ogranicenjima ['naziv_atributa'=>['vrijednost1','vrijednost2',..],['']=>['',''..]
		*/
		public function ldapAdminList($user,$password,$base){
			list($user,$base)=$this->clean($user,$base);
			$ret=$this->ldap->ldapAdminList($user,$password,$base);
			$this->set_error_obj($this->ldap);
			$this->set_result_obj($this->ldap);
			return $ret;
		}
		
		/*Funkcija služi anonimnom dohvatu binarnih atributa. Vraca njihovu base64 enkodiranu vrijednost.
		Ulazne vrijednosti:
			$base: base dn
			$filter: ldap filter
			$attributes: array atributa koji se dohvacaju
			$md5: md5 vrijednosti atributa koja se zeli dohvatiti (primarno sluzi izboru zeljene vrijednosti kod atributa s vise vrijednosti) parametar je opcionalan.
		Izlazna vrijednost: 
		false: nema rezultata
		array (u obliku koji vraca funkcija ldap_result_to_array) zapisa koji zadovoljavaju uvjete pretrage gdje su dohvacene vrijednosti base64 enkodirane
			false ili 0 ako nema nijednog zapisa koji zadovoljava uvjete pretrage
		*/
		public function anonBinSearch($base,$filter,$attributes,$md5=false){
			$attributes=$this->chk_filter_attrs($filter,$attributes);
			if(!is_array($attributes)){
				return false;
			}
			$ret=$this->ldap->anonBinSearch($base,$filter,$attributes);
			$this->set_error_obj($this->ldap);
			return $ret;
		}
		
		/*Funkcija služi dohvatu binarnih atributa u ime korisnika. Vraca njihovu base64 enkodiranu vrijednost.
		Ulazne vrijednosti:
			$user= uid korisnika (koji postavlja upit) u imeniku
			$password=base64 encodiran password,
			$base: base dn
			$filter: ldap filter
			$attributes: array atributa koji se dohvacaju
			$md5: md5 vrijednosti atributa koja se zeli dohvatiti (primarno sluzi izboru zeljene vrijednosti kod atributa s vise vrijednosti) parametar je opcionalan.
		Izlazna vrijednost: 
		false: nema rezultata
		array (u obliku koji vraca funkcija ldap_result_to_array) zapisa koji zadovoljavaju uvjete pretrage gdje su dohvacene vrijednosti base64 enkodirane
			false ili 0 ako nema nijednog zapisa koji zadovoljava uvjete pretrage
		*/
		public function userBinSearch($user,$password,$base,$filter,$attributes,$md5=false){
			list($user,$base)=$this->clean($user,$base);
			$attributes=$this->chk_filter_attrs($filter,$attributes);
			if(!is_array($attributes)){
				return false;
			}
			$ret=$this->ldap->userBinSearch($user,$password,$base,$filter,$attributes,$md5);
			$this->set_error_obj($this->ldap);
			return $ret;
		}
		
		/*Funkcija služi dohvatu binarnih atributa u ime administratora. Vraca njihovu base64 enkodiranu vrijednost.
		Ulazne vrijednosti:
			$user= uid korisnika (koji postavlja upit) u imeniku
			$password=base64 encodiran password,
			$base: base dn
			$filter: ldap filter
			$attributes: array atributa koji se dohvacaju
			$md5: md5 vrijednosti atributa koja se zeli dohvatiti (primarno sluzi izboru zeljene vrijednosti kod atributa s vise vrijednosti) parametar je opcionalan.
		Izlazna vrijednost: 
		false: nema rezultata
		array (u obliku koji vraca funkcija ldap_result_to_array) zapisa koji zadovoljavaju uvjete pretrage gdje su dohvacene vrijednosti base64 enkodirane
			false ili 0 ako nema nijednog zapisa koji zadovoljava uvjete pretrage
		*/
		public function ldapBinSearch($user,$password,$base,$filter,$attributes,$md5=false){
			list($user,$base)=$this->clean($user,$base);
			$attributes=$this->chk_filter_attrs($filter,$attributes);
			if(!is_array($attributes)){
				return false;
			}
			$ret=$this->ldap->ldapBinSearch($user,$password,$base,$filter,$attributes,$md5);
			$this->set_error_obj($this->ldap);
			return $ret;
		}
		
		/*Funkcija služi dohvatu org zapisa u ime administratora.
		Ulazne vrijednosti:
			$user: uid korisnika u imeniku
			$password: base64 enkodiran password korisnika iz imenika
			$base: base dn imenika
		Izlazna vrijednost: 
		false: nema rezultata
		array (u obliku koji vraca funkcija ldap_result_to_array) org zapisa ustanove
		*/
		
		public function ldapOrgInfo($user,$password,$base){
			list($user,$base)=$this->clean($user,$base);
			$ret=$this->ldap->ldapOrgInfo($user,$password,$base);
			$this->set_error_obj($this->ldap);
			return $ret;
		}
		
		/*Funkcija služi dohvatu org zapisa u ime korisnika.
		Ulazne vrijednosti:
			$user: uid korisnika u imeniku
			$password: base64 enkodiran password korisnika iz imenika
			$base: base dn imenika
		Izlazna vrijednost: 
		false: nema rezultata
		array (u obliku koji vraca funkcija ldap_result_to_array) org zapisa ustanove
		*/
		
		public function userOrgInfo($user,$password,$base){
			list($user,$base)=$this->clean($user,$base);
			$ret=$this->ldap->userOrgInfo($user,$password,$base);
			$this->set_error_obj($this->ldap);
			return $ret;
		}
		
		/*Funkcija služi dodavanju korisnika u LDAP imenik. Moze dodati jednog ili vise korisnika ovisno o ulaznim podacima. Ova funkcija ne zahtjeva sve obavezne atribute, nego samo njihov dio.
		Ulazne vrijednosti:
			$user: uid administratora u imeniku
			$password: base64 enkodiran password administratora iz imenika
			$base: base dn imenika
			$data: array vrijednosti s podacima o korisniku / korisnicima koji se dodaju. [ [dn]=>[[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]]...] 
				array mora sadrzavati atribute 'uid','userpassword','hredupersonoib','givenname','sn','mail','hredupersonprimaryaffiliation','hredupersonexpiredate' iz kojih se izvode vrijednosti ostalih obavezniha atributa.
				Ako je hredupersonprimaryaffiliation=gost onda hredupersonoib smije biti none, ali onda je obavezan atribut hredupersonuniquenumber s nekom od ostalih vrijednosti
		Izlazna vrijednost: 
			true: svi korisnici ispravno dodani
			false: bar jedna korisnik nije dodan
			poslije izvrsenja funkcija result() ovog objekta vraca array oblika [ [dn1]=>[[status]=>true|false (je li uspjelo dodavanje),[code]=>kod greske ako postoji,[message]=>poruka o gresci ako postoji],[dn2]=>[].... ]
		*/
		
		public function ldapAddUserLE($user,$password,$base,$data){
			list($user,$base)=$this->clean($user,$base);
			$ret=$this->ldap->ldapAddUserLE($user,$password,$base,$data);
			$this->set_error_obj($this->ldap);
			$this->set_result_obj($this->ldap);
			return $ret;	
		}
		
		/*Funkcija služi dodavanju korisnika u LDAP imenik. Moze dodati jednog ili vise korisnika ovisno o ulaznim podacima. Ulazni podaci moraju sadrzavati sve (obavezne)  atribute.
		Ulazne vrijednosti:
			$user: uid administratora u imeniku
			$password: base64 enkodiran password administratora iz imenika
			$base: base dn imenika
			$data: array vrijednosti s podacima o korisniku / korisnicima koji se dodaju. [ [dn]=>[[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]]...] array mora sadrzavati sve obavezne atribute.
		Izlazna vrijednost: 
			true: svi korisnici ispravno dodani
			false: bar jedna korisnik nije dodan
			poslije izvrsenja funkcija result() ovog objekta vraca array oblika [ [dn1]=>[[status]=>true|false (je li uspjelo dodavanje),[code]=>kod greske ako postoji,[message]=>poruka o gresci ako postoji],[dn2]=>[].... ]
		*/
		
		public function ldapAddUser($user,$password,$base,$data){
			list($user,$base)=$this->clean($user,$base);
			$ret=$this->ldap->ldapAddUser($user,$password,$base,$data);
			$this->set_error_obj($this->ldap);
			$this->set_result_obj($this->ldap);
			return $ret;	
		}
		
		/*Funkcija služi dodavanju atributa u korisnicki zapis u LDAP imeniku u ime administratora. Moze dodati jedan ili vise atributa za jednog ili vise korisnika.
		Ulazne vrijednosti:
			$user: uid administratora imenika
			$password: base64 enkodiran password administratora imenika
			$base: base dn imenika
			$data: array vrijednosti atributa se dodaju. [ [dn]=>[[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]]...] 
		Izlazna vrijednost: 
			true: svi atributi dodani svim korisnicima
			false: bar jedna greska kod dodavanja
			poslije izvrsenja funkcija result() ovog objekta vraca array oblika [ [dn1]=>[[status]=>true|false (je li uspjelo dodavanje),[code]=>kod greske ako postoji,[message]=>poruka o gresci ako postoji],[dn2]=>[].... ]
		*/
		public function ldapAddAttribute($user,$password,$base,$data){
			list($user,$base)=$this->clean($user,$base);
			$ret=$this->ldap->ldapAddAttribute($user,$password,$base,$data);
			$this->set_error_obj($this->ldap);
			$this->set_result_obj($this->ldap);
			return $ret;
		}
		
		/*Funkcija služi dodavanju atributa u korisnicki zapis u LDAP imeniku u ime korisnika.
		Ulazne vrijednosti:
			$user: uid korisnika u imeniku
			$password: base64 enkodiran password korisnika
			$base: base dn imenika
			$data: array vrijednosti atributa koji se dodaju. [ ['uid=$uid,$base']=>[[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]]...] 
		Izlazna vrijednost: 
			true: atribut(i) dodani
			false: greska kod dodavanja
			poslije izvrsenja funkcija result() ovog objekta vraca array oblika [ ['uid=$uid,$base']=>[[status]=>true|false (je li uspjelo dodavanje),[code]=>kod greske ako postoji,[message]=>poruka o gresci ako postoji],[dn2]=>[].... ]
		*/
		public function userAddAttribute($user,$password,$base,$data){
			list($user,$base)=$this->clean($user,$base);
			$ret=$this->ldap->userAddAttribute($user,$password,$base,$data);
			$this->set_error_obj($this->ldap);
			$this->set_result_obj($this->ldap);
			return $ret;
		}
		
		/*Funkcija služi brisanju korisnika iz LDAP imenika u ime administratora. Moze dodati izbrisati više korisnika
		Ulazne vrijednosti:
			$user: uid administratora u imeniku
			$password: base64 enkodiran password administratora
			$base: base dn imenika
			$data: array dn-ova korisnika koji se brisu.
		Izlazna vrijednost: 
			true: svi korisnici izbrisani
			false: bar greska u brisanju
			poslije izvrsenja funkcija result() ovog objekta vraca array oblika [ [dn1]=>[[status]=>true|false (je li uspjelo brisanje),[code]=>kod greske ako postoji,[message]=>poruka o gresci ako postoji],[dn2]=>[].... ]
		*/
		
		public function ldapDeleteUser($user,$password,$base,$data){
			list($user,$base)=$this->clean($user,$base);
			$ret=$this->ldap->ldapDeleteUser($user,$password,$base,$data);
			$this->set_error_obj($this->ldap);
			$this->set_result_obj($this->ldap);
			return $ret;
		}
		
		/*Funkcija služi brisanju neke vrijednosti nekog atributa korisnika iz LDAP imenika u ime administratora. Moze dodati izbrisati vise vrijednosti vise atributa za vise korisnika
		Ulazne vrijednosti:
			$user: uid administratora u imeniku
			$password: base64 enkodiran password administratora
			$base: base dn imenika
			$data: array vrijednosti atributa se brisu. [ [dn]=>[[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]],[dn1]=>[[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]]...] 
		Izlazna vrijednost: 
			true: svi atributi izbrisani
			false: bar jedna greska u brisanju
			poslije izvrsenja funkcija result() ovog objekta vraca array oblika [ [dn1]=>[[status]=>true|false (je li uspjelo brisanje),[code]=>kod greske ako postoji,[message]=>poruka o gresci ako postoji],[dn2]=>[].... ]
		*/
		
		public function ldapDeleteAttribute($user,$password,$base,$data){
			list($user,$base)=$this->clean($user,$base);
			$ret=$this->ldap->ldapDeleteAttribute($user,$password,$base,$data);
			$this->set_error_obj($this->ldap);
			$this->set_result_obj($this->ldap);
			return $ret;
		}
		
		/*Funkcija služi brisanju neke vrijednosti nekog atributa korisnika iz LDAP imenika u ime korisnika. Moze dodati izbrisati vise vrijednosti vise atributa za danog korisnika.
		Ulazne vrijednosti:
			$user: uid korisnika u imeniku
			$password: base64 enkodiran password korisnika
			$base: base dn imenika
			$data: array vrijednosti atributa se brisu. [ ['uid=$uid,$base']=>[[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]]...] 
		Izlazna vrijednost: 
			true: svi atributi izbrisani
			false: bar jedna greska u brisanju
			poslije izvrsenja funkcija result() ovog objekta vraca array oblika [ ['uid=$uid,$base']=>[[status]=>true|false (je li uspjelo brisanje),[code]=>kod greske ako postoji,[message]=>poruka o gresci ako postoji]]
		*/
		
		public function userDeleteAttribute($user,$password,$base,$data){
			list($user,$base)=$this->clean($user,$base);
			$ret=$this->ldap->userDeleteAttribute($user,$password,$base,$data);
			$this->set_error_obj($this->ldap);
			$this->set_result_obj($this->ldap);
			return $ret;
		}
		
		/*Funkcija služi promjeni vrijednosti atributa korisnika iz LDAP imenika u ime administratora. Moze promijeniti vrijednost vise atributa za vise korisnika
		Promjena se vrši metodom replace sto znaci da se za neki atribut izbrisu sve stare vrijednosti. Ako atribut ima vise vrijednosti potrebno je u arrayu poslati SVE nove vrijednosti
		Ulazne vrijednosti:
			$user: uid administratora u imeniku
			$password: base64 enkodiran password administratora
			$base: base dn imenika
			$data: array vrijednosti atributa se brisu. [ [dn]=>[[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]],[dn1]=>[[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]]...] 
		Izlazna vrijednost: 
			true: svi atributi promijenjeni
			false: bar jedna greska u promjeni
			poslije izvrsenja funkcija result() ovog objekta vraca array oblika [ [dn1]=>[[status]=>true|false (je li uspjelo brisanje),[code]=>kod greske ako postoji,[message]=>poruka o gresci ako postoji],[dn2]=>[].... ]
		*/
		
		public function ldapModifyAttribute($user,$password,$base,$data){
			list($user,$base)=$this->clean($user,$base);
			$ret=$this->ldap->ldapModifyAttribute($user,$password,$base,$data);
			$this->set_error_obj($this->ldap);
			$this->set_result_obj($this->ldap);
			return $ret;
		}
		
		/*Funkcija služi promjeni vrijednosti atributa korisnika. Moze promijeniti vrijednost vise atributa
		Promjena se vrši metodom replace sto znaci da se za neki atribut izbrisu sve stare vrijednosti. Ako atribut ima vise vrijednosti potrebno je u arrayu poslati SVE nove vrijednosti
		Ulazne vrijednosti:
			$user: uid korisnika u imeniku
			$password: base64 enkodiran password korisnika
			$base: base dn imenika
			$data: array vrijednosti atributa se mijenjaju. [ ['uid=$uid,$base']=>[[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]]...] 
		Izlazna vrijednost: 
			true: svi atributi promijenjeni
			false: bar jedna greska u promjeni
			poslije izvrsenja funkcija result() ovog objekta vraca array oblika [ ['uid=$uid,$base']=>[[status]=>true|false (je li uspjela promjena),[code]=>kod greske ako postoji,[message]=>poruka o gresci ako postoji]]
		*/
		
		public function userModifyAttribute($user,$password,$base,$data){
			list($user,$base)=$this->clean($user,$base);
			$ret=$this->ldap->userModifyAttribute($user,$password,$base,$data);
			$this->set_error_obj($this->ldap);
			$this->set_result_obj($this->ldap);
			return $ret;
		}
		
		/*Funkcija služi promjeni vrijednosti atributa iz org zapisa LDAP imenika u ime administratora. Moze promijeniti vrijednost vise atributa.
		Promjena se vrši metodom replace sto znaci da se za neki atribut izbrisu sve stare vrijednosti. Ako atribut ima vise vrijednosti potrebno je u arrayu poslati SVE nove vrijednosti
		Promjena vrijednosti atributa 'o','l','street','postalcode','postaladdress' propagira se u korisničke zapise svih korisnika u imeniku
		Atribut postaladdress se tvori iz vrijednosti ostalih atributa kao: o, street, postalcode, l
		Ulazne vrijednosti:
			$user: uid administratora u imeniku
			$password: base64 enkodiran password administratora
			$base: base dn imenika
			$data: array vrijednosti atributa koji se mijenjaju. [[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]] 
		Izlazna vrijednost: 
			true: atributi promijenjeni
			false: greska u promjeni
		*/
		
		public function ldapModifyOrgAttribute($user,$password,$base,$data){
			list($user,$base)=$this->clean($user,$base);
			$ret=$this->ldap->ldapModifyOrgAttribute($user,$password,$base,$data);
			$this->set_error_obj($this->ldap);
			$this->set_result([$base=>['status'=>$res,'code'=>$this->err_code(),'message'=>$this->err_message()]]);
			return $ret;
		}
		
		/*Funkcija služi brisanju vrijednosti atributa iz org zapisa LDAP imenika u ime administratora. Moze izbrisati vrijednost vise atributa.		
		Ulazne vrijednosti:
			$uid: user administratora u imeniku
			$password: base64 enkodiran password administratora
			$base: base dn imenika
			$data: array vrijednosti atributa koje se brisu. [[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]] 
		Izlazna vrijednost: 
			true: atributi izbrisani
			false: greska kod brisanja
		*/
		public function ldapDeleteOrgAttribute($user,$password,$base,$data){
			list($user,$base)=$this->clean($user,$base);
			$ret=$this->ldap->ldapDeleteOrgAttribute($user,$password,$base,$data);
			$this->set_error_obj($this->ldap);
			$this->set_result([$base=>['status'=>$res,'code'=>$this->err_code(),'message'=>$this->err_message()]]);
			return $ret;
		}
		
		/*Funkcija služi dodavanje vrijednosti atributa iz org zapisa LDAP imenika u ime administratora. Moze dodati vrijednost vise atributa.		
		Ulazne vrijednosti:
			$uid: user administratora u imeniku
			$password: base64 enkodiran password administratora
			$base: base dn imenika
			$data: array vrijednosti atributa koje se dodaju. [[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]] 
		Izlazna vrijednost: 
			true: atributi dodani
			false: greska kod dodavanja
		*/
		public function ldapAddOrgAttribute($user,$password,$base,$data){
			list($user,$base)=$this->clean($user,$base);
			$ret=$this->ldap->ldapAddOrgAttribute($user,$password,$base,$data);
			$this->set_error_obj($this->ldap);
			$this->set_result([$base=>['status'=>$res,'code'=>$this->err_code(),'message'=>$this->err_message()]]);
			return $ret;
		}
		
		/*Funkcija vraca listu korisnika kojima je zbog password policy-a e-identitet zakljucan ili ce uskoro biti.
		Ulazne vrijednosti:
			$user: uid administratora u imeniku
			$password: base64 enkodiran password administratora iz imenika
			$base: base dn administratora u imeniku
		Izlazna vrijednost: 
			false: greska
			true: uspjeh
			
		Po izvrsenju metode funkcija $obj->result() vraca array:
		['dn1'=>
			['uid'=>uid,
			'locked'=>true | false
			'authn_left'=>broj preostalih pokusaja autentikacije prije zakljucavanja acc-a,
			'pwd_expires_on'=>vrijeme isteka zaporke u formatu 'd.m.Y. H:i:s',
			'is_exipred'=>je li zaporka istekla 1|0,
			'time_left'=> ako zaporka nije istekla, za koliko vremena istice (formatirano)
			'time_left_s'=> ako zaporka nije istekla, za koliko vremena istice (u sekundama)],
		'dn2'=>[...], ....]
		*/
		
		public function ldapLockedList($user,$password,$base){
			list($user,$base)=$this->clean($user,$base);
			$ret=$this->ldap->ldapLockedList($user,$password,$base);
			$this->set_error_obj($this->ldap);
			$this->set_result_obj($this->ldap);
			return $ret;
		}
		
		/*Funkcija rezultat provjere uskladjenosti podataka s normama certificiranja
		Ulazne vrijednosti:
			$username: dio prije base koji zajedno s base tvori dn korisnika u imeniku. npr cn=hreduadmin
			$password: base64 enkodiran password korisnika iz imenika
			$base: base dn korisnika u imeniku
		Izlazna vrijednost: 
			false: greska
			true: uspjeh
			
		Po izvrsenju metode funkcija $obj->result() vraca array:
		
		*/
		
		public function certify($username,$password,$base){
			list($username,$base)=$this->clean($username,$base);
			$ret=$this->ldap->certify("$username,$base",$password,$base);
			$this->set_error_obj($this->ldap);
			$this->set_result_obj($this->ldap);
			return $ret;
		}
		
		private function clean($uid,$base){
			return [str_replace(' ','',strtolower($uid)),str_replace(' ','',strtolower($base))];
		}
		/* provjerava postoji li ldap filter i array atributa. AKo ne postoji postavlja odgovarajucu poruko o gresci i pise log
		Ulazne vrijednosti:
			$filter: ldap filter
			$attributes: array atributa koji se dohvacaju
		Izlazna vrijednost: 
			true | false
		
		*/
		private function chk_filter_attrs($filter,$attributes){
			if(empty($filter)){
				\aosi\Logger::write('LDAP filter missing.',6);
				$this->set_error(-65,'Nedostaje LDAP filter.');
				return false;
			}
			if(!is_array($attributes)){
				\aosi\Logger::write('Attributes list missing.',6);
				$this->set_error(-66,'Nedostaje popis atributa.');
				return false;
			}
			return array_map('strtolower', $attributes);
		}
	}



}

?>