<?php
namespace aosi{
	/*
	U slučaju greske u izvrsavanju bilo koje metode, po izvrsavanju metode kod greske se moze dobiti metodom $obj->err_code(), a poruka o gresci metodom $obj->err-message()
	*/
	class Ldap extends Simple{
		
		private $conf;
		//atributi koji se korisniku dodaju iz org zapisa ustanove
		private $org_attrs=array('o','l','street','postalcode','postaladdress');
		
		function __construct() {
			$this->conf=Config::get('ldap');
		}
		
		/*Funkcija primarno služi autentikaciji korisnika. 
		Uz rezultat binda (autentikaciju) provjerava status korisnika u smislu politike obavezne promjene zaporke
		Ako korisnik jos nije sam promijenio zaporku, nakon uspjesne autentikacije metoda $obj->result() vraca array:
			'authn_left'=>broj preostalih pokusaja autentikacije prije zakljucavanja acc-a,
			'pwd_expires_on'=>vrijeme isteka zaporke u formatu 'd.m.Y. H:i:s',
			'is_exipred'=>je li zaporka istekla 1|0,
			'time_left'=> ako zaporka nije istekla, za koliko vremena istice (formatirano)
			'time_left_s'=> ako zaporka nije istekla, za koliko vremena istice (u sekundama)
		Ulazne vrijednosti:
			$uid= uid korisnika u imeniku
			$password=base64 encodiran password,
			$base=base dn
		Izlazna vrijednost: 
			false: autentikacija neuspjesna
			true: autentikacija uspjesna
		*/
		public function userBind($uid,$password,$base){
			if($l=$this->bind($uid,$password,$base)){				
				$this->isLockedPpolicy("uid=$uid,$base");
				return $l;
			} else {
				//$this->isLocked("uid=$uid,$base");
				return false;
			}
		}
		
		/*Funkcija primarno služi autentikaciji / autorizaciji administratora imenika. 
		Uz rezultat binda (autentikaciju) provjerava ima li osoba administratorska prava.
		Ulazne vrijednosti:
		$uid: uid korisnika u imeniku
		$password: base64 encodiran password,
		$base: base dn
		Izlazna vrijednost: 
		false: autentikacija neuspjesna
		0 - Korisnik: autentikacija uspjesna, ali korisnik nema pravo administracije imenika, 
		>0 - Administrator: 1 - Administator definiran u konfiguracijskoj datoteci, 2 - Administrator definiran u imeniku, 3 - Administrator iz imenika s ograničenjem
		Ako je izlazna vrijednost 3 (administrator s ogranicenjem), onda metoda result ovog objekta vraca hash s ogranicenjima ['naziv_atributa'=>['vrijednost1','vrijednost2',..],['']=>['',''..]
		*/
		public function ldapBind($uid,$password,$base){
			if($l=$this->bind($uid,$password,$base)){
				if($res=$this->search($l, $base, "(uid=$uid)", ['hredupersonrole'])){
					$admintype=$this->isAdmin("uid=$uid,$base",$res["uid=$uid,$base"]);
					if($admintype>0){
						return $admintype;
					} else {
						$this->set_error(-15,"User uid=$uid,$base does not have admin privileges!");
						return $admintype;
					}
				} else {
					//nemoguca situacija
					\aosi\Logger::write('Record not found for authenticated user '."uid=$uid,$base",3);
					$this->set_error(-64,'Unexpected error.');
					return false;
				}
			} else {
				return false;
			}
		}
		/* 
		Funkcija vraca zapise koji zadovoljavaju uvjete pretrage zadane ldap filterom i omogucuje paginaciju. Za binarne atribute vraca md5 hash njihove vrijednosti.
		Zapisi su uvijek sortirani po dn-u
		Po izvrsenju funkcije metoda $obj->result(); vraca array u kojem je ['count'] ukupan broj zapisa koji zadovoljavaju uvjete pretrage
		Ulazni podaci:
			$uid: uid korisnika u imeniku
			$password: base64 enkodiran password korisnika iz imenika
			$base: base dn korisnika
			$filter: ldap filter
			$attributes: array atributa koji se dohvacaju
			$from: redni broj zapisa od kojeg se treba popis prikazati
			$size: broj zapisa koji se prikazuju
		Izlaz:
			array (u obliku koji vraca funkcija ldap_result_to_array) zapisa koji zadovoljavaju uvjete pretrage
			false ili 0 ako nema nijednog zapisa koji zadovoljava uvjete pretrage
		*/
		public function ldapList($uid,$password,$base,$filter,$attributes,$from,$size){
			if($admintype=$this->ldapBind($uid,$password,$base)){
				if($size>0){
					if(!$this->is_allowed_person('admin','view',$attributes)){
						return false;
					}
					$limits=[];
					if($admintype==3){
						$limits=$this->result();
					}
					$this->set_result(false);
					$limits['objectclass']=['hreduperson'];
					$filter=$this->filter_from_limits($filter,$limits);
					$l=$this->rootBind();
					$all=$this->search($l, $base, $filter, ['dn']);
					if(is_array($all)){
						$count=count($all);
						$this->set_result(['count'=>$count]);
						if($count>=$from && $from>0){
							$all=array_keys($all);
							sort($all,SORT_STRING);
							$ret=[];$i=0;$from--;
							while($i<$size && !empty($all[$i+$from])){
								$ret[]=$this->read($l, $all[$i+$from],'(objectclass=*)', $attributes);
								$i++;
							}
							return $ret;
						}
					}
				}
			}
			return false;
		}
		
			/* 
		Funkcija vraca zapise koji zadovoljavaju uvjete pretrage zadane ldap filterom. Za binarne atribute vraca md5 hash njihove vrijednosti.
		Zapisi su uvijek sortirani po dn-u
		Ulazni podaci:
			$uid: uid korisnika u imeniku
			$password: base64 enkodiran password korisnika iz imenika
			$base: base dn korisnika
			$filter: ldap filter
			$attributes: array atributa koji se dohvacaju
			$show_binary: true  | false (default) - ako je true za binarne atribute vraca njihovu base_64 encodiranu vrijednost, ako je false vraca md5 hash vrijednosti (uvazava se samo ako je ako je $conv_to_array: true)
			$md5: md5 vrijednosti atributa koja se zeli dohvatiti. Koristi se iskljucivo s binarnim atributima i primarno sluzi izboru zeljene vrijednosti kod atributa s vise vrijednosti. Parametar je opcionalan.
		Izlaz:
			array (u obliku koji vraca funkcija ldap_result_to_array) zapisa koji zadovoljavaju uvjete pretrage
			false ili 0 ako nema nijednog zapisa koji zadovoljava uvjete pretrage
		*/
		public function ldapSearch($uid,$password,$base,$filter,$attributes,$show_binary=false,$md5=false){
			if($admintype=$this->ldapBind($uid,$password,$base)){
				if(!$this->is_allowed_person('admin','view',$attributes)){
					return false;
				}
				$limits=[];
				if($admintype==3){
					$limits=$this->result();
				}
				$this->set_result(false);
				$filter=$this->filter_from_limits($filter,$limits);
				$l=$this->rootBind();
				return $this->search($l, $base, $filter, $attributes,true,$show_binary,$md5);
			}
			return false;
		}
		
			/* 
		Funkcija vraca zapise koji zadovoljavaju uvjete pretrage zadane ldap filterom. Za binarne atribute vraca md5 hash njihove vrijednosti.
		Ulazni podaci:
			$uid: uid korisnika u imeniku
			$password: base64 enkodiran password korisnika iz imenika
			$base: base dn korisnika
			$filter: ldap filter
			$attributes: array atributa koji se dohvacaju
			$show_binary: true  | false (default) - ako je true za binarne atribute vraca njihovu base_64 encodiranu vrijednost, ako je false vraca md5 hash vrijednosti (uvazava se samo ako je ako je $conv_to_array: true)
			$md5: md5 vrijednosti atributa koja se zeli dohvatiti. Koristi se iskljucivo s binarnim atributima i primarno sluzi izboru zeljene vrijednosti kod atributa s vise vrijednosti. Parametar je opcionalan.
		Izlaz:
			Izlaz:
			array (u obliku koji vraca funkcija ldap_result_to_array) zapisa koji zadovoljavaju uvjete pretrage. 
				Ako korisnik jos nije sam promijenio zaporku, u njegovom zapisu u rezultatu dodat će se još jedan atribut pwdpolicysubentry ciji je sadrzaj 
				array:['locked=>true|false,'authn_left'=>broj preostalih pokusaja autentikacije prije zakljucavanja acc-a,'pwd_expires_on'=>vrijeme isteka 
				zaporke u formatu 'd.m.Y. H:i:s','is_exipred'=>je li zaporka istekla 1|0,'time_left'=> ako zaporka nije istekla, za koliko vremena istice (formatirano),'time_left_s'=> ako zaporka nije istekla, za koliko vremena istice (u sekundama)]
			false ili 0 ako nema nijednog zapisa koji zadovoljava uvjete pretrage			
		*/
		public function userSearch($uid,$password,$base,$filter,$attributes,$show_binary=false,$md5=false){
			if($l=$this->userBind($uid,$password,$base)){
				if(!$this->is_allowed_person('user','view',$attributes)){
					return false;
				}
				$this->set_result(false);
				$ret=$this->search($l, $base, $filter, $attributes,true,$show_binary,$md5);
				if(is_array($ret)){
					foreach($ret as $dn=>$u){
						if($dn=="uid=$uid,$base"){
							$locked=$this->Islockedppolicy($dn);
							if($ppolicy=$this->result(true)){
								$ppolicy['locked']=$locked;
								$ret[$dn]['pwdpolicysubentry']=$ppolicy;
							}
						}
					}
					return $ret;
				}
			}
			return false;
		}
		
		/*Funkcija vraca zapise koji zadovoljavaju uvjete pretrage zadane ldap filterom. Za binarne atribute vraca md5 hash njihove vrijednosti.
		Binda se na imenik anonimno.
		Ulazni podaci:
			$filter: ldap filter
			$attributes: array atributa koji se dohvacaju
			$show_binary: true  | false (default) - ako je true za binarne atribute vraca njihovu base_64 encodiranu vrijednost, ako je false vraca md5 hash vrijednosti (uvazava se samo ako je ako je $conv_to_array: true)
			$md5: md5 vrijednosti atributa koja se zeli dohvatiti. Koristi se iskljucivo s binarnim atributima i primarno sluzi izboru zeljene vrijednosti kod atributa s vise vrijednosti. Parametar je opcionalan.
		Izlaz:
			array (u obliku koji vraca funkcija ldap_result_to_array) zapisa koji zadovoljavaju uvjete pretrage
			false ili 0 ako nema nijednog zapisa koji zadovoljava uvjete pretrage
		*/
		public function anonSearch($base,$filter,$attributes,$show_binary=false,$md5=false){
			if($l=$this->anonBind()){
				if(!$this->is_allowed_person('guest','view',$attributes)){
					return false;
				}
				return $this->search($l, $base, $this->filter_from_limits($filter), $attributes,true,$show_binary,$md5);
			}
			return false;
		}
		
		/*Funkcija služi dohvatu informacija o administratorima imenika. 
		Ulazne vrijednosti:
		$user= uid korisnika (koji postavlja upit) u imeniku
		$password=base64 encodiran password,
		$base=base dn
		Izlazna vrijednost: 
			false: operacija neuspjesna 
			true: operacija uspjesna
		ako je izlaz true, onda metoda result ovog objekta vraca array s podacima administratorima
		obj->result() vraca array: ['dn1'=>['uid'=>vrijednsot atributa uid,'cn'=>vrijednsot atributa cn,'hredupersonrole'=>vrijednsot atributa hredupersonrole,'admin_type'=>1|2|3,'limits'=>['naziv_atributa1'=>['vrijednost1','vrijednost2',..],'naziv_atributa2'=>['vrijednost1','vrijednost2',..],['']=>['',''..]]],'dn2'=>['admin_type'=>1|2|3]]]
		admin_type: 1 - Administator definiran u konfiguracijskoj datoteci, 2 - Administrator definiran u imeniku, 3 - Administrator iz imenika s ograničenjem
		Ako je admin_type 3 (administrator s ogranicenjem), član niza 'limits' sadrzi hash s ogranicenjima ['naziv_atributa'=>['vrijednost1','vrijednost2',..],['']=>['',''..]
		*/
		public function ldapAdminList($user,$password,$base){
			if($admintype=$this->ldapBind($user,$password,$base)){
				\aosi\Logger::write("AdminList called by uid=$user,$base. Admintype $admintype." ,8);
				$limits=[];
				if($admintype==3){
					$limits=$this->result();
					\aosi\Logger::write("Admin limits:" . array2txt($limits) ,8);
				}
				$this->set_result(null);
				$filter=$this->filter_from_limits("(hredupersonrole=*administrator imenika*)",$limits);
				$l=$this->rootBind();
				$ret=[];
				if($data=$this->search($l, $base, $filter, ['hredupersonrole','uid','cn'])){
					foreach($data as $dn=>$d){
						$admin_type=$this->isAdmin($dn,$d);
						$res=$this->result(true);
						switch($admin_type){
							case 2:
								$ret[$dn]=['uid'=>$d['uid'],'cn'=>$d['cn'],'hredupersonrole'=>$d['hredupersonrole'],'admin_type'=>$admin_type];
								break;
							case 3:
								$ret[$dn]=['uid'=>$d['uid'],'cn'=>$d['cn'],'hredupersonrole'=>$d['hredupersonrole'],'admin_type'=>$admin_type,'limits'=>$res];
								break;
						}
					}
				}
				if($admintype!=3){
					if(is_array($this->conf['admins'])){
						\aosi\Logger::write("Admins from config file: " . implode(';',$this->conf['admins']) ,8);
						foreach($this->conf['admins'] as $dn){
							if($data=$this->read($l, $dn, '(objectclass=*)', ['uid','cn'])){
								$ret[$dn]=['uid'=>$data[$dn]['uid'],'cn'=>$data[$dn]['cn'],'admin_type'=>1];
							}
						}				
					}
				}
				$this->set_result($ret);
				return true;
			}
			return false;
		}
		
		/*Funkcija služi dohvatu informacija o administratoru imenika
		Ulazne vrijednosti:
			$user= uid korisnika (koji postavlja upit) u imeniku
			$password=base64 encodiran password,
			$base=base dn
			$uid= uid korisnika na kojeg se odnosi upit
		Izlazna vrijednost: 
		false: autentikacija neuspjesna ili korisnik na kojeg se upit odnosi ne postoji u imeniku
		0 - Korisnik: Korisnik postoji, ali nema pravo administracije imenika, 
		>0 - Administrator: 1 - Administator definiran u konfiguracijskoj datoteci, 2 - Administrator definiran u imeniku, 3 - Administrator iz imenika s ograničenjem
		Ako je izlazna vrijednost 3 (administrator s ogranicenjem), onda metoda result ovog objekta vraca hash s ogranicenjima ['naziv_atributa'=>['vrijednost1','vrijednost2',..],['']=>['',''..]
		*/
		public function ldapAdminInfo($user,$password,$base,$uid){
			if($admintype=$this->ldapBind($user,$password,$base)){
				$limits=[];
				if($admintype==3){
					$limits=$this->result();
				}
				$this->set_result(null);
				$admintype=null;
				$filter=$this->filter_from_limits("",$limits);
				$l=$this->rootBind();
				if($data=$this->read($l, "uid=$uid,$base", $filter, ['hredupersonrole'])){
					return $this->isAdmin("uid=$uid,$base",$data["uid=$uid,$base"]);
				} else {
					\aosi\Logger::write("User $uid is not in directory for base $base!" ,6);
					$this->set_error(-16,"User $uid is not in directory for base $base!");
					return false;
				}
			}
			return false;
		}
		
		/*Funkcija služi anonimnom dohvatu binarnih atributa. Vraca njihovu base64 enkodiranu vrijednost.
		Ulazne vrijednosti:
			$base: base dn
			$filter: ldap filter
			$attributes: array atributa koji se dohvacaju
			$md5: md5 vrijednosti atributa koja se zeli dohvatiti (primarno sluzi izboru zeljene vrijednosti kod atributa s vise vrijednosti) parametar je opcionalan.
		Izlazna vrijednost: 
		false: nema rezultata
		array (u obliku koji vraca funkcija ldap_result_to_array) zapisa koji zadovoljavaju uvjete pretrage gdje su dohvacene vrijednosti base64 enkodirane
			false ili 0 ako nema nijednog zapisa koji zadovoljava uvjete pretrage
		*/
		public function anonBinSearch($base,$filter,$attributes,$md5=false){
			if(!$this->is_binary_arr($attributes)){
				\aosi\Logger::write("At least one of requested attributes is not binary (" . implode(', ' ,$attributes) .")!" ,4);
				$this->set_error(-33,"Bar jedan od traženih atributa nije binarni (" . implode(', ' ,$attributes) .")!");
				return false;
			}
			return $this->anonSearch($base,$filter,$attributes,true,$md5);
		}
		
		/*Funkcija služi dohvatu binarnih atributa u ime korisnika. Vraca njihovu base64 enkodiranu vrijednost.
		Ulazne vrijednosti:
			$uid: uid korisnika u imeniku
			$password: base64 enkodiran password korisnika iz imenika
			$base: base dn korisnika
			$filter: ldap filter
			$attributes: array atributa koji se dohvacaju
			$md5: md5 vrijednosti atributa koja se zeli dohvatiti (primarno sluzi izboru zeljene vrijednosti kod atributa s vise vrijednosti) parametar je opcionalan.
		Izlazna vrijednost: 
		false: nema rezultata
		array (u obliku koji vraca funkcija ldap_result_to_array) zapisa koji zadovoljavaju uvjete pretrage gdje su dohvacene vrijednosti base64 enkodirane
			false ili 0 ako nema nijednog zapisa koji zadovoljava uvjete pretrage
		*/
		public function userBinSearch($uid,$password,$base,$filter,$attributes,$md5=false){
			if(!$this->is_binary_arr($attributes)){
				\aosi\Logger::write("At least one of requested attributes is not binary (" . implode(', ' ,$attributes) .")!" ,4);
				$this->set_error(-33,"Bar jedan od traženih atributa nije binarni (" . implode(', ' ,$attributes) .")!");
				return false;
			}
			return $this->userSearch($uid,$password,$base,$filter,$attributes,true,$md5);
		}
		
		/*Funkcija služi dohvatu binarnih atributa u ime administratora. Vraca njihovu base64 enkodiranu vrijednost.
		Ulazne vrijednosti:
			$uid: uid korisnika u imeniku
			$password: base64 enkodiran password korisnika iz imenika
			$base: base dn korisnika
			$filter: ldap filter
			$attributes: array atributa koji se dohvacaju
			$md5: md5 vrijednosti atributa koja se zeli dohvatiti (primarno sluzi izboru zeljene vrijednosti kod atributa s vise vrijednosti) parametar je opcionalan.
		Izlazna vrijednost: 
		false: nema rezultata
		array (u obliku koji vraca funkcija ldap_result_to_array) zapisa koji zadovoljavaju uvjete pretrage gdje su dohvacene vrijednosti base64 enkodirane
			false ili 0 ako nema nijednog zapisa koji zadovoljava uvjete pretrage
		*/
		
		public function ldapBinSearch($uid,$password,$base,$filter,$attributes,$md5=false){
			if(!$this->is_binary_arr($attributes)){
				\aosi\Logger::write("At least one of requested attributes is not binary (" . implode(', ' ,$attributes) .")!" ,4);
				$this->set_error(-33,"Bar jedan od traženih atributa nije binarni (" . implode(', ' ,$attributes) .")!");
				return false;
			}
			return $this->ldapSearch($uid,$password,$base,$filter,$attributes,true,$md5);
		}
		
		/*Funkcija sluzi provjeri postoji li korisnik u imeniku u ime administratora. 
		Ulazne vrijednosti:
			$username: uid administratora u imeniku
			$password: base64 enkodiran password administratora iz imenika
			$base: base dn korisnika
			$uid: uid korisnika za kojeg se radi provjera
		Izlazna vrijednost: 
			false: nema rezultata
			array (u obliku koji vraca funkcija ldap_result_to_array) koji sadrži samo jedan korisnički atribut: uid
			
		Po izvrsenju metode funkcija $obj->result() vraca array:
			'locked'=>true | false
			'authn_left'=>broj preostalih pokusaja autentikacije prije zakljucavanja acc-a,
			'pwd_expires_on'=>vrijeme isteka zaporke u formatu 'd.m.Y. H:i:s',
			'is_exipred'=>je li zaporka istekla 1|0,
			'time_left'=> ako zaporka nije istekla, za koliko vremena istice (formatirano)
			'time_left_s'=> ako zaporka nije istekla, za koliko vremena istice (u sekundama)
		*/
		
		public function ldapUserExists($username,$password,$base,$uid){
			if(hrEduPerson::$attribute['uid']->check_value($uid)){
				if($r=$this->ldapSearch($username,$password,$base,"(uid=$uid)",['uid'])){
					$this->merge_result(['locked'=>$this->isLockedPpolicy("uid=$uid,$base")]);
					return $r;
				} else {
					if(!$this->err_message()){
						$this->set_error('-16',"Korisnik uid=$uid,$base ne postoji u imeniku.");
						\aosi\Logger::write("User uid=$uid,$base doesn't exists." ,4);
					}
					return false;
				}
			} else {
				$this->set_error('-67',hrEduPerson::$attribute['uid']->err_message());
				\aosi\Logger::write("uid $uid is not valid." ,4);
				return false;
			}
		}
		
		/*Funkcija služi dohvatu org zapisa u ime administratora.
		Ulazne vrijednosti:
			$uid: uid korisnika u imeniku
			$password: base64 enkodiran password korisnika iz imenika
			$base: base dn imenika
		Izlazna vrijednost: 
		false: nema rezultata
		array (u obliku koji vraca funkcija ldap_result_to_array) org zapisa ustanove
		*/
		
		public function ldapOrgInfo($uid,$password,$base){
			if($this->ldapBind($uid,$password,$base)){
				$l=$this->rootBind();
				return $this->read_simple($l, $base, '(objectclass=hreduorg)', HrEduOrg::get_allowed_attrs('admin','view'));
			}
			return false;
		}
		
		/*Funkcija služi dohvatu org zapisa u ime korisnika.
		Ulazne vrijednosti:
			$uid: uid korisnika u imeniku
			$password: base64 enkodiran password korisnika iz imenika
			$base: base dn imenika
		Izlazna vrijednost: 
		false: nema rezultata
		array (u obliku koji vraca funkcija ldap_result_to_array) org zapisa ustanove
		*/
		
		public function userOrgInfo($uid,$password,$base){
			if($l=$this->bind($uid,$password,$base)){
				return $this->read_simple($l, $base, '(objectclass=hreduorg)', HrEduOrg::get_allowed_attrs('user','view'));
			}
			return false;
		}
		
		/*Funkcija služi dodavanju korisnika u LDAP imenik. Moze dodati jednog ili vise korisnika ovisno o ulaznim podacima
		Ulazne vrijednosti:
			$uid: uid administratora u imeniku
			$password: base64 enkodiran password administratora iz imenika
			$base: base dn imenika
			$data: array vrijednosti s podacima o korisniku / korisnicima koji se dodaju. [ [dn]=>[[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]]...] 
				array mora sadrzavati atribute 'uid','userpassword','hredupersonoib','givenname','sn','mail','hredupersonprimaryaffiliation','hredupersonexpiredate' iz kojih se izvode vrijednosti ostalih obavezniha atributa.
				Ako je hredupersonprimaryaffiliation=gost onda hredupersonoib smije biti none, ali onda je obavezan atribut hredupersonuniquenumber s nekom od ostalih vrijednosti
		Izlazna vrijednost: 
			true: svi korisnici ispravno dodani
			false: bar jedna korisnik nije dodan
			poslije izvrsenja funkcija result() ovog objekta vraca array oblika [ [dn1]=>[[status]=>true|false (je li uspjelo dodavanje),[code]=>kod greske ako postoji,[message]=>poruka o gresci ako postoji],[dn2]=>[].... ]
		*/
		
		public function ldapAddUserLE($uid,$password,$base,$data){
			if($this->ldapBind($uid,$password,$base)){
				$limits=$this->result(true);
				$l=$this->rootBind();				
				$org_data=$this->read_simple($l, $base, '(objectclass=hreduorg)',$this->org_attrs);
				$ok=true;
				foreach($data as $dn=>$d){
					\aosi\Logger::write("Add user LE called for $dn.",4);
					if(substr($dn,-1*strlen($base))!=$base){
						\aosi\Logger::write("User $dn base dn doesn't match directory base dn $base.",6);
						$ret[$dn]=['status'=>false,'code'=>-16,'message'=>"Korisnikov dn $dn na spada u base dn imenika $base."];
						$ok=false;
						$this->err_code(true);
						$this->err_message(true);
					} else {
						$d['hredupersonaffiliation']=$d['hredupersonprimaryaffiliation'];
						$d['cn']=[$d['givenname'][0] . ' ' . $d['sn'][0]];
						$d['displayname']=[$this->to_ascii($d['cn'][0])];
						$res=$this->addUser($l,$base,$d,$limits,$org_data[$base],$uid);
						$ok&=$res;
						$ret[$dn]=['status'=>$res,'code'=>$this->err_code(true),'message'=>$this->err_message(true)];
					}
				}
				$this->set_result($ret);
				return $ok;
			}
			return false;
		}
		
		/*Funkcija služi dodavanju korisnika u LDAP imenik. Moze dodati jednog ili vise korisnika ovisno o ulaznim podacima. Ulazni podaci moraju sadrzavati sve (obavezne) atribute.
		Ulazne vrijednosti:
			$uid: uid administratora u imeniku
			$password: base64 enkodiran password administratora iz imenika
			$base: base dn imenika
			$data: array vrijednosti s podacima o korisniku / korisnicima koji se dodaju. [ [dn]=>[[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]]...] array mora sadrzavati sve obavezne atribute.
		Izlazna vrijednost: 
			true: svi korisnici ispravno dodani
			false: bar jedna korisnik nije dodan
			poslije izvrsenja funkcija result() ovog objekta vraca array oblika [ [dn1]=>[[status]=>true|false (je li uspjelo dodavanje),[code]=>kod greske ako postoji,[message]=>poruka o gresci ako postoji],[dn2]=>[].... ]
		*/
		
		public function ldapAddUser($uid,$password,$base,$data){
			if($this->ldapBind($uid,$password,$base)){
				$limits=$this->result(true);
				$l=$this->rootBind();				
				$org_data=$this->read_simple($l, $base, '(objectclass=hreduorg)',$this->org_attrs);
				$ok=true;
				foreach($data as $dn=>$d){
					\aosi\Logger::write("Add user called for $dn.",8);
					if(substr($dn,-1*strlen($base))!=$base){
						\aosi\Logger::write("User $dn base dn doesn't match directory base dn $base.",6);
						$ret[$dn]=['status'=>false,'code'=>-16,'message'=>"Korisnikov dn $dn na spada u base dn imenika $base."];
						$ok=false;
					} else {
						$res=$this->addUser($l,$base,$d,$limits,$org_data[$base],$uid);
						$ok&=$res;
						$ret[$dn]=['status'=>$res,'code'=>$this->err_code(true),'message'=>$this->err_message(true)];
					}
				}
				$this->set_result($ret);
				return $ok;
			}
			return false;
		}
		
		/*Funkcija služi dodavanju atributa u korisnicki zapis u LDAP imeniku u ime administratora. Moze dodati jedan ili vise atributa za jednog ili vise korisnika.
		Ulazne vrijednosti:
			$uid: uid administratora u imeniku
			$password: base64 enkodiran password administratora
			$base: base dn imenika
			$data: array vrijednosti atributa se dodaju. [ [dn]=>[[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]]...] 
		Izlazna vrijednost: 
			true: za sve korisnike atribut(i) dodani
			false: bar greska u dodavanju
			poslije izvrsenja funkcija result() ovog objekta vraca array oblika [ [dn1]=>[[status]=>true|false (je li uspjelo dodavanje),[code]=>kod greske ako postoji,[message]=>poruka o gresci ako postoji],[dn2]=>[].... ]
		*/
		
		public function ldapAddAttribute($uid,$password,$base,$data){
			if($this->ldapBind($uid,$password,$base)){
				$limits=$this->result(true);
				$l=$this->rootBind();
				$ok=true;
				foreach($data as $dn=>$d){
					\aosi\Logger::write("Add attributes " . \aosi\array2log($data,true) . " called to user $dn.",4);
					if(substr($dn,-1*strlen($base))!=$base){
						$ok=false;
						\aosi\Logger::write("User $dn base dn doesn't match directory base dn $base.",6);
						$ret[$dn]=['status'=>false,'code'=>-16,'message'=>"Korisnikov dn $dn na spada u base dn imenika $base."];
					} else {
						$usr_data=$this->read_simple($l, $dn, "(objectclass=*)", \aosi\HrEduPerson::get_allowed_attrs('admin','view'));
						$usr_data=$usr_data[$dn];
						if(is_array($usr_data)){
							//ako admin ima ogranicenje, treba provjeriti smije li dodati ovog korisnika. 
							//ako korisnik ima one vrijednosti onih atributa po kojima admin ima ogranicenje, onda smije
							$okok=true;
							if(is_array($limits)){
								if(!$this->is_allowed_admin($limits,$usr_data)){
									$ok=false;
									$okok=false;
									$ret[$dn]=['status'=>false,'code'=>$this->err_code(true),'message'=>$this->err_message(true)];
								}
							}
							if($okok){
								$res=$this->addAtribute($l,$base,$dn,$d,'admin',$usr_data,$uid,$limits);
								$ok&=$res;
								$ret[$dn]=['status'=>$res,'code'=>$this->err_code(true),'message'=>$this->err_message(true)];
							}
						} else {
							$ok=false;
							\aosi\Logger::write("User $dn doesn't exist in directory $base.",6);
							$ret[$dn]=['status'=>false,'code'=>-16,'message'=>"Korisnik $dn ne postoji u imeniku $base."];
						}
					}
				}
				$this->set_result($ret);
				return $ok;
			}
			return false;
		}
		
		/*Funkcija služi dodavanju atributa u korisnicki zapis u LDAP imeniku u ime korisnika.
		Ulazne vrijednosti:
			$uid: uid korisnika u imeniku
			$password: base64 enkodiran password korisnika
			$base: base dn imenika
			$data: array vrijednosti atributa koji se dodaju. [ ['uid=$uid,$base']=>[[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]]...] 
		Izlazna vrijednost: 
			true: atribut(i) dodani
			false: greska kod dodavanja
			poslije izvrsenja funkcija result() ovog objekta vraca array oblika [ ['uid=$uid,$base']=>[[status]=>true|false (je li uspjelo dodavanje),[code]=>kod greske ako postoji,[message]=>poruka o gresci ako postoji],[dn2]=>[].... ]
		*/
		
		public function userAddAttribute($uid,$password,$base,$data){
			$this->set_result(false);
			if($l=$this->userBind($uid,$password,$base)){
				$ok=true;
				foreach($data as $dn=>$d){
					if("uid=$uid,$base"!=$dn){
						\aosi\Logger::write("User can add attribute values only to his record. uid=$uid,$base!=$dn",6);
						$ret[$dn]=['status'=>false,'code'=>-1,'message'=>"Korisnik smije dodavati atribute samo u svoj zapis u LDAP imeniku."];
						$ok=false;
					} else {
						\aosi\Logger::write("User adding attributes " . \aosi\array2log($data,true) . " to user $dn.",4);
						$usr_data=$this->read_simple($l, $dn, "(objectclass=*)", \aosi\HrEduPerson::get_allowed_attrs('user','view'));
						$usr_data=$usr_data[$dn];
						if(is_array($usr_data)){
							$res=$this->addAtribute($l,$base,$dn,$d,'user',$usr_data,$uid);
							$ok&=$res;
							$ret[$dn]=['status'=>$res,'code'=>$this->err_code(true),'message'=>$this->err_message(true)];
						} else {
							$ok=false;
							\aosi\Logger::write("User $dn doesn't exist in directory $base.",6);
							$ret[$dn]=['status'=>false,'code'=>-16,'message'=>"Korisnik $dn ne postoji u imeniku $base"];
						}
					}
				}
				$this->set_result($ret);
				return $ok;
			}
			return false;
		}
		
		/*Funkcija služi brisanju korisnika iz LDAP imenika u ime administratora. Moze dodati izbrisati više korisnika
		Ulazne vrijednosti:
			$uid: uid administratora u imeniku
			$password: base64 enkodiran password administratora
			$base: base dn imenika
			$data: array dn-ova korisnika koji se brisu.
		Izlazna vrijednost: 
			true: svi korisnici izbrisani
			false: bar jedna greska u brisanju
			poslije izvrsenja funkcija result() ovog objekta vraca array oblika [ [dn1]=>[[status]=>true|false (je li uspjelo brisanje),[code]=>kod greske ako postoji,[message]=>poruka o gresci ako postoji],[dn2]=>[].... ]
		*/
		
		public function ldapDeleteUser($uid,$password,$base,$data){
			if($this->ldapBind($uid,$password,$base)){
				$limits=$this->result(true);
				$l=$this->rootBind();
				$ok=true;
				foreach($data as $dn){					
					\aosi\Logger::write("Delete called for user $dn.",4);
					if(substr($dn,-1*strlen($base))!=$base){
						$ok=false;
						\aosi\Logger::write("User $dn base dn doesn't match directory base dn $base.",6);
						$ret[$dn]=['status'=>false,'code'=>-16,'message'=>"Korisnikov $dn nije jednak base dn imenika $base."];
					} else {
						if($dn=="uid=$uid,$base"){
							\aosi\Logger::write("Admin can't delete himself.",6);
							$ok=false;
							$ret[$dn]=['status'=>false,'code'=>-16,'message'=>"Administrator ne može izbrisati svoj zapis iz imenika."];
						} else {
							$usr_data=$this->read_simple($l, $dn, "(objectclass=*)", Config::get('admin_limits_attributes'));
							$usr_data=$usr_data[$dn];
							if(is_array($usr_data)){
								//ako admin ima ogranicenje, treba provjeriti smije li izbrisati. 
								//ako korisnik ima one vrijednosti onih atributa po kojima admin ima ogranicenje, onda smije
								$okok=true;
								if(is_array($limits)){
									if(!$this->is_allowed_admin($limits,$usr_data)){
										$ok=false;
										$okok=false;
										$ret[$dn]=['status'=>false,'code'=>$this->err_code(true),'message'=>$this->err_message(true)];
									}
								}								
								//todo poziv before metode plugina
								if($okok){
									if(@ldap_delete($l,$dn)){
										//todo poziv after delete pluginova
										\aosi\Logger::write("User $dn deleted from LDAP.",4);
										$ret[$dn]=['status'=>true,'code'=>0,'message'=>"Korisnik $dn izbrisan iz imenika."];
										\aosi\ActionLog::write($uid,__FUNCTION__,$dn,true);
									} else {
										$ok=false;
										$err=$this->get_ldap_error($l);
										$ret[$dn]=['status'=>false,'code'=>$err['code'],'message'=>"Nije uspjelo brisanje korisnika. " . $err['message']];
										\aosi\Logger::write("Deleting of user $dn failed. Error: " . $err['code'] . ", " . $err['message'],4);
										\aosi\ActionLog::write($uid,__FUNCTION__,$dn,false);
									}
								}
							} else {
								$ok=false;
								\aosi\Logger::write("User $dn doesn't exist in directory $base.",6);
								$ret[$dn]=['status'=>false,'code'=>-16,'message'=>"Korisnik $dn ne postoji u imeniku $base"];
							}
						}
					}
				}
				$this->set_result($ret);
				return $ok;
			}
		}
		
		/*Funkcija služi brisanju neke vrijednosti nekog atributa korisnika iz LDAP imenika u ime administratora. Moze dodati izbrisati vise vrijednosti vise atributa za vise korisnika
		Ulazne vrijednosti:
			$uid: uid administratora u imeniku
			$password: base64 enkodiran password administratora
			$base: base dn imenika
			$data: array vrijednosti atributa se brisu. [ [dn]=>[[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]],[dn1]=>[[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]]...] 
		Izlazna vrijednost: 
			true: svi atributi izbrisani
			false: bar jedna greska u brisanju
			poslije izvrsenja funkcija result() ovog objekta vraca array oblika [ [dn1]=>[[status]=>true|false (je li uspjelo brisanje),[code]=>kod greske ako postoji,[message]=>poruka o gresci ako postoji],[dn2]=>[].... ]
		*/
		
		public function ldapDeleteAttribute($uid,$password,$base,$data){
			if($this->ldapBind($uid,$password,$base)){
				$limits=$this->result(true);
				$l=$this->rootBind();
				$ok=true;
				foreach($data as $dn=>$attributes){					
					\aosi\Logger::write("Delete attributes called for user $dn.",4);
					\aosi\Logger::write("Attributes:" . array2log($attributes,true),6);
					$this->reset_err();
					if(substr($dn,-1*strlen($base))!=$base){
						$ok=false;
						\aosi\Logger::write("User $dn base dn doesn't match directory base dn $base.",6);
						$ret[$dn]=['status'=>false,'code'=>-16,'message'=>"Korisnikov $dn nije jednak base dn imenika $base."];
					} else {
						$okok=true;
						if($dn=="uid=$uid,$base"){
							//admin ne smije razvlastiti sam sebe
							if(is_array($attributes['hredupersonrole'])){
								foreach($attributes['hredupersonrole'] as $v){
									if(preg_match('/administrator imenika/',$v)){
										$ok=false;
										$okok=false;
										\aosi\Logger::write("Admin can't revoke admin privileges for himself!",6);
										$ret[$dn]=['status'=>false,'code'=>-1,'message'=>"Administrator ne može sam sebi ukinuti administratorska prava."];
										break;
									}
								}
							}
						}
						if($okok){
							$usr_data=$this->read_simple($l, $dn, "(objectclass=*)", \aosi\HrEduPerson::get_allowed_attrs('admin','view'));
							$usr_data=$usr_data[$dn];
							\aosi\Logger::write("Old values:" . array2log($usr_data,true),6);
							if(is_array($usr_data)){
								//ako admin ima ogranicenje, treba provjeriti ima li ovlasti nad korisnikom
								//admin ne smije izbrisati vrijednosti atributa po kojima ima ogranicenje
								$okok=true;
								if(is_array($limits)){
									if(!$this->is_allowed_admin($limits,$usr_data)){
										$ok=false;
										$okok=false;
										$ret[$dn]=['status'=>false,'code'=>$this->err_code(true),'message'=>$this->err_message(true)];
									} elseif(!$this->is_allowed_attribute($limits,$attributes)){
										$ok=false;
										$okok=false;
										$ret[$dn]=['status'=>false,'code'=>$this->err_code(true),'message'=>$this->err_message(true)];
									}
								}
								if($okok){
									$res=$this->deleteAtribute($l,$base,$dn,$attributes,'admin',$usr_data,$uid);
									$ok&=$res;
									$ret[$dn]=['status'=>$res,'code'=>$this->err_code(true),'message'=>$this->err_message(true)];
								}
							} else {
								$ok=false;
								\aosi\Logger::write("User $dn doesn't exist in directory $base.",6);
								$ret[$dn]=['status'=>false,'code'=>-16,'message'=>"Korisnik $dn ne postoji u imeniku $base"];
							}
						}
					}
				}
				$this->set_result($ret);
				return $ok;
			}
		}
		
		/*Funkcija služi brisanju neke vrijednosti nekog atributa korisnika iz LDAP imenika u ime korisnika. Moze dodati izbrisati vise vrijednosti vise atributa za danog korisnika.
		Ulazne vrijednosti:
			$uid: uid korisnika u imeniku
			$password: base64 enkodiran password korisnika
			$base: base dn imenika
			$data: array vrijednosti atributa se brisu. [ ['uid=$uid,$base']=>[[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]]...] 
		Izlazna vrijednost: 
			true: svi atributi izbrisani
			false: bar jedna greska u brisanju
			poslije izvrsenja funkcija result() ovog objekta vraca array oblika [ ['uid=$uid,$base']=>[[status]=>true|false (je li uspjelo brisanje),[code]=>kod greske ako postoji,[message]=>poruka o gresci ako postoji]]
		*/
		
		public function userDeleteAttribute($uid,$password,$base,$data){
			$this->set_result(false);
			if($l=$this->userBind($uid,$password,$base)){
				$ok=true;
				foreach($data as $dn=>$d){
					if("uid=$uid,$base"!=$dn){
						\aosi\Logger::write("User can delete attribute values only in his record. uid=$uid,$base!=$dn",6);
						$ret[$dn]=['status'=>false,'code'=>-1,'message'=>"Korisnik smije brisati atribute samo iz svog zapisa u LDAP imeniku."];
						$ok=false;
					} else {
						\aosi\Logger::write("User deleting attributes " . \aosi\array2log($d,true) . " from user $dn.",4);
						$usr_data=$this->read_simple($l, $dn, "(objectclass=*)", \aosi\HrEduPerson::get_allowed_attrs('user','view'));
						$usr_data=$usr_data[$dn];
						if(is_array($usr_data)){
							$res=$this->deleteAtribute($l,$base,$dn,$d,'user',$usr_data,$uid);
							$ok&=$res;
							$ret[$dn]=['status'=>$res,'code'=>$this->err_code(true),'message'=>$this->err_message(true)];
						} else {
							$ok=false;
							\aosi\Logger::write("User $dn doesn't exist in directory $base.",6);
							$ret[$dn]=['status'=>false,'code'=>-16,'message'=>"Korisnik $dn ne postoji u imeniku $base"];
						}
					}
				}
				$this->set_result($ret);
				return $ok;
			}
			return false;
		}
		
		/*Funkcija služi promjeni vrijednosti atributa korisnika iz LDAP imenika u ime administratora. Moze promijeniti vrijednost vise atributa za vise korisnika
		Promjena se vrši metodom replace sto znaci da se za neki atribut izbrisu sve stare vrijednosti. Ako atribut ima vise vrijednosti potrebno je u arrayu poslati SVE nove vrijednosti
		Ulazne vrijednosti:
			$uid: uid administratora u imeniku
			$password: base64 enkodiran password administratora
			$base: base dn imenika
			$data: array vrijednosti atributa koji se mijenjaju. [ [dn]=>[[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]],[dn1]=>[[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]]...] 
		Izlazna vrijednost: 
			true: svi atributi promijenjeni
			false: bar jedna greska u promjeni
			poslije izvrsenja funkcija result() ovog objekta vraca array oblika [ [dn1]=>[[status]=>true|false (je li uspjelo brisanje),[code]=>kod greske ako postoji,[message]=>poruka o gresci ako postoji],[dn2]=>[].... ]
		*/
		
		public function ldapModifyAttribute($uid,$password,$base,$data){
			if($this->ldapBind($uid,$password,$base)){
				$limits=$this->result(true);
				$l=$this->rootBind();
				$ok=true;
				foreach($data as $dn=>$attributes){					
					\aosi\Logger::write("Modify attributes called for user $dn.",4);
					\aosi\Logger::write("Attributes:" . array2log($attributes,true),6);
					$this->reset_err();
					if(substr($dn,-1*strlen($base))!=$base){
						$ok=false;
						\aosi\Logger::write("User $dn base dn doesn't match directory base dn $base.",6);
						$ret[$dn]=['status'=>false,'code'=>-16,'message'=>"Korisnikov $dn nije jednak base dn imenika $base."];
					} else {
						$okok=true;
						if($dn=="uid=$uid,$base"){
							//admin ne smije razvlastiti sam sebe
							if(is_array($attributes['hredupersonrole'])){
								$okok=false;
								foreach($attributes['hredupersonrole'] as $v){
									if(preg_match('/administrator imenika/',$v)){
										$okok=true;
									}
								}
								if(!$okok){
									\aosi\Logger::write("Admin can't revoke admin privileges for himself!",6);
									$ret[$dn]=['status'=>false,'code'=>-1,'message'=>"Administrator ne može sam sebi ukinuti administratorska prava."];
								}
							}
						}
						if($okok){
							$usr_data=$this->read_simple($l, $dn, "(objectclass=*)", \aosi\HrEduPerson::get_allowed_attrs('admin','view'));
							$usr_data=$usr_data[$dn];
							\aosi\Logger::write("Old values:" . array2log($usr_data,true),6);
							if(is_array($usr_data)){
								//ako admin ima ogranicenje, treba provjeriti ima li ovlasti nad korisnikom
								//admin ne smije izbrisati vrijednosti atributa po kojima ima ogranicenje
								$okok=true;
								if(is_array($limits)){
									if(!$this->is_allowed_admin($limits,$usr_data)){
										$ok=false;
										$okok=false;
										$ret[$dn]=['status'=>false,'code'=>$this->err_code(true),'message'=>$this->err_message(true)];
									} elseif(!$this->is_allowed_attribute($limits,$attributes)){
										$ok=false;
										$okok=false;
										$ret[$dn]=['status'=>false,'code'=>$this->err_code(true),'message'=>$this->err_message(true)];
									}
								}
								//todo poziv before metode plugina
								if($okok){
									$res=$this->modifyAtribute($l,$base,$dn,$attributes,'admin',$usr_data,$uid);
									$ok&=$res;
									$ret[$dn]=['status'=>$res,'code'=>$this->err_code(true),'message'=>$this->err_message(true)];
								}
							} else {
								$ok=false;
								\aosi\Logger::write("User $dn doesn't exist in directory $base.",6);
								$ret[$dn]=['status'=>false,'code'=>-16,'message'=>"Korisnik $dn ne postoji u imeniku $base"];
							}
						}
					}
				}
				$this->set_result($ret);
				return $ok;
			}
		}
		
		/*Funkcija služi promjeni vrijednosti atributa korisnika. Moze promijeniti vrijednost vise atributa
		Promjena se vrši metodom replace sto znaci da se za neki atribut izbrisu sve stare vrijednosti. Ako atribut ima vise vrijednosti potrebno je u arrayu poslati SVE nove vrijednosti
		Ulazne vrijednosti:
			$uid: uid korisnika u imeniku
			$password: base64 enkodiran password korisnika
			$base: base dn imenika
			$data: array vrijednosti atributa se mijenjaju. [ ['uid=$uid,$base']=>[[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]]...] 
		Izlazna vrijednost: 
			true: svi atributi promijenjeni
			false: bar jedna greska u promjeni
			poslije izvrsenja funkcija result() ovog objekta vraca array oblika [ ['uid=$uid,$base']=>[[status]=>true|false (je li uspjela promjena),[code]=>kod greske ako postoji,[message]=>poruka o gresci ako postoji]]
		*/
		
		public function userModifyAttribute($uid,$password,$base,$data){
			$this->set_result(false);
			if($l=$this->userBind($uid,$password,$base)){
				$ok=true;
				foreach($data as $dn=>$d){
					if("uid=$uid,$base"!=$dn){
						\aosi\Logger::write("User can modify attribute values only in his record. uid=$uid,$base!=$dn",6);
						$ret[$dn]=['status'=>false,'code'=>-1,'message'=>"Korisnik smije mijenjati atribute samo iz svog zapisa u LDAP imeniku."];
						$ok=false;
					} else {
						\aosi\Logger::write("User modifying attributes " . \aosi\array2log($d,true) . " from user $dn.",4);
						$usr_data=$this->read_simple($l, $dn, "(objectclass=*)", \aosi\HrEduPerson::get_allowed_attrs('user','view'));
						$usr_data=$usr_data[$dn];
						if(is_array($usr_data)){
							$res=$this->modifyAtribute($l,$base,$dn,$d,'user',$usr_data,$uid);
							$ok&=$res;
							$ret[$dn]=['status'=>$res,'code'=>$this->err_code(true),'message'=>$this->err_message(true)];
						} else {
							$ok=false;
							\aosi\Logger::write("User $dn doesn't exist in directory $base.",6);
							$ret[$dn]=['status'=>false,'code'=>-16,'message'=>"Korisnik $dn ne postoji u imeniku $base"];
						}
					}
				}
				$this->set_result($ret);
				return $ok;
			}
			return false;
		}
		
		/*Funkcija služi promjeni vrijednosti atributa iz org zapisa LDAP imenika u ime administratora. Moze promijeniti vrijednost vise atributa.
		Promjena se vrši metodom replace sto znaci da se za neki atribut izbrisu sve stare vrijednosti. Ako atribut ima vise vrijednosti potrebno je u arrayu poslati SVE nove vrijednosti
		Promjena vrijednosti atributa 'o','l','street','postalcode','postaladdress' propagira se u korisničke zapise svih korisnika u imeniku
		Atribut postaladdress se tvori iz vrijednosti ostalih atributa kao: o, street, postalcode, l
		Ulazne vrijednosti:
			$uid: uid administratora u imeniku
			$password: base64 enkodiran password administratora
			$base: base dn imenika
			$data: array vrijednosti atributa koji se mijenjaju. [[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]] 
		Izlazna vrijednost: 
			true: atributi promijenjeni
			false: greska u promjeni
		*/
		
		public function ldapModifyOrgAttribute($uid,$password,$base,$data){
			$admin_type=$this->ldapBind($uid,$password,$base);
			//samo samo admin bez ogranicenja smije mijenjati org atribute
			if($admin_type==1 || $admin_type==2){
				$l=$this->rootBind();
				$ok=true;			
				\aosi\Logger::write("Modify org attributes called.",4);
				\aosi\Logger::write("Attributes:" . array2log($data),6);
				$org_data=$this->read_simple($l, $base, "(objectclass=*)", \aosi\HrEduOrg::get_allowed_attrs('admin','view'));
				$org_data=$org_data[$base];
				\aosi\Logger::write("Old values:" . array2log($org_data,true),6);
				if(is_array($org_data)){
					if(count($data)>0){
						//bazicna validacija vrijednosti atributa za promjenu
						$res=\aosi\HrEduOrg::validate($data,'admin',false);
						if($res['result']){
							//validacija vrijednosti atributa je uspjesna
							\aosi\Logger::write("Validation successful.",6);
							if(is_array($data['hreduorguniquenumber'])){
								$oib_found=false;
								if(is_array($data['hreduorgoib'])){
									$oib=$data['hreduorgoib'][0];
								} else {
									$oib=$org_data['hreduorgoib'][0];
								}
								foreach($data['hreduorguniquenumber'] as $val){
									list($pref,$tmp)=explode(':',$val);
									if($pref=='OIB'){
										$oib_found=true;
										if("OIB: $oib"!=$val){
											\aosi\Logger::write("Value OIB of hreduorguniquenumber should be same as value of hreduorgoib.",6);
											$this->set_err_message("Vrijednost OIB atributa hreduorguniquenumber mora biti jednaka onoj u atributu hreduorgoib.");
											return false;
										}
									}
								}
								if(!$oib_found){
									\aosi\Logger::write("Value OIB of hreduorguniquenumber should be same as value of hreduorgoib.",6);
									$this->set_err_message("Vrijednost OIB atributa hreduorguniquenumber mora biti jednaka onoj u atributu hreduorgoib.");
									return false;
								}
							} elseif(is_array($data['hreduorgoib'])) {
								if($data['hreduorgoib'][0]=='NONE'){
									\aosi\Logger::write("hreduorgoib cannot be set to NONE!",6);
									$this->set_err_message("Vrijednost atributa hreduperson ne može biti NONE.");
									return false;
								}
								$data['hreduorguniquenumber']=[];
								$oib_found=false;
								foreach($org_data['hreduorguniquenumber'] as $val){
									list($pref,$tmp)=explode(':',$val);
									if($pref=='OIB'){
										$oib_found=true;
										$data['hreduorguniquenumber'][]="OIB: " . $data['hreduorgoib'][0];
									} else {
										$data['hreduorguniquenumber'][]=$val;
									}
								}
								if(!$oib_found){
									$data['hreduorguniquenumber'][]="OIB: " . $data['hreduorgoib'][0];
								}
							}
							if(is_array($data['postaladdress'])){
								//postaladdres je atribut koji bi tvore vrijednosti drugih atributa i ne bi ga trebalo mijenjatii direktno
								$this->set_error(-64,"postaladdres je atribut čija se vrijednost tvori iz konkatenacijom vrijednosti atributa o, street, postalcode, l i ne bi ga se trebalo mijenjati direktno. Ako ga želite promijeniti, promijenite vrijednost nekog od atributa od kojih se tvori.");
								\aosi\Logger::write("Value of attribute postaladdres should be changed using attributes o, street, postalcode, l.",6);
								return false;
							}
							$usr_attrs=$this->user_org_attributes($data,$org_data);
							if(is_array($usr_attrs['postaladdress'])){
								$data['postaladdress']=$usr_attrs['postaladdress'];
							}
							\aosi\Logger::write("Modifying org record: " . \aosi\array2log($data) ,8);
							if(@ldap_mod_replace($l,$base, $data)){
								\aosi\Logger::write("Attribute(s) modifyed.",4);
								// ako se mijenja neki od org atributa koji su u korisnickim zapisima, treba ih promijeniti svim korisnicima.
								if(is_array($usr_attrs)){
									if(count($usr_attrs)>0){
										//treba promijeniti atribute svim korisnicima u imeniku
										$this->modify_all_users($l,$base,$usr_attrs);
									}
								}
								\aosi\ActionLog::write($uid,__FUNCTION__,$base,true,$data);
								return true;
							} else {
								$this->ldap_failed_finish($l,'Promjena vrijednosti org atributa nije uspjela.','Modifying org attribute(s) failed.',4,$uid,__FUNCTION__,$base,$data);
								return false;
							}
						} else {
							//validacija nije prosla
							\aosi\Logger::write("Validation failed. Attribute(s) not modifyed.",6);
							\aosi\Logger::write("Validation result: ". \aosi\array2log($res['data'],false),6);
							$this->set_err_message($this->validation_result_to_txt($res['data']));
							return false;
						}
					} else {
						$this->set_error(-64,"Nema atributa za promjenu.");
						\aosi\Logger::write("Nothing to modify.",6);
						return false;
					}
				} else {
					\aosi\Logger::write("Org record $base doesn't exist in directory.",6);
					$this->set_error(-64,"Org zapis $base ne postoji u imeniku.");
					return false;
				}
			} else {
				\aosi\Logger::write("User uid=$uid,$base isn't allowed to change org record.",6);
				$this->set_error(-15,"Niste ovlašteni promijeniti atribute u org zapisu LDAP imenika $base.");
				return false;
			}
		}
		
		/*Funkcija služi brisanju vrijednosti atributa iz org zapisa LDAP imenika u ime administratora. Moze izbrisati vrijednost vise atributa.		
		Ulazne vrijednosti:
			$uid: uid administratora u imeniku
			$password: base64 enkodiran password administratora
			$base: base dn imenika
			$data: array vrijednosti atributa koje se brisu. [[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]] 
		Izlazna vrijednost: 
			true: atributi izbrisani
			false: greska kod brisanja
		*/
		
		public function ldapDeleteOrgAttribute($uid,$password,$base,$data){
			$admin_type=$this->ldapBind($uid,$password,$base);
			//samo samo admin bez ogranicenja smije mijenjati org atribute
			if($admin_type==1 || $admin_type==2){
				$l=$this->rootBind();	
				\aosi\Logger::write("Delete org attributes called.",4);
				\aosi\Logger::write("Attributes:" . array2log($data),6);
				$org_data=$this->read_simple($l, $base, "(objectclass=*)", \aosi\HrEduOrg::get_allowed_attrs('admin','view'));
				$org_data=$org_data[$base];
				\aosi\Logger::write("Old values:" . array2log($org_data,true),6);
				if(is_array($org_data)){
					if(count($data)>0){
						//ne smije se brisati vrijedost atributa hreduorguniquenumber koja sadrzi OIB
						if(is_array($data['hreduorguniquenumber'])){
							foreach($data['hreduorguniquenumber'] as $val){
								list($pref,$tmp)=explode(':',$val);
								if($pref=='OIB'){
									\aosi\Logger::write("You can't delete value OIB of attribute hreduorguniquenumber.",6);
									$this->set_err_message("Ne možete izbrisati vrijednost OIB atributa hreduorguniquenumber.");
									return false;
								}
							}
						}
						foreach($data as $name=>$value){
							if(is_object(\aosi\HrEduOrg::$attribute[$name])){
								//smije li korisnik / admin mijenjati vrijednost atributa
								if(!\aosi\HrEduOrg::$attribute[$name]->is_allowed('admin','change')){
									$this->set_err_message("Niste ovlašteni mijenjati vrijednost atributa $name.");
									\aosi\Logger::write("You are not allowed to change value of attribute $name.",6);
									return false;
								}
								//ako je obavezan
								if(\aosi\HrEduOrg::$attribute[$name]->must=='true'){
									//ako moze imati samo jednu vrijednost ne smije se brisati
									if(\aosi\HrEduOrg::$attribute[$name]->multiple=='false'){
										$this->set_err_message("Atribut $name je obavezan. Ne smijete mu izbrisati vrijednost.");
										\aosi\Logger::write("Attribute $name is obligatory.",6);
										return false;
									} else {
										if(count($value)>=count($org_data[$name]) || count($value)==0){
											$this->set_err_message("Atribut $name je obavezan. Ne smijete mu izbrisati sve vrijednosti.");
											\aosi\Logger::write("Attribute $name is obligatory.",6);
											return false;
										}
									}
								}
							} else {
								$this->set_err_message("Atribut $name ne postoji u HrEduOrg shemi.");
								\aosi\Logger::write("Attribute $name doesn't exist in HrEduOrg schema.",6);
								return false;
							}
						}
						
						$replace_result=$this->delete_no_matching_rule_attributes($l,$base,$data,\aosi\HrEduOrg::class,$org_data,$uid,__FUNCTION__);
				
						if(count($data)>0){
							//sve ok brisemo u LDAP-u
							if(@ldap_mod_del($l,$base, $data)){
								\aosi\Logger::write("Attribute(s) deleted from LDAP.",4);
								\aosi\ActionLog::write($uid,__FUNCTION__,$base,true,$data);
								return true;
							} else {
								$this->ldap_failed_finish($l,'Brisanje org atributa nije uspjelo.','Deleting org attribute(s) failed.',4,$uid,__FUNCTION__,$base,$data);
								return false;
							}
						} else {
							return $replace_result;
						}
					} else {
						$this->set_error(-64,"Nema atributa za brisanje.");
						\aosi\Logger::write("Nothing to delete.",6);
						return false;
					}
				} else {
					\aosi\Logger::write("Org record $base doesn't exist in directory.",6);
					$this->set_error(-64,"Org zapis $base ne postoji u imeniku.");
					return false;
				}
			} else {
				\aosi\Logger::write("User uid=$uid,$base isn't allowed to change org record.",6);
				$this->set_error(-15,"Niste ovlašteni promijeniti atribute u org zapisu LDAP imenika $base.");
				return false;
			}
		}
		
		/*Funkcija služi dodavanje vrijednosti atributa iz org zapisa LDAP imenika u ime administratora. Moze dodati vrijednost vise atributa.		
		Ulazne vrijednosti:
			$uid: uid administratora u imeniku
			$password: base64 enkodiran password administratora
			$base: base dn imenika
			$data: array vrijednosti atributa koje se dodaju. [[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]] 
		Izlazna vrijednost: 
			true: atributi dodani
			false: greska kod dodavanja
		*/
		
		public function ldapAddOrgAttribute($uid,$password,$base,$data){
			$admin_type=$this->ldapBind($uid,$password,$base);
			//samo samo admin bez ogranicenja smije mijenjati org atribute
			if($admin_type==1 || $admin_type==2){
				$l=$this->rootBind();	
				\aosi\Logger::write("Add org attributes called.",4);
				\aosi\Logger::write("Attributes:" . array2log($data),6);
				$org_data=$this->read_simple($l, $base, "(objectclass=*)", \aosi\HrEduOrg::get_allowed_attrs('admin','view'));
				$org_data=$org_data[$base];
				\aosi\Logger::write("Old values:" . array2log($org_data,true),6);
				if(is_array($org_data)){
					if(count($data)>0){
						//bazicna validacija vrijednosti atributa za dodavanje
						$res=\aosi\HrEduOrg::validate($data,'admin',false);
						if($res['result']){
							//validacija vrijednosti atributa je uspjesna
							\aosi\Logger::write("Validation successful.",6);
							foreach($data as $name=>$val){
								//ako vec postoji atribut, a ne smije imati vise vrijednosti, ne moze ga se dodati
								if(is_array($org_data[$name]) && \aosi\HrEduOrg::$attribute[$name]->multiple!='true'){
									\aosi\Logger::write("Attribute $name can have only one value.",6);
									$this->set_err_message("Atribut $name smije imati samo jednu vrijednost.");
									return false;
								}
							}
							if(is_array($data['hreduorguniquenumber'])){
								foreach($data['hreduorguniquenumber'] as $val){
									list($pref,$tmp)=explode(':',$val);
									if($pref=='OIB'){
										\aosi\Logger::write("You can't add value OIB of attribute hreduorguniquenumber.",6);
										$this->set_err_message("Ne možete dodati vrijednost OIB atributa hreduorguniquenumber.");
										return false;
									}
									foreach($org_data['hreduorguniquenumber'] as $v){
										list($pref2,$tmp)=explode(':',$v);
										if($pref==$pref2){
											\aosi\Logger::write("Attribute hreduorguniquenumber already has value $pref. You can't add another value.",6);
											$this->set_err_message("Atribut hreduorguniquenumber već ima vrijednost $pref. Ne možete dodati još jednu vrijednost.");
											return false;
										}
									}
								}
							}
							$replace_result=$this->add_no_matching_rule_attributes($l,$base,$data,\aosi\HrEduOrg::class,$org_data,$uid,__FUNCTION__);
							if(count($data)>0){
								if(@ldap_mod_add($l,$base, $data)){
									\aosi\Logger::write("Attribute(s) added to LDAP.",4);
									\aosi\ActionLog::write($uid,__FUNCTION__,$base,true,$data);
									return true;
								} else {
									$this->ldap_failed_finish($l,'Dodavanje org atributa LDAP nije uspjelo.','Adding attribute(s) to LDAP failed.',4,$uid,__FUNCTION__,$base,$data);
									return false;
								}
							} else {
								return $replace_result;
							}
						} else {
							//validacija nije prosla
							\aosi\Logger::write("Validation failed. Attribute is not added.",6);
							\aosi\Logger::write("Validation result: ". \aosi\array2log($res['data'],false),6);
							$this->set_err_message($this->validation_result_to_txt($res['data']));
							return false;
						}
					} else {
						$this->set_error(-64,"Nema atributa za dodavanje.");
						\aosi\Logger::write("Nothing to add.",6);
						return false;
					}
				} else {
					\aosi\Logger::write("Org record $base doesn't exist in directory.",6);
					$this->set_error(-64,"Org zapis $base ne postoji u imeniku.");
					return false;
				}
			} else {
				\aosi\Logger::write("User uid=$uid,$base isn't allowed to change org record.",6);
				$this->set_error(-15,"Niste ovlašteni promijeniti atribute u org zapisu LDAP imenika $base.");
				return false;
			}
		}

	
	/*Funkcija vraca listu korisnika kojima je zbog password policy-a e-identitet zakljucan ili ce uskoro biti.
		Ulazne vrijednosti:
			$uid: uid administratora u imeniku
			$password: base64 enkodiran password administratora iz imenika
			$base: base dn administratora u imeniku
		Izlazna vrijednost: 
			false: greska
			true: uspjeh
			
		Po izvrsenju metode funkcija $obj->result() vraca array:
		['dn1'=>
			['uid'=>uid,
			'locked'=>true | false
			'authn_left'=>broj preostalih pokusaja autentikacije prije zakljucavanja acc-a,
			'pwd_expires_on'=>vrijeme isteka zaporke u formatu 'd.m.Y. H:i:s',
			'is_exipred'=>je li zaporka istekla 1|0,
			'time_left'=> ako zaporka nije istekla, za koliko vremena istice (formatirano)
			'time_left_s'=> ako zaporka nije istekla, za koliko vremena istice (u sekundama)],
		'dn2'=>[...], ....]
		*/
		
		public function ldapLockedList($uid,$password,$base){
			if($admintype=$this->ldapBind($uid,$password,$base)){
				$limits=[];
				if($admintype==3){
					$limits=$this->result();
				}
				$this->set_result(false);
				$pwdpolicysubentry=\aosi\Config::get('ppolicy_dn_prefix') . ',' .$base;
				$l=$this->rootBind();
				if($ppolicy=$this->read_simple($l, $pwdpolicysubentry, "(objectclass=*)", ['*'])){
					$ppolicy=reset($ppolicy);
					$filter=$this->filter_from_limits("(pwdpolicysubentry=$pwdpolicysubentry)",$limits);
					if($locked=$this->search_simple($l, $base, $filter, ['pwdpolicysubentry','pwdchangedtime','pwdgraceusetime','pwdLockout','uid'])){
						$ret=[];
						foreach($locked as $dn=>$usr_data){
							$this->merge_result(['locked'=>$this->isLocked($dn,$usr_data,$ppolicy),'uid'=>$usr_data['uid']]);
							$ret[$dn]=$this->result(true);
						}
					} else {
						$this->set_err_message("U imeniku nema zaključanih korisnika, a niti korisnika koji će uskoro biti zaključani zbog politike obavezne promjene zaporke.");
					}
					$this->set_result($ret);
					return true;
				} else {
					$this->set_error(-64,"Nije pronađen zapis s definicijom politike promjene zaporke $pwdpolicysubentry.");
					\aosi\Logger::write("Error: ppolicy record $pwdpolicysubentry doesn't exist.",1);
					return false;
				}
			}
			return false;
		}
		
		public function certify($dn,$password,$base){
			$unique=['mail','hredupersonuniquenumber','hredupersonoib'];
			//prije 90 dana
			$datum=strtotime(date("Ymd", strtotime("-90 day")));
			if($this->dnBind($dn,$password)){
				$l=$this->rootBind();
				if($users=$this->search_simple($l, $base, '(&(uid=*)(objectclass=hreduperson))', ['uid','mail','hredupersonuniquenumber','hredupersonexpiredate','hredupersonprimaryaffiliation','hredupersonoib'])){
					$unique_vals=[];
					$duplicated=[];
					$invalid=[];
					$expired=[];
					$student_none=[];
					$cnt=0;
					foreach($users as $dn=>$u){
						$cnt++;
						foreach($unique as $attr){
							//za sve vrijednosti atributa koji trebaju biti jedistveni valja 
							//provjeriti postoji li vec u imeniku korisnik koji ima tu vrijednost
							foreach($u[$attr] as $v){
								if(isset($unique_vals[$attr][$v])){
									if(!is_array($duplicated[$attr][$v])){
										$duplicated[$attr][$v]=[$unique_vals[$attr][$v]];
									}
									array_push($duplicated[$attr][$v],$dn);
								} else {
									$unique_vals[$attr][$v]=$dn;
								}
								if(!\aosi\HrEduPerson::$attribute[$attr]->check_value($v)){
									$invalid[$attr][$dn]=$v;
								}
							}
							if($u['hredupersonexpiredate'][0]=='NONE'){
								if($u['hredupersonprimaryaffiliation'][0]=='učenik' || $u['hredupersonprimaryaffiliation'][0]=='student'){
									$student_none[$dn]=$u['hredupersonprimaryaffiliation'][0];
								}
							} elseif($u['hredupersonexpiredate'][0]<$datum){
								$expired[$dn]=$u['hredupersonexpiredate'][0];
							}
						}
					}
					print_r($duplicated);
					print_r($invalid);
				} else {
					$this->set_error(-64,"Nije pronađen nijedan zapis.");
					\aosi\Logger::write("No records found.",6);
					return false;
				}
			} else {
				return false;
			}
		}
		
		private function valid_email($mail){
			return preg_match('/^[\w\-\_\.\+!#\$%&\'\*\+\/=\?\^_`]+\@([\w\-\_]+\.)+[a-zA-Z]{2,}$/',$mail);
		}
		
		
		/*
			Funkcija svim korisnicima u imeniku mijenja vrijednost atributa koje dobije kao argument
		Ulaz:
			$l: ldap resurs (bindan kao root)
			$base: base dn imenika
			$data: array vrijednosti atributa se mijenjaju. [[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]]...] 
		
		*/
		
		private function modify_all_users($l,$base,$data){
			\aosi\Logger::write("Modifying data for all users: " . array2log($data,true),6);
			$sr=@ldap_search($l, $base, '(uid=*)', ['uid']);
			$res=@ldap_get_entries($l, $sr);
			\aosi\Logger::write("Found " . $res['count'] . ' records to modify. Modifying.',6);
			$ok=0;
			if($res['count']>0){
				for($i=0;$i<$res['count'];$i++){
					
					//todo plugin before i after modify
					
					if(@ldap_mod_replace($l,$res[$i]['dn'],$data)){
						\aosi\Logger::write("Attribute(s) modifyed for user " . $res[$i]['dn'],8);
						$ok++;
					} else {
						$err=$this->get_ldap_error($l);
						$errtxt= $res[$i]['dn'] .":" . $err['code'] .":" . $err['message'];
						\aosi\Logger::write("Modifying attribute(s) failed for user " . $errtxt,8);
						$errors.="|$errtxt";
					}
				}
				\aosi\Logger::write("Modifying succesful for $ok users.",6);
				if($errors) \aosi\Logger::write("Errors:" . $errors,6);
			}
			return $ok;
		}
		
		
		/*
		
		funkcija provjerava mijenja li se neki od atributa koji su u korisnickom zapisu naslijedjeni iz org zapisa.
		Ako se mijenja u izlaz je array s vrijednostima atributa koje treba promijeniti svim korisnickim zapisima u imeniku.
		'o','l','street','postalcode','postaladdress'
		Ulaz:
			$data: popis atributa koji se mijenjaju u org zapisu
			$old_data: vrijednosti org zapisa prije promjene
		Izlaz:
			false | array vrijednosti atributa koje treba promijeniti svim korisnicima u imeniku.
		
		*/
		private function user_org_attributes($data,$old_data){
			$ret=[];
			$vals=[];
			foreach($this->org_attrs as $a){
				if(is_array($data[$a])){
					$ret[$a]=$data[$a];
					$vals[$a]=$data[$a];
				} else {
					$vals[$a]=$old_data[$a];
				}
			}
			if(count($ret)>0){
				//neki od atributa je promijenjen, treba promijeniti vrijedost atributa postaladdress
				$ret['postaladdress'][0]=$vals['o'][0] . ', ' . $vals['street'][0] . ', ' . $vals['postalcode'][0] . ', '. $vals['l'][0]; 
				return $ret;
			}
			return false;
		}
		
		/*Funkcija služi dodavanju atributa u korisnicki zapis u LDAP imeniku. Moze dodati jedan ili vise atributa za jednog korisnika.
		Ulazne vrijednosti:
			$l: ldap resurs (bindan kao root)
			$base: base dn imenika
			$data: array vrijednosti atributa se dodaju. [[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]]...] 
			$usertype: admin|user tip korisnika koji zove operaciju
			$usr_data: trenutni podaci o korisniku nad cijim se zapisom izvodi operacija iz imenika. moraju sadrzavati minimalno atribute cije se vrijednosti pokusavaju izbrisati
			$callerUid: uid pozivatelja funkcije (akcije) - za potrebe pisanja u log
		Izlazna vrijednost: 
			true: atributi dodani
			false: nije uspjelo dodavanje
		*/
		
		private function deleteAtribute($l,$base,$dn,$data,$usertype,$usr_data,$callerUid){
			if(count($data)>0){
				if(is_array($data['userpassword'])){
					//password se ne smije izbrisati.
					$this->set_err_message("Ne možete izbrisati vrijednsot atributa userpassword.");
					\aosi\Logger::write("Can't delete the password.",6);
					return false;
				}
				//ne smije se brisati vrijedost atributa hredupersonaffiliation koja sadrzi primaryaffiliation
				if(is_array($data['hredupersonaffiliation'])){
					\aosi\Logger::write("hrEduPersonAffiliation:" . \aosi\array2log($data['hredupersonaffiliation']) ,8);
					if(in_array($usr_data['hredupersonprimaryaffiliation'][0],$data['hredupersonaffiliation'])){
						//ne smije se izbrisati vrijednost atributa hredupersonaffiliation koja sadrzi primaryaffiliation
						$this->set_err_message("Ne možete izbrisati vrijednsot " . $usr_data['hredupersonprimaryaffiliation'][0] . " atributa hrEduPersonAffiliation zato što je to ujedno vrijednost atributa hrEduPersonPrimaryAffiliation.");
						\aosi\Logger::write("You are not allowed to delete value " .$usr_data['hredupersonprimaryaffiliation'][0] ." of attribute hrEduPersonAffiliation." ,6);
						return false;
					}
				}
				//ne smije se brisati vrijedost atributa hredupersonuniquenumber koja sadrzi OIB
				if(is_array($data['hredupersonuniquenumber'])){
					foreach($data['hredupersonuniquenumber'] as $val){
						list($pref,$tmp)=explode(':',$val);
						if($pref=='OIB'){
							\aosi\Logger::write("You can't delete value OIB of attribute hredupersonuniquenumber.",6);
							$this->set_err_message("Ne možete izbrisati vrijednost OIB atributa hredupersonuniquenumber.");
							return false;
						}
					}
				}
				foreach($data as $name=>$value){
					if(is_object(\aosi\HrEduPerson::$attribute[$name])){
						//smije li korisnik / admin mijenjati vrijednost atributa
						if(!\aosi\HrEduPerson::$attribute[$name]->is_allowed($usertype,'change')){
							$this->set_err_message("Niste ovlašteni mijenjati vrijednost atributa $name.");
							\aosi\Logger::write("You are not allowed to change value of attribute $name.",6);
							return false;
						}
						//ako je obavezan
						if(\aosi\HrEduPerson::$attribute[$name]->must=='true'){
							//ako moze imati samo jednu vrijednost ne smije se brisati
							if(\aosi\HrEduPerson::$attribute[$name]->multiple=='false'){
								$this->set_err_message("Atribut $name je obavezan. Ne smijete mu izbrisati vrijednost.");
								\aosi\Logger::write("Attribute $name is obligatory.",6);
								return false;
							} else {
								if(count($value)>=count($usr_data[$name]) || count($value)==0){
									$this->set_err_message("Atribut $name je obavezan. Ne smijete mu izbrisati sve vrijednosti.");
									\aosi\Logger::write("Attribute $name is obligatory.",6);
									return false;
								}
							}
						}
					} else {
						$this->set_err_message("Atribut $name ne postoji u hrEduPerson shemi.");
						\aosi\Logger::write("Attribute $name doesn't exist in hrEduPerson schema.",6);
						return false;
					}
				}				
				
				$replace_result=$this->delete_no_matching_rule_attributes($l,$dn,$data,\aosi\HrEduPerson::class,$usr_data,$callerUid,debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 2)[1]['function']);
				
				if(count($data)>0){
					//sve ok brisemo u LDAP-u
					if(@ldap_mod_del($l,$dn, $data)){
						//todo poziv after add pluginova
						\aosi\Logger::write("Attribute(s) deleted from LDAP.",4);
						\aosi\ActionLog::write($callerUid,debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 2)[1]['function'],$dn,true,$data);
						return true;
					} else {
						$this->ldap_failed_finish($l,'Brisanje atributa nije uspjelo.','Deleting attribute(s) failed.',4,$callerUid,debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 2)[1]['function'],$dn,$data);
						return false;
					}
				} else {
					return $replace_result;
				}
			} else {
				$this->set_err_message("Nema atributa za brisanje.");
				\aosi\Logger::write("Nothing to delete.",6);
				return false;
			}
		}
		
		/* Funkcija brise vrijednosti atribute za koje ne postoji equality matching rule
		Zbog nepostojanja equality matching rule kod takvih se atributa ne moze izbrisati zadana vrijednost 
		Ova funkcija dohvatit sve vrijednosti i napravi replace bez vrijedosti koju treba izbrisati.
		OPREZ! array atributa za brisanje se proslijdjuje u funkciju by reference i funkcija iz njega brise vrijednosti koje replacea!
		Ulaz:
			$l: ldap resurs,
			$dn: dn zapisa na koji se odnosi brisanje
			$data: array atributa za brisanje
			$schemaobj: staticki objekt s definicijom sheme (\aosi\HrEduPerson | \aosi\HrEduOrg),
			$old_data: trenutni podaci o zapisu na kojem se izvodi brisanje
			$callerUid: uid pozivatelja funkcije (akcije) - za potrebe pisanja u action log
			$callerMethod: metoda koju treba upisati u action log
		Izlaz:
			true (nema atributa za replace ili uspjesan replace) | false (nije uspio replace)
			*/
		private function delete_no_matching_rule_attributes($l,$dn,&$data,$schemaobj,$old_data,$callerUid,$callerMethod){
			$replace=[];
			foreach($schemaobj::$no_eq_matching_rule as $name){
				if(is_array($data[$name]) && count($data[$name])>0){
					if(is_array($old_data[$name])){
						//slazemo array vrijednosti za replace. u arrayu su sve vrijednosti atributa koje se ne pokusavaju izbrisati.
						$replace[$name]=[];
						foreach($old_data[$name] as $k=>$val){
							if($schemaobj::$attribute[$name]->type=='binary'){
								$val=md5($val);
							}
							if(!in_array($val,$data[$name])){
								$replace[$name][]=$old_data[$name][$k];
							}
						}
						if(count($replace[$name])>0){
							//ako se neki atribut mijenja replace-om, onda ga brisemo iz arraya za brisnaje
							unset($data[$name]);
						} else {
							//ako ne ostaje nijedna (stara) vrijednost koja nije jednaka vrijednostima za brisanje, onda se brisu sve vrijednosti.
							unset($replace[$name]);
							$data[$name]=[];
						}
					} else {
						//nema starih vrijednosti, nema se sto brisati
						unset($data[$name]);
					}
				}
			}
			//ako postoje 
			if(count($replace)>0){
				//postoje vrijednosti koje treba replace-ati
				if(@ldap_mod_replace($l,$dn, $replace)){
					//todo poziv after replace pluginova
					\aosi\Logger::write("Attribute(s) without maching rule " . implode(',', array_keys($replace)) . " modifyed.",4);
					\aosi\ActionLog::write($callerUid,$callerMethod,$dn,true,$replace);
				} else {
					$this->ldap_failed_finish($l,'Promjena vrijednosti atributa bez matching rule-a nije uspjela.','Modifying attribute(s) without maching rule failed.',4,$callerUid,$callerMethod,$dn,$replace);
					return false;
				}
			}
			return true;
		}
		
		/* Funkcija dodaje vrijednosti atributa za koje ne postoji equality matching rule
		Zbog nepostojanja equality matching rule kod takvih se atributa ne moze dodati vrijednost 
		Ova funkcija dohvati sve vrijednosti vec upisane u taj atribut, doda im novu vrijednost i napravi replace.
		OPREZ! array atributa za dodavanje se proslijdjuje u funkciju by reference i funkcija iz njega brise vrijednosti koje replacea!
		Ulaz:
			$l: ldap resurs,
			$dn: dn zapisa na koji se odnosi brisanje
			$data: array atributa za brisanje
			$schemaobj: staticki objekt s definicijom sheme (\aosi\HrEduPerson | \aosi\HrEduOrg),
			$old_data: trenutni podaci o zapisu na kojem se izvodi dodavanje
			$callerUid: uid pozivatelja funkcije (akcije) - za potrebe pisanja u action log
			$callerMethod: metoda koju treba upisati u action log
		Izlaz:
			true (nema atributa za replace ili uspjesan replace) | false (nije uspio replace)
			*/
		private function add_no_matching_rule_attributes($l,$dn,&$data,$schemaobj,$old_data,$callerUid,$callerMethod){
			$replace=[];
			foreach($schemaobj::$no_eq_matching_rule as $name){
				if(is_array($data[$name]) && count($data[$name])>0){
					//slazemo array vrijednosti za replace. u arrayu su sve vrijednosti atributa.
					$replace[$name]=$data[$name];
					if(is_array($old_data[$name])){
						foreach($old_data[$name] as $k=>$val){
	/*						if($schemaobj::$attribute[$name]->type=='binary'){
								$e=false;
								foreach($replace[$name] as $v){
									if($val==$v) $e=true;
								}
								if(!$e) $replace[$name][]=$old_data[$name][$k];
							} else {
								if(!in_array($val,$replace[$name])){
									$replace[$name][]=$old_data[$name][$k];
								}
							} */
							if(!in_array($val,$replace[$name])){
								$replace[$name][]=$old_data[$name][$k];
							}
						}
					}
					unset($data[$name]);
				}
			}
			//ako postoje 
			if(count($replace)>0){
				print_r(hide_password($replace));
				//postoje vrijednosti koje treba replace-ati
				if(@ldap_mod_replace($l,$dn, $replace)){
					//todo poziv after replace pluginova
					\aosi\Logger::write("Attribute(s) without maching rule " . implode(',', array_keys($replace)) . " modifyed.",4);
					\aosi\ActionLog::write($callerUid,$callerMethod,$dn,true,$replace);
				} else {
					$this->ldap_failed_finish($l,'Promjena vrijednosti atributa bez matching rule-a nije uspjela.','Modifying attribute(s) without maching rule failed.',4,$callerUid,$callerMethod,$dn,$replace);
					return false;
				}
			}
			return true;
		}
		
		
		/*Funkcija služi dodavanju atributa u korisnicki zapis u LDAP imeniku. Moze dodati jedan ili vise atributa za jednog korisnika.
		Ulazne vrijednosti:
			$l: ldap resurs (bindan kao root)
			$base: base dn imenika
			$data: array vrijednosti atributa se dodaju. [[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]]...] 
			$usertype: admin|user tip korisnika koji zove operaciju
			$usr_data: trenutni podaci o korisniku nad cijim se zapisom izvodi operacija iz imenika
			$callerUid: uid pozivatelja funkcije (akcije) - za potrebe pisanja u log
			$admin_limits: array ogranicenja administratora (ako je admin s ogranicenjem)
		Izlazna vrijednost: 
			true: atributi dodani
			false: nije uspjelo dodavanje
		*/
		
		private function addAtribute($l,$base,$dn,$data,$usertype,$usr_data,$callerUid,$admin_limits=false){
			if(is_array($data['userpassword'])){
				//password se ne smije dodati
				$this->set_err_message("Ne možete dodati vrijednsot atributa userpassword.");
				\aosi\Logger::write("Can't add the password.",6);
				return false;
			}
			if(count($data)>0){
				//bazicna validacija vrijednosti atributa za dodavanje
				$res=\aosi\HrEduPerson::validate($data,$usertype,false);
				if($res['result']){
					//validacija vrijednosti atributa je uspjesna
					\aosi\Logger::write("Validation successful.",6);
					
					$unique=Config::get('unique');
					foreach($data as $name=>$val){
						//ako vec postoji atribut, a ne smije imati vise vrijednosti, ne moze ga se dodati
						if(is_array($usr_data[$name]) && \aosi\HrEduPerson::$attribute[$name]->multiple!='true'){
							\aosi\Logger::write("Attribute $name can have only one value.",6);
							$this->set_err_message("Atribut $name smije imati samo jednu vrijednost.");
							return false;
						}
						if(in_array($name,$unique)){
							//provjera jedinstvenosti 
							if(!$this->is_unique($l,$base,$name,$val)){
								$this->set_err_message("Vrijednost atributa $name mora jedinstvena na razini imenika.");
								\aosi\Logger::write("Value of attribute $name should be unique.",6);
								return false;
							}
						}
						if(in_array($name,$this->org_attrs)){
							//atributi vezani uz org zapis se ne smiju mijenjati samo tako.
							$this->set_err_message("Vrijednost atributa $name naslijeđuje se iz org zapisa i ne može se samostalno mijenjati.");
							\aosi\Logger::write("Value of $name is defined in org record and can't be changed.",6);
							return false;
						}
						if(is_array($admin_limits) && is_array($admin_limits[$name])){
							if(!$this->is_allowed_attribute($admin_limits,[$name=>$val])){
								return false;
							}
						}
					}
					if(is_array($data['hredupersonuniquenumber'])){
						foreach($data['hredupersonuniquenumber'] as $val){
							list($pref,$tmp)=explode(':',$val);
							if($pref=='OIB'){
								\aosi\Logger::write("You can't add value OIB of attribute hredupersonuniquenumber.",6);
								$this->set_err_message("Ne možete dodati vrijednost OIB atributa hredupersonuniquenumber.");
								return false;
							}
							foreach($usr_data['hredupersonuniquenumber'] as $v){
								list($pref2,$tmp)=explode(':',$v);
								if($pref==$pref2){
									\aosi\Logger::write("Attribute hredupersonuniquenumber already has value $pref. You can't add another value.",6);
									$this->set_err_message("Atribut hredupersonuniquenumber već ima vrijednost $pref. Ne možete dodati još jednu vrijednost.");
									return false;
								}
							}
						}
					}
					
					$replace_result=$this->add_no_matching_rule_attributes($l,$dn,$data,\aosi\HrEduPerson::class,$usr_data,$callerUid,debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 2)[1]['function']);
				
					if(count($data)>0){
						if(@ldap_mod_add($l,$dn, $data)){
							//todo poziv after add pluginova
							\aosi\Logger::write("Attribute(s) added to LDAP.",4);
							\aosi\ActionLog::write($callerUid,debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 2)[1]['function'],$dn,true,$data);
							return true;
						} else {
							$this->ldap_failed_finish($l,'Dodavanje atributa nije uspjelo.','Adding attribute(s) to LDAP failed.',4,$callerUid,debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 2)[1]['function'],$dn,$data);
							return false;
						}
					} else {
						return $replace_result;
					}
				} else {
					//validacija nije prosla
					\aosi\Logger::write("Validation failed. Attribute is not added.",6);
					\aosi\Logger::write("Validation result: ". \aosi\array2log($res['data'],false),6);
					$this->set_err_message($this->validation_result_to_txt($res['data']));
					return false;
				}
			} else {
				$this->set_err_message("Nema atributa za dodavanje.");
				\aosi\Logger::write("Nothing to add.",6);
				return false;
			}
		}
		
		/*
		Funkcija služi promjeni vrijednosti atributa korisnika iz LDAP imenika. Moze promijeniti vrijednosti vise atributa za korisnika $dn
		Promjena se vrši metodom replace sto znaci da se za neki atribut izbrisu sve stare vrijednosti. Ako atribut ima vise vrijednosti potrebno je u arrayu poslati SVE nove vrijednosti
		Ulazne vrijednosti:
			$l: ldap resurs (bindan kao root)
			$base: base dn imenika
			$data: array vrijednosti atributa se mijenjaju. [[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]]...] 
			$usertype: admin|user tip korisnika koji zove operaciju
			$usr_data: trenutni podaci o korisniku nad cijim se zapisom izvodi operacija iz imenika
			$callerUid: uid pozivatelja funkcije (akcije) - za potrebe pisanja u log
		Izlazna vrijednost: 
			true: atributi promijenjeni
			false: nije uspjela promjena
		*/
		
		private function modifyAtribute($l,$base,$dn,$data,$usertype,$usr_data,$callerUid){
			if(count($data)>0){
				//bazicna validacija vrijednosti atributa za promjenu
				$res=\aosi\HrEduPerson::validate($data,$usertype,false);
				if($res['result']){
					//validacija vrijednosti atributa je uspjesna
					\aosi\Logger::write("Validation successful.",6);
					
					$unique=Config::get('unique');
					foreach($data as $name=>$val){
						if(in_array($name,$unique)){
							//provjera jedinstvenosti 
							if(!$this->is_unique_modify($l,$base,$name,$val,$dn)){
								$this->set_err_message("Vrijednost atributa $name mora jedinstvena na razini imenika.");
								\aosi\Logger::write("Value of attribute $name should be unique.",6);
								return false;
							}
						}
						if(in_array($name,$this->org_attrs)){
							//atributi vezani uz org zapis se ne smiju mijenjati samo tako.
							$this->set_err_message("Vrijednost atributa $name naslijeđuje se iz org zapisa i ne može se samostalno mijenjati.");
							\aosi\Logger::write("Value of $name is defined in org record and can't be changed.",6);
							return false;
						}
					}
					//hredupersonuniquenumber mora sadrzavati OIB
					if(is_array($data['hredupersonuniquenumber'])){
						$oib_found=false;
						if(is_array($data['hredupersonoib'])){
							$oib=$data['hredupersonoib'][0];
						} else {
							$oib=$usr_data['hredupersonoib'][0];
						}
						foreach($data['hredupersonuniquenumber'] as $val){
							list($pref,$tmp)=explode(':',$val);
							if($pref=='OIB'){
								$oib_found=true;
								if($oib=='NONE'){
									\aosi\Logger::write("If hredupersonoib=NONE hredupersonuniquenumber can't contain value OIB.",6);
									$this->set_err_message("Ako je hredupersonoib=NONE, atribut hredupersonuniquenumber ne smije imati vrijednost OIB.");
									return false;
								} elseif("OIB: $oib"!=$val){
									\aosi\Logger::write("Value OIB of hredupersonuniquenumber should be same as value of hredupersonoib.",6);
									$this->set_err_message("Vrijednost OIB atributa hredupersonuniquenumber mora biti jednaka onoj u atributu hredupersonoib.");
									return false;
								}
							}
						}
						if(!$oib_found && $oib!='NONE'){
							\aosi\Logger::write("Value OIB of hredupersonuniquenumber should be same as value of hredupersonoib.",6);
							$this->set_err_message("Vrijednost OIB atributa hredupersonuniquenumber mora biti jednaka onoj u atributu hredupersonoib.");
							return false;
						}
					} elseif(is_array($data['hredupersonoib'])) {
						$data['hredupersonuniquenumber']=[];
						$oib_found=false;
						foreach($usr_data['hredupersonuniquenumber'] as $val){
							list($pref,$tmp)=explode(':',$val);
							if($pref=='OIB'){
								$oib_found=true;
								if($data['hredupersonoib'][0]!='NONE'){
									$data['hredupersonuniquenumber'][]="OIB: " . $data['hredupersonoib'][0];
								}
							} else {
								$data['hredupersonuniquenumber'][]=$val;
							}
						}
						if(!$oib_found && $oib!='NONE'){
							$data['hredupersonuniquenumber'][]="OIB: " . $data['hredupersonoib'][0];
						}
						if(count($data['hredupersonuniquenumber'])==0){
							\aosi\Logger::write("hrEduPersonOIB cannot be set to NONE because OIB is the only value in hrEduPersonUniqueNumber!",6);
							$this->set_err_message("Ne možete postaviti vrijednost atributa hreduperson u NONE ako atribut hredupersonuniquenumber na sadrži neku drugu vrijednost osim vrijednosti OIB.");
							return false;
						}
					}
					//hredupersonaffiliation mora sadrzavati primaryaffiliation
					if(is_array($data['hredupersonaffiliation'])){
						if(is_array($data['hredupersonprimaryaffiliation'])){
							$affiliation=$data['hredupersonprimaryaffiliation'][0];
						} else {
							$affiliation=$usr_data['hredupersonprimaryaffiliation'][0];
						}
						if(!in_array($affiliation,$data['hredupersonaffiliation'])){
							\aosi\Logger::write("hrEduPersonPrimaryAffiliation can be set only to one of the values in hrEduPersonAffiliation!",6);
							$this->set_err_message("Vrijednost atributa hredupersonprimaryaffiliation mora biti jedna od vrijednosti atributa hredupersonaffiliation.");
							return false;
						}
					} elseif(is_array($data['hredupersonprimaryaffiliation'])){
						//hredupersonaffiliation mora sadrzavati primaryaffiliation
						if(!in_array($data['hredupersonprimaryaffiliation'][0],$usr_data['hredupersonaffiliation'])){
							\aosi\Logger::write("hrEduPersonPrimaryAffiliation can be set only to one of the values in hrEduPersonAffiliation!",6);
							$this->set_err_message("Vrijednost atributa hredupersonprimaryaffiliation mora biti jedna od vrijednosti atributa hredupersonaffiliation.");
							return false;
						}
					}
					
					
					//promjena passworda - obratiti paznju tko mijenja
					if(is_array($data['userpassword'])){
						//uvijek resetirati ppolicy
						$this->reset_ppolicy($dn);
						if($usertype=='admin'){
							//ako ga mijenja admin, valja dodati ppolicy
							$data['pwdpolicysubentry']=[Config::get('ppolicy_dn_prefix') . ',' . $base];
						}
						$data['userpassword']=[$this->encode_password($data['userpassword'][0])];
					}
					
					if(@ldap_mod_replace($l,$dn, $data)){
						//todo poziv after add pluginova
						\aosi\Logger::write("Attribute(s) modifyed.",4);
						\aosi\ActionLog::write($callerUid,debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 2)[1]['function'],$dn,true,$data);
						return true;
					} else {
						$this->ldap_failed_finish($l,'Promjena vrijednosti atributa nije uspjela.','Modifying attribute(s) failed.',4,$callerUid,debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 2)[1]['function'],$dn,$data);
						return false;
					}
					
				} else {
					//validacija nije prosla
					\aosi\Logger::write("Validation failed. Attribute is not modifyed.",6);
					\aosi\Logger::write("Validation result: ". \aosi\array2log($res['data'],false),6);
					$this->set_err_message($this->validation_result_to_txt($res['data']));
					return false;
				}
			} else {
				$this->set_err_message("Nema atributa za promjenu.");
				\aosi\Logger::write("Nothing to modify.",6);
				return false;
			}
		}
		
		
		/*Funkcija služi dodavanju korisnika u LDAP imenik.
		Ulazne vrijednosti:
			$l: ldap resurs (bindan kao root)
			$base: base dn imenika
			$data: array vrijednosti s podacima o korisniku [[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]]]
			$admin_limits: ako dodavanje vrsi administrator imenika s ogranicenjem, onda je u ovom parametru array s ogranicenjima ['naziv_atributa'=>['vrijednost1','vrijednost2',..],['']=>['',''..]
			$org_data: podaci iz org zapisa ustanove. Neki atributi u korisnickom zapisu se prepisuju iz org zapisa (o,l,street,postalCode,postalAddress)
			$callerUid: korisnik koji je zvao metodu (za potrebe pisanja u action log)
		Izlazna vrijednost: 
			true: svi korisnici ispravno dodani
			false: bar jedna korisnik nije dodan
		array (u obliku koji vraca funkcija ldap_result_to_array) org zapisa ustanove
		*/
		
		private function addUser($l,$base,$data,$admin_limits,$org_data,$callerUid){
			$dn="uid=". $data['uid'][0] . ",$base";
			\aosi\Logger::write("Adding user $dn Data: ". \aosi\array2log($data,true),4);
			//dodajemo vrijednosti atributa iz org zapisa
			$realm=$this->realm_from_base($base);
			$data=$this->add_atribute_values($realm,$data,$org_data);
			
			$res=\aosi\HrEduPerson::validate($data,'admin',true);
			if($res['result']){
				//validacija vrijednosti atributa je uspjesna
				\aosi\Logger::write("Validation successful.",6);
				
				//ako admin ima ogranicenje, treba provjeriti smije li dodati ovog korisnika. 
				//ako korisnik ima one vrijednosti onih atributa po kojima admin ima ogranicenje, onda smije
				if(is_array($admin_limits)){
					if(!$this->is_allowed_admin($admin_limits,$data)){
						return false;
					}
					if(!$this->is_allowed_attribute($admin_limits,$data)){
						return false;
					}
				}
				//primarna povezanost s ustanovom (hredupersonprimaryaffiliation) mora biti jedna od vrijednosti atributa povezanost s ustanovom (hredupersonaffiliation)
				if(!in_array($data['hredupersonprimaryaffiliation'][0],$data['hredupersonaffiliation'])){
					$this->set_err_message('Vrijednost atributa temeljna povezanost s ustanovom (hrEduPersonPrimaryAffiliation) mora biti jedna od vrijednosti atributa povezanost s ustanovom (hrEduPersonAffiliation).');
					\aosi\Logger::write("Value of attribute hrEduPersonPrimaryAffiliation is not one of the values in hrEduPersonAffiliation!",6);
					return false;
				}
				
				//hrEduPersonOIB smije biti NONE samo ako je primarna povezanost s ustanovom gost
				if($data['hredupersonoib'][0]=='NONE'){
					if($data['hredupersonprimaryaffiliation'][0]!='gost'){
						$this->set_err_message('Vrijednost atributa hrEduPersonOIB smije biti NONE samo ako je temeljna povezanost s ustanovom (hrEduPersonPrimaryAffiliation) gost.');
						\aosi\Logger::write("Value of attribute hrEduPersonOIB can be NONE only if hrEduPersonPrimaryAffiliation is gost.",6);
						return false;
					}
				} else {
				//ako hrEduPersonOIB nije NONE, onda hrEduPersonUniqueNumber mora sadrzavati OIB
					$test=false;
					foreach($data['hredupersonuniquenumber'] as $d){
						if($d=='OIB: ' . $data['hredupersonoib'][0]){
							$test=true;
							break;
						}
					}
					if(!$test){
						$this->set_err_message('Vrijednost atributa hrEduPersonOIB mora biti jedna od vrijednosti atributa hrEduPersonUniqueNumber.');
						\aosi\Logger::write("Value of attribute hrEduPersonOIB should be one of the values in hrEduPersonUniqueNumber.",6);
						return false;
					}
				}
				//zaporka ne smije sadrzavati uid
				if(stristr($data['userpassword'][0],$data['uid'][0])!=false){
					$this->set_err_message('Zaporka ne smije sadržavati korisničku oznaku.');
					\aosi\Logger::write("Password shouldn't contain uid.",6);
					return false;
				}

				//hrEduPersonUniqueID mora biti uid@realm.hr 
				if($data['hredupersonuniqueid'][0]!=$data['uid'][0] . '@' . $realm .'.hr'){
					$this->set_err_message('Vrijednost atributa hrEduPersonUniqueID mora biti uid@realm.hr.');
					\aosi\Logger::write("hrEduPersonUniqueID should be uid@realm.hr.",6);
					return false;
				}
				
				//provjera jedinstvenosti 
				$unique=Config::get('unique');
				$unique[]='uid';
				foreach($unique as $name){
					if(!$this->is_unique($l,$base,$name,$data[$name])){
						$this->set_err_message("Vrijednost atributa $name mora jedinstvena na razini imenika.");
						\aosi\Logger::write("Value of $name should be unique.",6);
						return false;
					}
				}
				//validacija je prosla
				//dodajemo atribute potrebne za dodavanje zapisa
				$data['objectclass']=['person','organizationalPerson','inetOrgPerson','hrEduPerson','schacContactLocation'];
				$data['pwdpolicysubentry']=[Config::get('ppolicy_dn_prefix') . ',' . $base];
				$data['userpassword']=[$this->encode_password($data['userpassword'][0])];
				//todo poziv before add pluginova
				
				if(@ldap_add($l,$dn, $data)){
					//todo poziv after add pluginova
					
					\aosi\Ldif::write_ldif($dn,$data,'add',Config::get('ldif'));
					
					\aosi\Logger::write("User added to LDAP.",4);
					\aosi\ActionLog::write($callerUid,debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 2)[1]['function'],$dn,true,$data);
					return true;
				} else {
					$this->ldap_failed_finish($l,'Dodavanje u LDAP nije uspjelo.','Adding to LDAP failed.',4,$callerUid,debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 2)[1]['function'],$dn,$data);
					return false;
				}
			} else {
				//validacija nije prosla
				\aosi\Logger::write("Validation failed. User is not added.",4);
				\aosi\Logger::write("Validation result: ". \aosi\array2log($res['data'],false),6);
				$this->set_err_message($this->validation_result_to_txt($res['data']));
				return false;
			}
		}
		
		/*
		Funkcija podesava atribute koji se automatski postavljaju za korisnika prilikom dodavanja.
		Atributi o,l,street,postalCode,postalAddress se uzimaju iz org zapisa ustanove
		Ulaz:
			$realm: domena ustanove (bez.hr)
			$data: zapis korisnika
			$org_data: podaci iz org zapisa
		Izlaz:
			array s korisnickim podacima u kojem su podesene sve vrijednosti koje se dodaju automatski.
		*/
		private function add_atribute_values($realm,$data,$org_data){
			foreach($this->org_attrs as $attr){
				$data[$attr][0]=$org_data[$attr][0];
			}
			//hredupersonuniqueid se izvodi iz uida i realma
			$data['hredupersonhomeorg']=[$realm . '.hr'];
			$data['hredupersonuniqueid']=[$data['uid'][0] . '@' . $data['hredupersonhomeorg'][0]];
			$data['hredupersonpersistentid']=[md5($data['hredupersonuniqueid'][0] . $realm . time())];
			return $this->set_OIB($data);
		}
		
		/*
		Funkcija podesava vrijednost atributa OIB u hredupersonuniquenumber sukladno vrijednosti atributa hredupersonoib
		Ulaz:
			$data: array s vrijednostima atributa koji mora minimalno sadrzavati hredupersonuniquenumber i hredupersonoib
		Izlaz:
			ulazni array s podesenom vrijednoscu atributa hredupersonuniquenumber sukladno vrijednosti atributa hredupersonoib
		*/
		private function set_OIB($data){
			$found=false;
			if(is_array($data['hredupersonuniquenumber'])){
				foreach($data['hredupersonuniquenumber'] as $k=>$d){
					if(substr($d,0,4)=='OIB:'){
						$found=true;
						if($data['hredupersonoib'][0]=='NONE'){
							unset($data['hredupersonuniquenumber'][$k]);
						} else {
							$data['hredupersonuniquenumber'][$k]='OIB: ' . $data['hredupersonoib'][0];
						}
					}
				}
				if(!$found && $data['hredupersonoib'][0]!='NONE'){
					$data['hredupersonuniquenumber'][]='OIB: ' . $data['hredupersonoib'][0];
				}
			} elseif($data['hredupersonoib'][0]!='NONE'){
				$data['hredupersonuniquenumber']=['OIB: ' . $data['hredupersonoib'][0]];
			}
			return $data;
		}
				
		/* 
		Funkcija izvrsi anonimni bind na LDAP i vrati ldap resurs
		Ulazni podaci:
			$base: base dn korisnika
		Izlaz:
			ldap resurs ako je korisnik uspjesno autenticiran
			false operacija nije uspjela
		*/
		private function anonBind(){
			if($l = @ldap_connect($this->conf['uri'])){
				ldap_set_option($l, LDAP_OPT_PROTOCOL_VERSION, 3);
				if($ldapbind = @ldap_bind($l)){
					\aosi\Logger::write('Anon bind successful.',8);
					return $l;
				} else {
					$err=$this->get_ldap_error($l);
					$this->set_error($err['code'],$err['message']);
					\aosi\Logger::write('Bind unsuccessful '."uid=$uid,$base. Error: " . $err['code'] . ", " . $err['message'],8);
					return false;
				}
			} else {
				$this->set_error(-18,'Cant connect to LDAP server ' . $this->conf['uri']);
				\aosi\Logger::write($this->err_message(),1);
				return false;
			}			
		}
		
		/* 
		Funkcija izvrsi bind na LDAP kao korisnik i vrati ldap resurs
		Ulazni podaci:
			$uid: uid korisnika u imeniku
			$password: base64 enkodiran password korisnika iz imenika
			$base: base dn korisnika
		Izlaz:
			ldap resurs ako je korisnik uspjesno autenticiran
			false ako korisnik nije autenticiran
		*/
		private function bind($uid,$password,$base){
			if (empty($uid) or empty($password) or empty($base)){
				\aosi\Logger::write('Missing uid, password or base',5);
				$this->set_error(49,'Wrong username or password.');
				return false;
			}
			if($l = @ldap_connect($this->conf['uri'])){
				ldap_set_option($l, LDAP_OPT_PROTOCOL_VERSION, 3);
				ldap_set_option($l, LDAP_OPT_REFERRALS, 0);
				if($ldapbind = @ldap_bind($l, "uid=$uid,$base", base64_decode($password))){
					\aosi\Logger::write('Bind successful '."uid=$uid,$base",8);
					return $l;
				} else {
					$err=$this->get_ldap_error($l);
					$this->set_error($err['code'],$err['message']);
					\aosi\Logger::write('Bind unsuccessful '."uid=$uid,$base. Error: " . $err['code'] . ", " . $err['message'],8);
					return false;
				}
			} else {
				$this->set_error(-18,'Cant connect to LDAP server ' . $this->conf['uri']);
				\aosi\Logger::write($this->err_message(),1);
				return false;
			}			
		}
		
		/* 
		Funkcija služi autentikaciji korisnika pomoću dn-a (npr cn=hreduadmin,dc=test,dc=hr). 
		Funkcija izvrsi bind na LDAP kao korisnik i vrati ldap resurs
		Ulazni podaci:
			$uid: dn korisnika u imeniku
			$password: base64 enkodiran password korisnika iz imenika
			$base: base dn korisnika
		Izlaz:
			ldap resurs ako je korisnik uspjesno autenticiran
			false ako korisnik nije autenticiran
		*/
		private function dnBind($dn,$password){
			if (empty($dn) or empty($password)){
				\aosi\Logger::write('Missing dn or password',5);
				$this->set_error(49,'Wrong dn or password.');
				return false;
			}
			if($l = @ldap_connect($this->conf['uri'])){
				ldap_set_option($l, LDAP_OPT_PROTOCOL_VERSION, 3);
				ldap_set_option($l, LDAP_OPT_REFERRALS, 0);
				if($ldapbind = @ldap_bind($l, $dn, base64_decode($password))){
					\aosi\Logger::write('Bind successful '.$dn,8);
					return $l;
				} else {
					$err=$this->get_ldap_error($l);
					$this->set_error($err['code'],$err['message']);
					\aosi\Logger::write('Bind unsuccessful '."$dn. Error: " . $err['code'] . ", " . $err['message'],8);
					return false;
				}
			} else {
				$this->set_error(-18,'Cant connect to LDAP server ' . $this->conf['uri']);
				\aosi\Logger::write($this->err_message(),1);
				return false;
			}			
		}
		
		//privatna funkcija izvrsi bind na imenik kao root i vrati ldap resurs
		private function rootBind(){
			if($l = @ldap_connect($this->conf['uri'])){
				ldap_set_option($l, LDAP_OPT_PROTOCOL_VERSION, 3);
				if($ldapbind = @ldap_bind($l,$this->conf['rootdn'],$this->conf['pass'])){
					\aosi\Logger::write('Binded as ' . $this->conf['rootdn'],8);
					return $l;
				} else {
					$this->set_error(-18,"Can't bind as " . $this->conf['rootdn']);
					\aosi\Logger::write($this->err_message(),1);
					\aosi\Logger::write("Wrong rootdn or pass in aosi.config!",1);
					return false;
				}
			} else {
				$this->set_error(-18,"Can't connect to LDAP server " . $this->conf['uri']);
				\aosi\Logger::write($this->err_message(),1);
				return false;
			}
		}
		/*Funkcija provjerava ima li osoba administratorska prava.
		Ulazne vrijednosti:
			$dn: dn korisnika u imeniku
			$usrdata: korisnikov zapis iz imenika koji minimalno mora sadrzavati atribut hredupersonrole
		Izlazna vrijednost: 
			0: korisnik nije administrator
			1 - Administator definiran u konfiguracijskoj datoteci, 
			2 - Administrator definiran u imeniku, 
			3 - Administrator iz imenika s ograničenjem
		Ako je izlazna vrijednost 3 (administrator s ogranicenjem), onda metoda result ovog objekta vraca hash s ogranicenjima ['naziv_atributa'=>['vrijednost1','vrijednost2',..],['']=>['',''..]
		*/
		private function isAdmin($dn,$usrdata){
			$this->set_result(false);
			$kword='administrator imenika';
			if(@in_array($kword,$usrdata['hredupersonrole'])){
				\aosi\Logger::write("User $dn is admin from directory",6);
				return 2;
			}
			if(@in_array($dn,$this->conf['admins'])){
				\aosi\Logger::write("User $dn is admin from configuration file",6);
				return 1;
			}
			if(@is_array($usrdata['hredupersonrole'])){
				$kword='urn:'.$kword;
				foreach($usrdata['hredupersonrole'] as $k=>$v){
					if(substr($v,0,strlen($kword))==$kword){
						$limits=substr($v,strlen($kword)+1);
						\aosi\Logger::write("User $dn is limited admin from directory $limits",6);
						$la=explode(':',$limits);
						$i=0;
						while($i<count($la)){
							$result[$la[$i]][]=$la[$i+1];
							$i++;$i++;
						}
						$this->set_result($result);
						return 3;
						break;
					}
				}
			}
			\aosi\Logger::write("User $dn doesn't have admin privileges!",6);
			return 0;
		}
		/*Funkcija provjerava je li acc zakljucan zbog ppolicy-a
		Ulazna vrijednost:
			dn korisnika
		Izlazne vrijednosti:
			true ako je zakljucan
			false ako nije
		
		Po izvrsenju metode funkcija $obj->result() vraca array:
			'authn_left'=>broj preostalih pokusaja autentikacije prije zakljucavanja acc-a,
			'pwd_expires_on'=>vrijeme isteka zaporke u formatu 'd.m.Y. H:i:s',
			'is_exipred'=>je li zaporka istekla 1|0,
			'time_left'=> ako zaporka nije istekla, za koliko vremena istice (formatirano)
			'time_left_s'=> ako zaporka nije istekla, za koliko vremena istice (u sekundama)
		*/
		private function isLockedPpolicy($dn){
			$this->set_result(false);
			$l=$this->rootBind();
			$res=$this->read_simple($l, $dn, "(objectclass=*)", ['pwdpolicysubentry','pwdchangedtime','pwdgraceusetime','pwdLockout']);
			if(is_array($res)){
				$res=reset($res);
				\aosi\Logger::write("dn: $dn, ppolicy attributes: " . array2log($res),6);
				if(is_array($res['pwdpolicysubentry'])){
					$ppolicy=$this->read_simple($l, $res['pwdpolicysubentry'][0], "(objectclass=*)", ['*']);
					if(is_array($ppolicy)){
						$ppolicy=reset($ppolicy);
						return $this->isLocked($dn,$res,$ppolicy);
					} else {
						\aosi\Logger::write("Default ppolicy entry missing dn:" . $res['pwdpolicysubentry'][0],2);
					}
				}
			}
			\aosi\Logger::write("User is not locked dn:" . $dn,8);
			return false;
		}
		
		/*Funkcija provjerava je li acc zakljucan zbog ppolicy-a
		Ulazna vrijednost:
			dn korisnika
			$usr_data: array s podacima o korisniku koji mora sadrzavati vrijednsoti atributa: 'pwdpolicysubentry','pwdchangedtime','pwdgraceusetime','pwdLockout'
			$ppolicy: array s podacima iz zapisa kojim se devinira ppolicy za novog korisnika u imeniku. mora sadrzavati: 'pwdgraceauthnlimit', 'pwdmaxage'
		Izlazne vrijednosti:
			true ako je zakljucan
			false ako nije
		Po izvrsenju metode funkcija $obj->result() vraca array:
			'authn_left'=>broj preostalih pokusaja autentikacije prije zakljucavanja acc-a,
			'pwd_expires_on'=>vrijeme isteka zaporke u formatu 'd.m.Y. H:i:s',
			'is_exipred'=>je li zaporka istekla 1|0,
			'time_left'=> ako zaporka nije istekla, za koliko vremena istice (formatirano)
			'time_left_s'=> ako zaporka nije istekla, za koliko vremena istice (u sekundama)
		*/
		private function isLocked($dn,$usr_data,$ppolicy){
			$this->set_result(false);
			//broj preostalih autentikacija
			$ret['authn_left']=$ppolicy['pwdgraceauthnlimit'][0] - @count($usr_data['pwdgraceusetime']);
			if(!empty($usr_data['pwdchangedtime'][0]) && !empty($ppolicy['pwdmaxage'][0])){
				$ret['pwd_expires_on']=date('d.m.Y. H:i:s',strtotime($usr_data['pwdchangedtime'][0])+$ppolicy['pwdmaxage'][0]);
				if(strtotime($usr_data['pwdchangedtime'][0])+$ppolicy['pwdmaxage'][0]>time()){
					$ret['is_exipred']=0;
					$ret['time_left_s']=strtotime($usr_data['pwdchangedtime'][0])+$ppolicy['pwdmaxage'][0]-time();
					$ret['time_left']=$this->sec_to_time($ret['time_left_s']);
					$this->set_result($ret);
					\aosi\Logger::write("Account will be locked in " . $ret['time_left'] . " dn:$dn",6);
					return false;
				} else {
					$ret['is_exipred']=1;
					$this->set_result($ret);
					if($ret['authn_left']<=0){
						\aosi\Logger::write("Account locked due to the password policy restrictions dn:$dn",6);
						return true;
					} else {
						\aosi\Logger::write("Password expired on "  . $ret['pwd_expires_on'] ." authn left before locking " . $ret['authn_left'] ." dn:$dn",6);
						return false;
					}
				}
			}
			$this->set_result($ret);
			return false;
		}
		
		/* funkcija brise poveznicu korisnika i politike obavezne promjene zaporke
		Ulaz:
			$dn: dn korisnika u imeniku,
		*/
		private function reset_ppolicy($dn){
			$l=$this->rootBind();
			$ok=true;
			//$attrs=['pwdpolicysubentry','pwdaccountlockedtime','pwdChangedTime','pwdGraceUseTime'];
			$ret=@ldap_mod_del($l,$dn,['pwdpolicysubentry'=>[]]);
			if($ret){
				\aosi\Logger::write("pwdpolicysubentry deleted for $dn.",8);
			} else {
				$err=$this->get_ldap_error($l);
				\aosi\Logger::write("Failed to delete pwdpolicysubentry for user $dn. Error: " . $err['code'] . ", " . $err['message'],4);
				$ok&=false;
			}
			$ret=@ldap_mod_replace($l,$dn, ['pwdreset'=>['FALSE']]);
			if($ret){
				\aosi\Logger::write("pwdReset set for $dn.",8);
			} else {
				$err=$this->get_ldap_error($l);
				\aosi\Logger::write("Failed to set pwdReset for user $dn. Error: " . $err['code'] . ", " . $err['message'],4);
				$ok&=false;
			}
			return $ok;
		}
		
		
		/*
		Napravi search u LDAP imeniku i vrati rezultat. Koristi se ako se kao rezultat ocekuje vise zapisa
		Ulazni podaci:
			$ldap: resource identifyer konekcije na ldap imenik
			$base: base-dn stabla na kojem se radi pretraga
			$filter: ldap filter pretrage
			$attrs: array atributa koji se dohvacaju 
			$conv_to_array: true (default) | false - ako je false vraca se array koji vrati ldapsearch, ako je true vraca se prilagodeni array
			$show_binary: true  | false (default) - ako je true za binarne atribute vraca njihovu base_64 encodiranu vrijednost, ako je false vraca md5 hash vrijednosti (uvazava se samo ako je ako je $conv_to_array: true)
			$md5: md5 vrijednosti atributa koja se zeli dohvatiti. Koristi se iskljucivo s binarnim atributima i primarno sluzi izboru zeljene vrijednosti kod atributa s vise vrijednosti. Parametar je opcionalan.
		Izlaz:
			ovisno o parametru $conv_to_array ili array koji vraca ldap_get_entries ili prilagodjeni koji vraca funkcija ldap_result_to_array
		*/
		private function search($ldap, $base, $filter, $attrs,$conv_to_array=true,$show_binary=false,$md5=false){
			if(!is_array($attrs)){
				\aosi\Logger::write("Missing attributes list.",4);
				$this->set_error(-65,"Missing attributes list.");
				return false;
			}
			$sr=@ldap_search($ldap, $base, $filter, $attrs);
			$res=@ldap_get_entries($ldap, $sr);
			\aosi\Logger::write("Search bdn:$base, filter:$filter returned " . $res['count'] . ' entries',8);
			if($res['count']>0){
				if($conv_to_array){
					return $this->ldap_result_to_array($res,$show_binary,$md5);
				} else {
					return $res;
				}
			}
			return false;
		}
		
		/*
		Napravi search u LDAP imeniku i vrati rezultat. Koristi se ako se kao rezultat ocekuje vise zapisa
		Ulazni podaci:
			$ldap: resource identifyer konekcije na ldap imenik
			$base: base-dn stabla na kojem se radi pretraga
			$filter: ldap filter pretrage
			$attrs: array atributa koji se dohvacaju 
		Izlaz:
			 prilagodjeni koji vraca funkcija ldap_result_to_array
		*/
		private function search_simple($ldap, $base, $filter, $attrs){
			if(!is_array($attrs)){
				\aosi\Logger::write("Missing attributes list.",4);
				$this->set_error(-65,"Missing attributes list.");
				return false;
			}
			$sr=@ldap_search($ldap, $base, $filter, $attrs);
			$res=@ldap_get_entries($ldap, $sr);
			\aosi\Logger::write("Search bdn:$base, filter:$filter returned " . $res['count'] . ' entries',8);
			if($res['count']>0){
				return $this->simple_ldap_result_to_array($res);
			}
			return false;
		}
		
		/*
		Napravi search u LDAP imeniku i vrati rezultat. Koristi se ako se kao rezultat dohvaca jedan zapis
		Ulazni podaci:
			$ldap: resource identifyer konekcije na ldap imenik
			$base: base-dn stabla na kojem se radi pretraga (idealno dn zapisa koji se dohvaca)
			$filter: ldap filter pretrage (ako je za $base dan dn, onda ovdje objectclass=*
			$attrs: array atributa koji se dohvacaju 
			$conv_to_array: true (default) | false - ako je false vraca se array koji vrati ldap_get_entries, ako je true vraca se prilagodeni array
			$show_binary: true  | false (default) - ako je true za binarne atribute vraca njihovu base_64 encodiranu vrijednost, ako je false vraca md5 hash vrijednosti (uvazava se samo ako je ako je $conv_to_array: true)
			$md5: md5 vrijednosti atributa koja se zeli dohvatiti. Koristi se iskljucivo s binarnim atributima i primarno sluzi izboru zeljene vrijednosti kod atributa s vise vrijednosti. Parametar je opcionalan.
		Izlaz:
			ako nije nadjen nijedan zapis = false
			ako je nadjen bar jedan zapis ovisno o parametru $conv_to_array ili array koji vraca ldap_get_entries ili prilagodjeni koji vraca funkcija ldap_result_to_array
		*/
		private function read($ldap, $base, $filter, $attrs,$conv_to_array=true,$show_binary=false,$md5=false){
			if(!is_array($attrs)){
				\aosi\Logger::write("Missing attributes list.",4);
				$this->set_error(-65,"Missing attributes list.");
				return false;
			}
			$sr=@ldap_read($ldap, $base, $filter, $attrs);
			$res=@ldap_get_entries($ldap, $sr);
			\aosi\Logger::write("Read bdn:$base, filter:$filter returned " . $res['count'] . ' entries',8);
			if($res['count']>0){
				if($conv_to_array){
					return $this->ldap_result_to_array($res,$show_binary,$md5);
				} else {
					return $res;
				}
			}
			return false;
		}
		
		/*
		Napravi search u LDAP imeniku i vrati rezultat. Koristi se ako se kao rezultat dohvaca jedan zapis
		Ulazni podaci:
			$ldap: resource identifyer konekcije na ldap imenik
			$base: base-dn stabla na kojem se radi pretraga (idealno dn zapisa koji se dohvaca)
			$filter: ldap filter pretrage (ako je za $base dan dn, onda ovdje objectclass=*
			$attrs: array atributa koji se dohvacaju 
		Izlaz:
			ako nije nadjen nijedan zapis = false
			ako je nadjen bar jedan zapis array koji vraca ldap_get_entries ili prilagodjeni koji vraca funkcija ldap_result_to_array
		*/
		private function read_simple($ldap, $base, $filter, $attrs){
			if(!is_array($attrs)){
				\aosi\Logger::write("Missing attributes list.",4);
				$this->set_error(-65,"Missing attributes list.");
				return false;
			}
			$sr=@ldap_read($ldap, $base, $filter, $attrs);
			$res=@ldap_get_entries($ldap, $sr);
			\aosi\Logger::write("Read bdn:$base, filter:$filter returned " . $res['count'] . ' entries',8);
			if($res['count']>0){
				return $this->simple_ldap_result_to_array($res);
			}
			return false;
		}
		
		/*
		Pretvara array koji vrati ldapsearch u array bez varijabli count i ostalih smetala
		izlazni array je oblika
		[ [dn]=>[[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]]...]
		Ulazni podaci:
			array koji vrati ldap_get_entries
			$show_binary: true  | false (default) - ako je true za binarne atribute vraca njihovu base_64 encodiranu vrijednost, ako je false vraca md5 hash vrijednosti (uvazava se samo ako je ako je $conv_to_array: true)
			$md5: md5 vrijednosti atributa koja se zeli dohvatiti. Koristi se iskljucivo s binarnim atributima i primarno sluzi izboru zeljene vrijednosti kod atributa s vise vrijednosti. Parametar je opcionalan.
		Izlaz:
			array oblika
		[ [dn]=>[[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]]...]
		*/
		private function ldap_result_to_array($res,$show_binary=false,$md5=false){
			$ret=array();
			for($i=0;$i<$res['count'];$i++){
				$ret[$res[$i]['dn']]=[];
				for($j=0;$j<$res[$i]['count'];$j++){
					$key=$res[$i][$j]=='homephone' ? 'hometelephonenumber' : $res[$i][$j];
					unset($res[$i][$res[$i][$j]]['count']);
					if($this->is_binary($key)) {
						$r=$this->encode_binary($res[$i][$key],$show_binary,$md5);
						if(is_array($r)){
							if(count($r)>0){
								$ret[$res[$i]['dn']][$key]=$r;
							}
						}
					} else {
						$ret[$res[$i]['dn']][$key]=$res[$i][$res[$i][$j]];
					}
				}
			}
			return $ret;
		}
		
		/*
		Pretvara array koji vrati ldapsearch u array bez varijabli count i ostalih smetala
		izlazni array je oblika
		[ [dn]=>[[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]]...]
		Ulazni podaci:
			array koji vrati ldap_get_entries
		Izlaz:
			array oblika
		[ [dn]=>[[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]]...]
		*/
		private function simple_ldap_result_to_array($res){
			$ret=array();
			for($i=0;$i<$res['count'];$i++){
				$ret[$res[$i]['dn']]=[];
				for($j=0;$j<$res[$i]['count'];$j++){
					$key=$res[$i][$j]=='homephone' ? 'hometelephonenumber' : $res[$i][$j];
					unset($res[$i][$res[$i][$j]]['count']);
					$ret[$res[$i]['dn']][$key]=$res[$i][$res[$i][$j]];
				}
			}
			return $ret;
		}
		
		/*
		Pretvara vrijeme u sekundama u formatirano vrijeme
		
		Ulazni podaci:
			vrijeme u seksundama
		Izlaz:
			vrijeme oblika X dana Y h Z min Q sek
		*/
		private function sec_to_time($s){
			$min=60;
			$hour=$min*60;
			$day=$hour*24;
			
			if($s>$day){
				$ret=intdiv($s,$day);
				$ret.= $ret>1 ? " dana " : " dan ";
				$s=$s % $day;
			}
			$ret.=intdiv($s,$hour) . " h ";
			$s=$s % $hour;
			$ret.=intdiv($s,$min) . " min ";
			$s=$s % $min;
			$ret.="$s sek";
			return $ret;
		}
		
		/*
		Funkcija od ulaznog arraya napravi LDAP filter sa zadanim operatorom
		Ako atribut ima više vrijednosti, među njima je operator ILI
		Ulaz:
			$filter_array: array oblika: [[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]]...]
			$operator: LDAP operator među razlicitim atributima
			$append: postojeci filter koji se appenda na kraj novonastalog
		*/
		private function mk_ldap_filter($filter_array, $operator='&',$append=''){
			$ret="($operator";
			foreach($filter_array as $key=>$val){
				if(count($val)>1){
					$ret.='(|';
					foreach($val as $v){
						$ret.="($key=$v)";
					}
					$ret.=')';
				} else {
					$ret.="($key=" . $val[0] .")";
				}
			}
			return $ret . $append .")";
		}
		
		/* funkcija provjerava smije li admin s ogranicenjem pristupiti pojedinom korisniku
		Ulaz:
			$admin_limits: array s ogranicenjima administratora ['naziv_atributa'=>['vrijednost1','vrijednost2',..],['']=>['',''..]
			$data: podaci korisnika iz LDAP imenika. trebaju sadrzavati vrijednosti atributa po kojima je moguce ograniciti administraciju imenika (ou i hredupersonaffiliation)
		Izlaz:
			true | false
		*/
		private function is_allowed_admin($admin_limits,$data){
			\aosi\Logger::write("limits: " . \aosi\array2log($admin_limits,false) . " data: " . \aosi\array2log($data,false),8);
			//ako admin ima ogranicenje, treba provjeriti smije handlati ovog korisnika. 
			//ako korisnik ima one vrijednosti onih atributa po kojima admin ima ogranicenje, onda smije
			foreach($admin_limits as $name=>$limits){
				$allowed=false;
				if(isset($data[$name])){
					foreach($limits as $limit){
						if(is_array($data[$name])){
							if(in_array($limit,$data[$name])){
								$allowed=true;
							}
						}
					}
				}
				if(!$allowed){
					$this->set_err_message('Korisnik nema postavljenu vrijednost ' . implode(' ili ', $limits) .' atributa ' . $name .'. Da bi administrator imenika koji ima postavljeno ograničenje pristupa imeniku ' . $name .': ' . implode(' ili ', $limits) . ' imao ovlasti nad korisnikom, korisniku mora biti postavljena odgovarajuca vrijednost navedenog atributa.');
					\aosi\Logger::write("Admin is not allowed to handle this user due to restriction $name: " . \aosi\array2log($limits,false) . ".",4);
					return false;
				}
			}
			\aosi\Logger::write("Admin is allowed to handle this user.",8);
			return true;
		}
		
		/* funkcija provjerava smije li admin s ogranicenjem promijeniti / dodati vrijednost atributa
		ako admin ima postavljeno ogranicenje na nekom atributu smije dodati / promijeniti vrijednost tog atributa samo u onu koja pise u njegovom ogranicenju
		Ulaz:
			$admin_limits: array s ogranicenjima administratora ['naziv_atributa'=>['vrijednost1','vrijednost2',..],['']=>['',''..]
			$data: podaci korisnika iz LDAP imenika trebaju sadrzavati vrijednosti atributa po kojima je moguce ograniciti administraciju imenika (ou i hredupersonaffiliation)
		Izlaz:
			true | false
		*/
		private function is_allowed_attribute($admin_limits,$data){
			\aosi\Logger::write("limits: " . \aosi\array2log($admin_limits,false) . " data: " . \aosi\array2log($data,false),8);
			//ako admin ima ogranicenje, treba provjeriti smije promijeniti vrijednost atributa. 
			//ako mijenja vrijednost atributa po kojem admin ima ograničenje u onu po kojoj je ogranicen, onda smije
			$allowed=true;
			foreach(Config::get('admin_limits_attributes') as $name){
				if(is_array($data[$name]) && is_array($admin_limits[$name])){
					foreach($data[$name] as $v){
						//ako admin ima postavljeno ogranicenje po nekom atributu, u podacima mora biti tocno ta vrijednost na koju admin ima postavljeno ogranicenje
						if(!in_array($v,$admin_limits[$name])){
							$allowed&=false;
							\aosi\Logger::write("Due to admin restrictions, admin isn't allowed to change value of attribute $name to $v.",6);
							$this->append_err_message("Zbog svojih ograničenja administrator nema pravo promijeniti vrijednot $v atributa $name.");
						}
					}
				}
			}
			return $allowed;
		}		
		
		/*
		Funkcija na zadani filter doda ogranicenja administratora zadana finkcijom isAdmin
		Uz to u filter dodaje uvjet (objectclass=hreduperson) kako bi se prikazali samo ljudi iz imenika
		Ulaz:
			$limits: array koji funkcija isAdmin postavi u result oblika: [[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]]...]
			$append: postojeci filter koji se appenda na kraj novonastalog
		*/
		private function filter_from_limits($filter,$limits=[]){
			//$limits['objectclass']=['hreduperson'];
			return $this->mk_ldap_filter($limits,'&',$filter);
		}
		
		/*
		Funkcija na provjerava smije li određena kategorija osobe vidjeti / promijeniti vrijednost nekog atributa iz hrEduPerson sheme
		Ulaz:
			$usertype: admin | user | guest 
			$action: view | change
			$attributes: array atributa za provjeru
		Izlaz:
			true | false (ako ne smije bar jedan)
		*/
		
		private function is_allowed_person($usertype,$action,$attributes){
			$allowed=true;
			if($action=='view'){$txt='vidjeti';} else {$txt='promijeniti';}
			foreach($attributes as $attr){
				if(is_object(HrEduPerson::$attribute[$attr])){
					if(!(HrEduPerson::$attribute[$attr]->is_allowed($usertype,$action))){
						$allowed=false;
						\aosi\Logger::write("$usertype is not allowed to $action value of attribute $attr.",4);
						$this->append_err_message("Niste ovlašteni $txt vrijednost atributa " . $attr .".");
					}
				} else {
					\aosi\Logger::write("Attribute $attr doesn't exist..",4);
					$this->append_err_message("$attr ne postoji u shemi.");
					return false;
				}
			}
			return $allowed;
		}
		
		/*
		Funkcija na provjerava smije li određena kategorija osobe vidjeti / promijeniti vrijednost nekog atributa iz hrEduOrg sheme
		Ulaz:
			$usertype: admin | user | guest 
			$action: view | change
			$attributes: array atributa za provjeru
		Izlaz:
			true | false (ako ne smije bar jedan)
		*/
		
		private function is_allowed_org($usertype,$action,$attributes){
			$allowed=true;
			if($action=='view'){$txt='vidjeti';} else {$txt='promijeniti';}
			foreach($attributes as $attr){
				if(is_object(HrEduOrg::$attribute[$attr])){
					if(!(HrEduOrg::$attribute[$attr]->is_allowed($usertype,$action))){
						$allowed=false;
						\aosi\Logger::write("$usertype is not allowed to $action value of attribute $attr.",4);
						$this->append_err_message("Niste ovlašteni $txt vrijednost atributa " . $attr .".");
					}
				} else {
					\aosi\Logger::write("Attribute $attr doesn't exist..",4);
					$this->append_err_message("$attr ne postoji u shemi.");
					return false;
				}
			}
			return $allowed;
		}
		
		/*
		Funkcija na provjerava je li atribut binarni za array atributa
		Ulaz:
			$attributes: array atributa za provjeru
		Izlaz:
			true | false (ako bar jedan nije)
		*/
		private function is_binary_arr($attributes){
			foreach($attributes as $attr){
				if(!$this->is_binary($attr)){
					return false;
				}
			}
			return true;
		}
		
		/*
		Funkcija na provjerava je li atribut binarni
		Ulaz:
			$attr: naziv atributa za provjeru
		Izlaz:
			true | false
		*/
		private function is_binary($attr){
			$attr=strtolower($attr);
			if(is_object(HrEduPerson::$attribute[$attr])){
				if(\aosi\HrEduPerson::$attribute[$attr]->type=='binary'){
					return true;
				}
			}
			return false;
		}
		
		/*
		Funkcija vraca base_64 enkodiranu vrijednost ili md5 vrijednosti ovisno o parametru show_binary
		Ulaz:
			$data: array vrijednosti atributa
		Izlaz:
			array enkodiranih vrijednosti (md5 ili base64 ovisno o parametrima)
		*/
		
		private function encode_binary($data,$show_binary,$md5=false){
			$ret=[];
			if(is_array($data)){
				foreach($data as $d){
					$m=md5($d);
					if(!$md5 || $m==$md5){
						if($show_binary){
							$ret[]=base64_encode($d);
						} else {
							$ret[]=$m;
						}
					}
				}
			}
			return $ret;
		}
		
		
		/*funkcija provjerava je li vrijednost atributa jedinstvena u imeniku.
		
		Ulaz: 
			$l: ldap resurs
			$base: base dn
			$name: naziva tributa
			$val: aray vrijednosti atributa ['vrijednost1',''vrijednost2'....]
		Izlaz:
			true | false
		*/
		
		private function is_unique($l,$base,$name, $val){
			foreach($val as $k=>$v){
				if($v=='NONE'){
					unset($val[$k]);
				}
			}
			if(count($val)>0){
				if($this->search($l, $base, $this->mk_ldap_filter([$name=>$val], '&','(objectclass=hreduperson)'), ['uid'],false)){
					return false;
				}
			}
			return true;
		}	
		/*funkcija provjerava je li vrijednost atributa jedinstvena u imeniku.
		od funkcije is_unique je razlicita utoliko sto iz pretrage iskljucuje korisnika za kojeg se radi provjera
		
		Ulaz: 
			$l: ldap resurs
			$base: base dn
			$name: naziva tributa
			$val: aray vrijednosti atributa ['vrijednost1',''vrijednost2'....]
			$dn: dn korisnika za kojeg se vrsi provjera
		Izlaz:
			true | false
		*/
		
		private function is_unique_modify($l,$base,$name, $val,$dn){
			$uid=$this->uid_from_dn($dn);
			if($this->search($l, $base, $this->mk_ldap_filter([$name=>$val], '&','(&(objectclass=hreduperson)(!(uid=' . $uid . ')))'), ['uid'],false)){
				return false;
			} else {
				return true;
			}
		}
		
		private function realm_from_base($base){
			list($realm,$tmp)=explode(',',$base);
			$realm=trim($realm);
			return substr($realm,3);
		}
		
		private function uid_from_dn($dn){
			$tmp=explode(',',$dn);
			$tmp[0]=trim($tmp[0]);
			return substr($tmp[0],4);
		}
		
		private function to_ascii($str){
			 $trans = [
      // Croatian letters
      '0106' => 'C', // TJ
      '0107' => 'c', // tj
      '010C' => 'C', // CH
      '010D' => 'c', // ch
      '0110' => 'Dj', // DJ
      '0111' => 'dj', // dj
      '0160' => 'S', // SH
      '0161' => 's', // sh
      '017D' => 'Z', // ZH
      '017E' => 'z', // zh
      // German letters
      '00C4' => 'Ae',  // AE
      '00E4' => 'ae',  // ae
      '00D6' => 'Oe',  // OE
      '00F6' => 'oe',  // oe
      '00DC' => 'Ue',  // UE
      '00FC' => 'ue',  // ue
      '00DF' => 'ss',  // ss
      ];
      foreach($trans as $k=>$v){
      	$str=preg_replace('/\x' . $k . '/', $v, $str);
		  }
		  return $str;
		}
		
		private function validation_result_to_txt($data){
			$ret='';
			foreach($data as $k=>$v){
				if(!$v['result']){
					$ret.=$v['message'];
				}
			}
			return $ret;
		}
		
		private function encode_password($password){
			 return '{SHA}' . base64_encode(sha1($password,true));
		}
		
		private function get_ldap_error($l){
			ldap_get_option($l, LDAP_OPT_DIAGNOSTIC_MESSAGE, $err);
			ldap_get_option($l, LDAP_OPT_ERROR_NUMBER, $code);
			$err.=ldap_error($l);
			return ['message'=>$err,'code'=>$code];
		}
		
		/* funkcija postavi poruku i kod greske, upise log i action log za neuspjelu operaciju u ldapu
		Ulaz:
			$l: link na LDAP
			$err_message: poruka o gresci na koju se konkatenira LDAP-ova poruka o gresci
			$log_message: poruka o gresci za pisanje u log. na nju se konkatenira LDAP-ova poruka o gresci
			$log_severity: ozbiljnost poruke za pisanje u log
			$callerUid: uid pozivatelja funkcije (akcije) na koju se zapis odnosi
			$methodName: naziv pozvane funkcije (akcije)
			$subjectDn: dn subjekta akcije
			$errorCode: kod greske
			$data: array u kojem su kljucevi atributi akcije koja nije uspjela
		*/
		private function ldap_failed_finish($l,$err_message,$log_message,$log_severity,$callerUid,$methodName,$subjectDn,$data=false){
			$err=$this->get_ldap_error($l);
			$this->set_error($err['code'], $err_message . " " . $err['message']);
			\aosi\Logger::write("$log_message Code: " . $err['code'] . ", " . $err['message'] ,$log_severity);
			\aosi\ActionLog::write($callerUid,$methodName,$subjectDn,$err['code'],$data);
		}
	}
}

?>