<?php
namespace aosi {

class Ldif	{
	protected $options = array(
        'sort'    => true,
        'version' => 1,
        'wrap'    => 78
    );
	protected $versionWritten = false;

	protected function __construct(array $options = array()){
  	$this->options = array_merge($this->options, $options);
  }
  
 	public static function write_ldif($dn,$data,$action,$filename){
      $encoder = new static();
      $txt="dn: $dn" . PHP_EOL;
      $done=false;
    	switch($action){
    		case 'addUser': 
    			$txt.="changetype: add" . PHP_EOL . $encoder->_encode($data) . PHP_EOL;
    			$done=true;
    			break;
    		case 'deleteUser':
    			$txt.="changetype: delete" . PHP_EOL;
    			$done=true;
    			break;
    		case 'deleteAttribute':
    			$action='delete';
    			break;
    		case 'addAttribute':
    			$action='add';
    			break;
    		case 'modifyAttribute':
    			$action='replace';
    			break;
    		default:
    			return;
    	}
    	if(!$done){
    		$txt.="changetype: modify" . PHP_EOL;
    		$txt.=$encoder->handle_attributes($action,$data,$encoder);
    	}
      return @file_put_contents($filename, $txt . PHP_EOL,FILE_APPEND);
  }
  
	protected function handle_attributes($action,$data,$encoder){
		$txt ="";
		foreach($data as $name=>$value){
			$txt.="$action: $name" . PHP_EOL;
			if(count($value)>0){
				$txt.=$encoder->_encode([$name=>$value]). PHP_EOL;
			}
			$txt.='-' . PHP_EOL;
		}
		$txt=trim($txt, "-" . PHP_EOL);
		 return $txt . PHP_EOL;
	}
  
  public static function encode($value, array $options = array())
    {
        $encoder = new static($options);

        return $encoder->_encode($value);
    }


    protected function _encode($value)
    {
        if (is_scalar($value)) {
            return $this->encodeString($value);
        } elseif (is_array($value)) {
            return $this->encodeAttributes($value);
        }

        return;
    }

    /**
     * Encodes $string according to RFC2849
     *
     * @link http://www.faqs.org/rfcs/rfc2849.html
     *
     * @param  string  $string
     * @param  bool $base64
     * @return string
     */
    protected function encodeString($string, &$base64 = null)
    {
        $string = (string) $string;
        if (!is_numeric($string) && empty($string)) {
            return '';
        }

        /*
         * SAFE-INIT-CHAR = %x01-09 / %x0B-0C / %x0E-1F /
         *                  %x21-39 / %x3B / %x3D-7F
         *                ; any value <= 127 except NUL, LF, CR,
         *                ; SPACE, colon (":", ASCII 58 decimal)
         *                ; and less-than ("<" , ASCII 60 decimal)
         *
         */
        $unsafeInitChar = array(0, 10, 13, 32, 58, 60);
        /*
         * SAFE-CHAR      = %x01-09 / %x0B-0C / %x0E-7F
         *                ; any value <= 127 decimal except NUL, LF,
         *                ; and CR
         */
        $unsafeChar = array(0, 10, 13);

        $base64 = false;
        for ($i = 0, $len = strlen($string); $i < $len; $i++) {
            $char = ord(substr($string, $i, 1));
            if ($char >= 127) {
                $base64 = true;
                break;
            } elseif ($i === 0 && in_array($char, $unsafeInitChar)) {
                $base64 = true;
                break;
            } elseif (in_array($char, $unsafeChar)) {
                $base64 = true;
                break;
            }
        }
        // Test for ending space
        if (substr($string, -1) == ' ') {
            $base64 = true;
        }

        if ($base64 === true) {
            $string = base64_encode($string);
        }

        return $string;
    }

    /**
     * Encodes an attribute with $name and $value according to RFC2849
     *
     * @link http://www.faqs.org/rfcs/rfc2849.html
     *
     * @param  string       $name
     * @param  array|string $value
     * @return string
     */
    protected function encodeAttribute($name, $value)
    {
        if (!is_array($value)) {
            $value = array($value);
        }

        $output = '';

        if (count($value) < 1) {
            return $name . ': ';
        }

        foreach ($value as $v) {
            $base64    = null;
            $v         = $this->encodeString($v, $base64);
            $attribute = $name . ':';
            if ($base64 === true) {
                $attribute .= ': ' . $v;
            } else {
                $attribute .= ' ' . $v;
            }
            if (isset($this->options['wrap']) && strlen($attribute) > $this->options['wrap']) {
                $attribute = trim(chunk_split($attribute, $this->options['wrap'], PHP_EOL . ' '));
            }
            $output .= $attribute . PHP_EOL;
        }

        return trim($output, PHP_EOL);
    }

    /**
     * Encodes a collection of attributes according to RFC2849
     *
     * @link http://www.faqs.org/rfcs/rfc2849.html
     *
     * @param  array $attributes
     * @return string
     */
    protected function encodeAttributes(array $attributes)
    {
        $string     = '';
        $attributes = array_change_key_case($attributes, CASE_LOWER);
        if (!$this->versionWritten && array_key_exists('dn', $attributes) && isset($this->options['version'])
            && array_key_exists('objectclass', $attributes)
        ) {
            $string .= sprintf('version: %d', $this->options['version']) . PHP_EOL;
            $this->versionWritten = true;
        }

        if (isset($this->options['sort']) && $this->options['sort'] === true) {
            ksort($attributes, SORT_STRING);
            if (array_key_exists('objectclass', $attributes)) {
                $oc = $attributes['objectclass'];
                unset($attributes['objectclass']);
                $attributes = array_merge(array('objectclass' => $oc), $attributes);
            }
            if (array_key_exists('dn', $attributes)) {
                $dn = $attributes['dn'];
                unset($attributes['dn']);
                $attributes = array_merge(array('dn' => $dn), $attributes);
            }
        }
        foreach ($attributes as $key => $value) {
            $string .= $this->encodeAttribute($key, $value) . PHP_EOL;
        }

        return trim($string, PHP_EOL);
    }
}

}
	
?>