<?php
namespace aosi{
	
	class Schema {
		static private $conf;
		
		public static function load($type){
			self::$conf=Config::get('schema');
			if(file_exists(self::$conf['cache']. $type .'.obj')){
				\aosi\Logger::write(self::$conf['cache']. $type .'.obj existis. Unserializing.' ,8);
				$attribute=@unserialize(@file_get_contents(self::$conf['cache']. $type .'.obj'));
			}
			if(!is_array($attribute)){
				//ne postoji serializirana kopija atributa, treba procitati xml
				\aosi\Logger::write(self::$conf['cache']. $type . ".obj doesn't existis." ,8);
				$attribute=self::readSchema(self::$conf[$type]);
				self::save($attribute,$type);
			}
			return $attribute;
		}
		
		private static function readSchema ($schemaxml){
			\aosi\Logger::write('Reading schema from ' . $schemaxml ,8);
			if(!file_exists($schemaxml)){
				\aosi\Logger::write("Schema xml file $schemaxml dosen't exists. Exiting!" ,1);
				exit();
			}
			$p = xml_parser_create('UTF-8');
			xml_parser_set_option($p, XML_OPTION_CASE_FOLDING, 0);
		  xml_parser_set_option($p, XML_OPTION_SKIP_WHITE, 1);
			xml_parse_into_struct($p, @file_get_contents($schemaxml), $vals,$index);
			if(xml_get_error_code ( $p )){
				\aosi\Logger::write("Error parsing $schemaxml! " . xml_error_string(xml_get_error_code($p)) . " line: " . xml_get_current_line_number($p) ." col: " . xml_get_current_column_number($p),1);
				exit();
			}
			\aosi\Logger::write("Schema xml file $schemaxml parsed." ,7);
			xml_parser_free($p);
			$ind=$index['attribute'];
			unset($index);
			$i=0;
			$attribute=[];
			while($i<count($ind)){
				if($vals[$ind[$i]]['tag']=='attribute' && $vals[$ind[$i]]['type']=='open'){
					$ldapname=$vals[$ind[$i]]['attributes']['ldapname'];
					$start=$ind[$i];
					while($vals[$ind[$i]]['type']!='close' && $i<count($ind)){
						$i++;
					}
					if($vals[$ind[$i]]['type']!='close'){
						\aosi\Logger::write("Schema xml file $schemaxml is corrupted 1. Exiting!" ,1);
						exit();
					} else {
						$end=$ind[$i];
						$attr_def_array=@array_slice($vals,$start,$end-$start);
						\aosi\Logger::write("Initializing attribute $ldapname.",8);
						$attribute[strtolower($ldapname)]=new Attribute($attr_def_array);
						$i++;
					}
				} else {
					//print_r($this);
					\aosi\Logger::write("Schema xml file $schemaxml is corrupted 2. Exiting!" ,1);
					exit();
				}
			}
			return $attribute;
		}
		
		private static function save($attribute,$type){
			\aosi\Logger::write("Saving attribute array to " . self::$conf['cache']. $type .'.obj' ,8);
			file_put_contents(self::$conf['cache']. $type .'.obj',serialize($attribute));
		}
		
		//vraca array atributa binarnog tipa
		public static function binary_attrs($attributes){
			$ret=[];
			foreach($attributes as $name=>$attr){
				if($attr->type=='binary'){
					$ret[]=$name;
				}
			}
			return $ret;
		}
		//vraca array atributa nad kojima tip korisnika $usertype (admin|user|guest) smije provesti akciju $action (view|change) ako je $must true, funkcija vraca samo obavezne atribute
		
		public static function allowed_attrs($usertype,$action,$attributes,$must=false){
			$ret=[];
			foreach($attributes as $name=>$val){
				if($val->{$action}[$usertype]=='true' && (!$must || ($must && $val->must=='true'))){
					$ret[]=$name;
				}
			}
			return $ret;
		}
		/* Izvrsava provjeru (validaciju, autorizaciju) vrijednosti podataka za sve atribute u shemi na način da poziva odgovarajuce metode pojedinog atributa.
		Ova metoda ne moze provjeriti medjuovisnosti atributa vec samo validira njihove vrijednosti i provjerava smije li pojedini kategorija korisnika (admin|user|guest) promijeniti sadrzaj atributa.
		Ulaz:
			$usertype: admin|user|guest
			$attributes: array objekata atributa u shemi
			$data: array vrijednosti atributa koje se validiraju oblika [[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]]...]
			$new: true|false (ako je rijec o akciji dodavanja korisnika onda true, u svim ostalim slucajevima false)
		Izlaz:
			array('result'=>true|false: true ako je prosla validacija za sve atribute false ako nije prosla za bar jedan,'data'=>[['naziv_atributa'=>['result'=>true|false: true ako je prosla validacija,'message'=>poruka o gresci za taj atribut]]]
		*/
		public static function check_all($usertype,$attributes,$data,$new=false){
		//	print "data ";print_r($data);print "\n\n";
		//	print "attributes ";print_r($attributes);print "\n\n";
			$ok=true;
			if($new){
				foreach($attributes as $name=>$attr){
					$ret[$name]['result']=$attr->validate($data[$name],$usertype,$new);
					$ret[$name]['message']=$attr->err_message(true);
					$ok&=$ret[$name]['result'];
					\aosi\Logger::write("Checking: $name "  . $ret[$name]['message'],8);
					unset($data[$name]);
				}
				if(is_array($data)){
					if(count($data)>0){
						$ret[$name]=[];
						$ret[$name]['result']=false;
						$ret[$name]['message']="Atributi " . implode(', ', array_keys($data)). " ne postoji u shemi";
						\aosi\Logger::write("Checking: $name failed." .$ret[$name]['message'],8);
						$ok=false;
					}
				}
			} else {
				foreach($data as $name=>$attr){
					if(is_object($attributes[$name])){
						$ret[$name]=[];
						$ret[$name]['result']=$attributes[$name]->validate($attr,$usertype,$new);
						$ret[$name]['message']=$attributes[$name]->err_message(true);
						\aosi\Logger::write("Checking: $name " .$ret[$name]['message'],8);
						$ok&=$ret[$name]['result'];
					} else {
						$ret[$name]=[];
						$ret[$name]['result']=false;
						$ret[$name]['message']="Atribut $name ne postoji u shemi";
						\aosi\Logger::write("Checking: $name failed." .$ret[$name]['message'],8);
						$ok=false;
					}
				}
			}
			return ['result'=>$ok,'data'=>$ret];
		}
		
	}
	
	class HrEduPerson {
		static public $attribute;
		static public $no_eq_matching_rule=array('jpegphoto','facsimiletelephonenumber');
		
		public static function l(){
			self::$attribute=Schema::load('hreduperson');
		}
		/*
			vraca array atributa iz sheme kojima je tip binary
			Izlaz:
				array atributa binarnog tipa
		*/
		public static function get_binary_attrs(){
			return Schema::binary_attrs(self::$attribute);
		}
		
		/*
			vraca array atributa nad kojima tip korisnika $usertype (admin|user|guest) smije provesti akciju $action (view|change)
			Ulaz:
				$usertype: admin|user|guest
				$action: view|change
				$must: true|false (ako je true, funkcija vraca samo obavezne atribute)
			Izlaz:
				array atributa nad kojima tip korisnika $usertype (admin|user|guest) smije provesti akciju $action (view|change)
		*/
		public static function get_allowed_attrs($usertype,$action,$must=false){
			return Schema::allowed_attrs($usertype,$action,self::$attribute,$must);
		}
		
		/* Izvrsava provjeru (validaciju, autorizaciju) vrijednosti podataka za sve atribute u shemi na način da poziva odgovarajuce metode pojedinog atributa.
		Ova metoda ne moze provjeriti medjuovisnosti atributa vec samo validira njihove vrijednosti i provjerava smije li pojedini kategorija korisnika (admin|user|guest) promijeniti sadrzaj atributa.
		Ulaz:
			$usertype: admin|user|guest
			$data: array vrijednosti atributa koje se validiraju oblika [[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]]...]
			$new: true|false (ako je rijec o akciji dodavanja korisnika onda true, u svim ostalim slucajevima false)
		Izlaz:
			array('result'=>true|false: true ako je prosla validacija za sve atribute false ako nije prosla za bar jedan,'data'=>[['naziv_atributa'=>['result'=>true|false: true ako je prosla validacija,'message'=>poruka o gresci za taj atribut]]]
		*/
		public static function validate($data,$usertype,$new=false){
			return Schema::check_all($usertype,self::$attribute,$data,$new);
		}
	}
	
	class HrEduOrg {
		static public $attribute;
		static public $no_eq_matching_rule=array('facsimiletelephonenumber');
		
		public static function l(){
			self::$attribute=Schema::load('hreduorg');
		}
		/*
			vraca array atributa iz sheme kojima je tip binary
			Izlaz:
				array atributa binarnog tipa
		*/
		public static function get_binary_attrs(){
			return Schema::binary_attrs(self::$attribute);
		}
		
		/*
			vraca array atributa nad kojima tip korisnika $usertype (admin|user|guest) smije provesti akciju $action (view|change)
			Ulaz:
				$usertype: admin|user|guest
				$action: view|change
				$must: true|false (ako je true, funkcija vraca samo obavezne atribute)
			Izlaz:
				array atributa nad kojima tip korisnika $usertype (admin|user|guest) smije provesti akciju $action (view|change)
		*/
		public static function get_allowed_attrs($usertype,$action,$must=false){
			return Schema::allowed_attrs($usertype,$action,self::$attribute,$must);
		}
		
		/* Izvrsava provjeru (validaciju, autorizaciju) vrijednosti podataka za sve atribute u shemi na način da poziva odgovarajuce metode pojedinog atributa.
		Ova metoda ne moze provjeriti medjuovisnosti atributa vec samo validira njihove vrijednosti i provjerava smije li pojedini kategorija korisnika (admin|user|guest) promijeniti sadrzaj atributa.
		Ulaz:
			$data: array vrijednosti atributa koje se validiraju oblika [[atribut1]=>[[vrijednost1],[vrijednost2],...][atribut2]=>[[vrijednost1]]...]
			$usertype: admin|user|guest
			$new: true|false (ako je rijec o akciji dodavanja korisnika onda true, u svim ostalim slucajevima false)
		Izlaz:
			array('result'=>true|false: true ako je prosla validacija za sve atribute false ako nije prosla za bar jedan,'data'=>[['naziv_atributa'=>['result'=>true|false: true ako je prosla validacija,'message'=>poruka o gresci za taj atribut]]]
		*/
		public static function validate($data,$usertype,$new=false){
			return Schema::check_all($usertype,self::$attribute,$data,$new);
		}
	}
	
	class Attribute extends Simple {
	
		public $ldapname;
		public $name;
		public $type;
		public $attrtype; //atributi tipa (duljina, naziv sifrarnika ...)
		public $desc;
		public $regex;
		public $must;
		public $multiple;
		public $view;
		public $change;
		public $vocabulary;
		public $value;
		public $hredupersonuniquenumber_prefix=array('JMBG','JMBAG','LOCAL_NO','MBZ','OIB','PASSPORT_NO');
		public $hreduorguniquenumber_prefix=array('MBUST','RKDP','MZOS_SIFRA','OIB');
		
		function __construct($attr_def_array){
			$i=0;
			//print_r($attr_def_array);
			while($i<count($attr_def_array)){
				$attr=$attr_def_array[$i];
				switch($attr['tag']){
						case 'attribute':{
							$this->ldapname=trim($attr['attributes']['ldapname']);
							break;
						}
						case 'regex':{
							if (! (trim($attr['value']) == "") ){
								$this->regex='/' . trim($attr['value']) . '/';
							}
							break;
						}
						case 'name':
						case 'desc':{
							if (! (trim($attr['value']) == '') ){
								$this->{$attr['tag']}=trim($attr['value']);
							}
							
							break;
						}
						case 'type':{
							if($attr['type']=='open'){
								$level=$attr['level'];
								$i++;
								if($attr_def_array[$i]['level']<=$level){
									\aosi\Logger::write("Schema xml file $schemaxml is corrupted 3. Exiting!" ,1);
									exit();
								} else {
									while($attr_def_array[$i]['level']>$level){ 
										$this->type=$attr_def_array[$i]['tag'];
										if (is_array($attr_def_array[$i]['attributes'])) {
									    	foreach ($attr_def_array[$i]['attributes'] as $key => $val) {
									    		$this->attrtype[$key] = $val;
											}
										}
										$i++;
									}
									if($attr_def_array[$i]['tag']=='type' && $attr_def_array[$i]['type']=='close'){
										//$i++;
									} else {
										\aosi\Logger::write("Schema xml file $schemaxml is corrupted 4. Exiting!" ,1);
										exit();				
									}
								}
								
							} else {
								\aosi\Logger::write("Schema xml file $schemaxml is corrupted 5. Exiting!" ,1);
								exit();
							}
							break;
						}
						case 'status':{
							$this->must=trim($attr['attributes']['must']);
							break;
						}
						case 'frequency':{
							$this->multiple=trim($attr['attributes']['multiple']);
							if($this->multiple=='true'){
								$this->value=array();
							}
							break;
						}
						case 'view':
						case 'change':{
							if (is_array($attr['attributes'])) {
					    	foreach ($attr['attributes'] as $key => $val) {
					    		$this->{$attr['tag']}[$key] = $val;
							}
						}
						break;
						}
				}	
				$i++;
			}	
			if($this->type=='vocabulary'){
				if(isset($this->attrtype['name']) && isset($this->attrtype['uri'])){
					$this->vocabulary=$this->load_vocabulary($this->attrtype['name']);
				}
			}
		}
		
		private function load_vocabulary($name){
			$conf=Config::get('schema');
			$vocabularyxml=$conf['vocabularies'] . $name .".xml";
			if(!file_exists($vocabularyxml)){
				\aosi\Logger::write("Vocabulary xml file $vocabularyxml dosen't exists. Exiting!" ,1);
				exit();
			}
			$p = xml_parser_create('UTF-8');
			xml_parser_set_option($p, XML_OPTION_CASE_FOLDING, 0);
		  xml_parser_set_option($p, XML_OPTION_SKIP_WHITE, 1);
			xml_parse_into_struct($p, @file_get_contents($vocabularyxml), $vals,$index);
			if(xml_get_error_code ( $p )){
				\aosi\Logger::write("Error parsing $vocabularyxml! " . xml_error_string(xml_get_error_code($p)) . " line: " . xml_get_current_line_number($p) ." col: " . xml_get_current_column_number($p),1);
				exit();
			}
			\aosi\Logger::write("Vocabulary $vocabularyxml parsed." ,7);
			xml_parser_free($p);
			
			$ind=$index['vocabularyentry'];
			unset($index);
			$i=0;
			while($i<count($ind)){
				if($vals[$ind[$i]]['tag']=='vocabularyentry' && $vals[$ind[$i]]['type']=='complete'){
					$vocabulary[$vals[$ind[$i]]['attributes']['key']]=trim($vals[$ind[$i]]['value']);
				} else {
					\aosi\Logger::write("Vocabulary xml file $vocabularyxml is corrupted 1. Exiting!" ,1);
					exit();
				}
				$i++;
			}
			return $vocabulary;
		}
		
		function check_value($val){
			$ok=true;
			if($this->regex){
				if(!preg_match($this->regex,$val)){
					$ok=false;
					$this->append_err_message("Atribut " . $this->ldapname . " nema ispravnu sintaksu.");
				}
			}
			switch ($this->type){
				case 'string':
					if(!$this->attrtype['length']){
						$this->attrtype['length']=255;
					}
					if(strlen($val)>$this->attrtype['length']){
						$ok=false;
						$this->append_err_message("Atribut " . $this->ldapname ." je predugacak. On može sadržavati najviše ". $this->attrtype['length'] . " znakova.");
					}
					break;
				case 'number':
					$pomint=(int) $val;
					if($pomint!=$val){
						$ok=false;
						$this->append_err_message("Atribut " . $this->ldapname ." mora imati cjelobrojnu vrijednost.");
					}
					break;
				case 'date':
					if($val!='NONE'){
						//list($pomd,$pomm,$pomg)=explode('.',$val,3);
						if(preg_match('/^\d{8}$/',$val)){
							if(!@checkdate(substr($val,4,2),substr($val,-2),substr($val,0,4))){
								$ok=false;
								$this->append_err_message("Vrijednost atributa " . $this->ldapname ." nije ispravan datum.");
							}
						} else {
							$ok=false;
							$this->append_err_message("Vrijednost atributa " . $this->ldapname ." nije ispravan datum.");
						}
					}
					break;
				case 'vocabulary':
					if(!(substr($val,0,4)=='urn:' && $this->ldapname=='hrEduPersonRole')){
						if(!isset($this->vocabulary[$val])){
							$ok=false;
							$this->append_err_message("Atribut " . $this->ldapname ." mora imati jednu od vrijednosti iz šifrarnika " . $this->attrtype['name'] .".");
						}
					}
					break;
			}
			//custom check
			switch($this->ldapname){
				case 'userPassword':
					if(!$this->check_pwd($val)){
						$ok=false;
					}
					break;
				case 'hrEduPersonOIB':
				case 'hrEduOrgOIB':
					if($val!='NONE'){
						if(!$this->check_oib($val)){
							$this->append_err_message("OIB u atributu " . $this->ldapname ." nije ispravan.");
							$ok=false;
						}
					}
					break;
				case 'hrEduPersonUniqueNumber':
					if(!$this->check_unique_number($val)){
						$this->append_err_message("Neispravna vrijednost $val atributa " . $this->ldapname .".");
						$ok=false;
					}
					break;
				case 'hrEduOrgUniqueNumber':
					if(!$this->check_org_unique_number($val)){
						$this->append_err_message("Neispravna vrijednost $val atributa " . $this->ldapname .".");
						$ok=false;
					}
					break;
			}
			return $ok;
		}
		
		private function check_unique_number($val){
			list($pref,$v)=explode(':',$val);
			if(!in_array($pref,$this->hredupersonuniquenumber_prefix)){
				$this->append_err_message("Neispravan prefiks u atributu " . $this->ldapname .". Prefiks mora biti jedna od vrijednosti: " . implode (', ', $this->hredupersonuniquenumber_prefix) .".");
				return false;
			}
			$v=trim($v);
			if($v=='NONE' || !$v){				
				return false;
			}
			switch($pref){
				case 'JMBG':
					if(!preg_match('/^\d{13}$/',$v)){
						return false;
					}
					break;
				case 'JMBAG':
					if(!preg_match('/^\d{10}$/',$v)){
						return false;
					}
					break;
				case 'MBZ':
					if(!preg_match('/^\d+$/',$v)){
						return false;
					}
					break;
				case 'OIB':
					return $this->check_oib($v);
					break;
			}
			return true;
		}
		
		private function check_org_unique_number($val){
			list($pref,$v)=explode(':',$val);
			if(!in_array($pref,$this->hreduorguniquenumber_prefix)){
				$this->append_err_message("Neispravan prefiks u atributu " . $this->ldapname .". Prefiks mora biti jedna od vrijednosti: " . implode (', ', $this->hreduorguniquenumber_prefix) .".");
				return false;
			}
			$v=trim($v);
			if($v=='NONE' || !$v){				
				return false;
			}
			switch($pref){
				case 'MBUST':
				case 'RKDP':
				case 'MZOS_SIFRA':
					if(!preg_match('/^\d+$/',$v)){
						return false;
					}
					break;
				case 'OIB':
					return $this->check_oib($v);
					break;
			}
			return true;
		}
		
		private function check_oib($oib) {
			if($oib=='NONE'){
				return true;
			}
      // OIB ima 11 znamenaka i mora biti numeric.
      if (!preg_match('/^\d{11}$/',$oib)){
         return false;
      }
      // Posljednja tj. 11. znamenka je kontrolna znamenka. Dobivena  je
      // izračunom  iz  prethodnih  10  znamenaka  po međunarodnoj  normi  ISO 7064 (MOD 11, 10).
      // Prva znamenka zbroji se s brojem 10. U sljedećim koracima to će biti ostatak koji će se zbrajati
      // s idućom znamenkom.
      $ostatak = 10;

      // Prođi kroz sve znamenke, osim zadnje.
      for($i=0;$i<10;$i++) {
         // Dohvati trenutni znak iz OIBa i castaj ga u int kako bismo mogli raditi operacije.
         $trenutnaZnamenka = (int) $oib[$i];
         // 1. Prva znamenka zbroji se s brojem 10, a svaka sljedeća s ostatkom u prethodnom koraku.
         $zbroj = $trenutnaZnamenka + $ostatak;
         // 2. Dobiveni  zbroj  cjelobrojno  (s  ostatkom)  podijeli  se  brojem  10;  ako  je  dobiveni
         // ostatak 0 zamijeni se brojem 10 (ovaj broj je tzv. međuostatak)
         $meduOstatak = $zbroj % 10;
         if ( $meduOstatak == 0) {
            $meduOstatak = 10;
         }
         // 3. Dobiveni međuostatak pomnoži se brojem 2
         $umnozak = $meduOstatak * 2;
         // 4. Dobiveni  umnožak  cjelobrojno  (s  ostatkom)  podijeli se  brojem  11;  ovaj  ostatak
         // matematički nikako ne može biti 0 jer je rezultat prethodnog koraka uvijek paran broj
         $ostatak = $umnozak % 11;
         // 5. Slijedeća znamenka zbroji se s ostatkom u prethodnom koraku...
         // 6. Ponavljaju se koraci 2, 3, 4 i 5  dok se ne potroše sve znamenke...
      }
      // 7. Razlika izmenu broja 11 i ostatka u zadnjem koraku je kontrolna znamenka.
      // Ako je ostatak 1 kontrolna znamenka je 0 (11 1=10, a 10 ima dvije znamenke)
      if ( $ostatak == 1 ) {
         $kontrolnaZnamenka = 0;
      } else {
         $kontrolnaZnamenka = 11 - $ostatak;
      }
      // Provjeri dali kontrolne znamenka odgovara onoj u OIBu
      if (((int)$oib[10]) == $kontrolnaZnamenka ) {
         return true;
      }
      // Ako smo došli tu, kontrola nije prošla.
      return false;
   }
		
		private function check_pwd($pwd){
			if(strlen($pwd)<8){
	      $this->append_err_message('Zaporka mora biti duža od 8 znakova.');
	      return false;
      }
      if(@preg_match_all('/\d/', $pwd, $pom_arr)<2){
	      $this->append_err_message('Zaporka mora sadržavati barem 2 znamenke.');
	      return false;
      }
      if(@preg_match_all('/[a-zA-ZčćžšđČĆŽŠĐ]/', $pwd, $pom_arr)<2){
	      $this->append_err_message('Zaporka mora sadržavati barem 2 slova.');
	      return false;
      }
			return true;
		}
		
		function validate($value=NULL,$usertype,$new=false){
			//print_r($value);
			if($this->change[$usertype]!='true' && !$new){
				$this->append_err_message("Niste ovlašteni mijenjati vrijednost atributa " . $this->ldapname .".");
				$ok=false;
			} else {
				$ok=true;
				if(is_array($value)){
					if(count($value)>1){
						if($this->multiple=='false'){
							$ok=false;
							$this->append_err_message("Atribut " . $this->ldapname . " smije imati samo jednu vrijednost.");
						} else {
							$tmpval=$value;
							foreach ($tmpval as $k=> $v) {
								if($v==''){
									unset($tmpval[$k]);
								}
							}
							if(count($tmpval)<1){
								if($this->must=='true'){
									$ok=false;
									$this->append_err_message("Atribut " . $this->ldapname ." mora imati bar jednu vrijednost.");
								}
							} else {
								foreach ($value as $key=>$val){
									if(!$this->check_value($val)){
										$ok=false;
									}
								}
							}
			
						}
					} else {
						$val=array_pop($value);
						$val=trim($val);
						if(is_null($val) || $val==''){
							if($this->must!='false'){
								$this->append_err_message("Atribut " . $this->ldapname ." je obavezan.");
								$ok=false;
							}
						} else {
							if(!$this->check_value($val)){
								$ok=false;
							}
						}
					}
				} else {
					if($this->must!='false'){
						$this->append_err_message("Atribut " . $this->ldapname ." je obavezan.");
						$ok=false;
					}
				}
			}
			return $ok;
		}
		
		/*
		Funkcija na provjerava smije li određena kategorija osobe vidjeti / promijeniti vrijednost atributa
		Ulaz:
			$usertype: admin | user | guest 
			$action: view | change
		Izlaz:
			true | false
		*/
		
		public function is_allowed($usertype,$action){
			if($this->{$action}[$usertype]!='true'){
				return false;
			}
			return true;
		}
		
	}
	
	
}
?>