<?php
namespace aosi{
	class Config
	{
	    static private $data;
	
	    static public function load($configFile='aosi_config.php') {
	        self::$data = include($configFile);
	        if(!is_array(self::$data)) die ("Can't load config file $configFile");
	    }
	
	    static public function get($key) {
	         // code stuff to deal with sections and keys, but basically
	         return self::$data[$key];
	    }
	}
	
	class Logger
	{
		static private $opts;
		static private $id;
		
		static public function prepare() {
			self::$opts=Config::get('log');
			self::$id=substr(uniqid() . rand(10,99) , 6);
		}
		//severity 1	Alert	alert, 2	Critical	crit, 3	Error	err, 4	Warning	warn, 5	Notice	notice, 6	Informational	info, 7	Debug	debug
		static public function write($message,$severity=6) {
			if(self::$opts['level']>=$severity){
				if(self::$opts['backtrace']){
					$message.=" " . print_r(debug_backtrace(0),true);
				}
				$caller=debug_backtrace()[1]['class'] ? debug_backtrace()[1]['class'] . ":" . debug_backtrace()[1]['function'] : debug_backtrace()[1]['function'];
				$message=self::$id . " [" . $caller . "] " .$message;
				if(self::$opts['use_syslog']){
					return self::syslog($severity,$message);
				} else {
					return self::writelog($severity,$message);
				}
			}
		}
		
		static private function syslog($severity,$message){
			openlog("AOSI", LOG_ODELAY, self::$opts['syslog_facility']);
			syslog($severity, addslashes($message));
			closelog();
		}
		
		static private function writelog($severity,$message){
			//error_log(date("YmdHis") . " " .$message . "\n",3,);
			file_put_contents(self::$opts['file'], date("YmdHis") . " " . $severity ." " . $message . "\n",FILE_APPEND);
		}
		
	}
	
	class ActionLog
	{
		/* Funkcija pise zapis u action.log
		
		Ulaz:
			$callerUid: uid pozivatelja funkcije (akcije) na koju se zapis odnosi
			$methodName: naziv pozvane funkcije (akcije)
			$subjectDn: dn subjekta akcije
			$errorCode: kod greske
			$params: array u kojem su kljucevi atributi akcije
		*/
		//<e><d>datum dd.mm.gggg.</d><t>vrijeme hh:mm:ss</t><w>uid onog koji zove metodu</w><a>metoda</a><s>dn</s><c>kod (uspjeh:0, sve ostalo greska)</c><p>parametri</p></e>
		static public function write($callerUid,$methodName,$subjectDn,$errorCode,$params=false) {
			if($errorCode===true){
				$errorCode=0;
			} elseif(!$errorCode){
				//unexpected error
				$errorCode=-64;
			}
			if(is_array($params)) $params=implode(',', array_keys($params));
			file_put_contents(Config::get('actionlog'),'<e><d>' . date('d.m.Y.') . '</d><t>' . date('H:i:s') . '</t><w>'. $callerUid . '</w><a>' . $methodName . '</a><s>' . $subjectDn . '</s><c>' . $errorCode . '</c><p>' . $params . "</p></e>\n",FILE_APPEND);
		}
		static public function read() {
			return file_get_contents(Config::get('actionlog'));
		}
		
	}
	
	class Simple {
		private $err_code;
		private $err_message;
		private $result;
		
		//ako je reset=true izbrise vrijednost nakon sto je vrati
		public function err_code($reset=false){
			$m=$this->err_code;
			if($reset) $this->err_code=false;
			return $m;
		}
		//ako je reset=true izbrise vrijednost nakon sto je vrati
		public function err_message($reset=false){
			$m=$this->err_message;
			if($reset) $this->err_message=false;
			return $m;
		}
		function reset_err(){
			$this->err_message=false;
			$this->err_code=false;
		} 
		public function set_err_code($err_code){
			$this->err_code=$err_code;
		}
		public function set_err_message($err_message){
			$this->err_message=$err_message;
		}
		public function set_error_obj($obj){
			$this->err_code=$obj->err_code();
			$this->err_message=$obj->err_message();
			$obj->reset_err();
			
		}
		public function set_error($err_code,$err_message){
			$this->err_code=$err_code;
			$this->err_message=$err_message;
		}
		//ako je reset=true izbrise vrijednost nakon sto je vrati
		public function result($reset=false){
			$m=$this->result;
			if($reset) $this->result=false;
			return $m;
		}
		public function set_result($result){
			$this->result=$result;
		}
		public function merge_result($result){
			$this->result=@array_merge($this->result(true), $result);
		}
		public function set_result_obj($obj){
			$this->result=$obj->result(true);
		}
		public function append_err_message($err_message){
			$this->err_message.= " ".$err_message;
		}
	}

	
	/*
		Funkcija ulazni array pretvori u json (za pisanje u log). ako postoji par 'userpassword'=>vrijednost maskira ga
		Ulaz:
			$data: array vrijednosti atributa
		Izlaz:
			json enkodirani string koji je reprezentacija ulaznog arraya s maskiranim passwordom
		private function array2log($data,$hide_password){
			if($hide_password){
				$data=$this->hide_password($data);
			}
			return str_replace(["\n","\r","\t"],[" ", " ", " "],print_r($data,true));			
		}*/
		
		
 		function array2log($data,$hide_password=false){
			if($hide_password){
				$data=hide_password($data);
			}
			return array2txt($data,true);
		}
		
		function array2txt($data){
			$ret='';
			if(is_array($data)){
				foreach($data as $key=>$val){
					$ret.="{" . $key ."}=>[";
					if(is_array($val)){
						$ret.=array2txt($val);
					} else {
						$ret.= $val;
					}
					$ret.="],";
				}
			}
			return rtrim($ret,',');
		}
		
		/*
			Funkcija u ualaznom array-u trazi par 'userpassword'=>vrijednost i maskira ga
		Ulaz:
			$data: array vrijednosti atributa
		Izlaz:
			array s maskiranim passwordom
		*/
		
		function hide_password($data){
			if(is_array($data)){
				if(isset($data['userpassword'])){
					$data['userpassword']='*';
				} elseif(isset($data['jpegphoto'])){
					$data['jpegphoto']='*'; 
				} else {
					foreach($data as $key=>$val){
						if(isset($val['userpassword'])){
							$data[$key]['userpassword'][0]='*';
						}elseif(isset($val['jpegphoto'])){
							$data[$key]['jpegphoto'][0]=md5($data[$key]['jpegphoto'][0]);
							//$data[$key]['jpegphoto'][0]='*';
						}
					}
				}
			}
			return $data;
		}
}

?>