<?php


namespace App\Extensions;

use Illuminate\Auth\AuthenticationException;
use Illuminate\Contracts\Auth\Authenticatable;
use Illuminate\Contracts\Auth\Guard;

/**
 * Klasa za Auth (login, logout - spremanje i micanje korisnika iz sesije).
 * Class LdapGuard
 * @package App\Extensions
 */
class LdapGuard implements Guard
{
    private $sessionKey = 'ldapuser';

    /**
     * @inheritDoc
     */
    public function check()
    {
        return session()->has($this->sessionKey);
    }

    /**
     * @inheritDoc
     */
    public function guest()
    {
        return ! session()->has($this->sessionKey);
    }

    /**
     * @inheritDoc
     */
    public function user()
    {
        if ($this->check()) {
            return unserialize(session()->get($this->sessionKey));
        }

        return null;
    }

    /**
     * @inheritDoc
     */
    public function id()
    {
        if ($this->check()) {
            return $this->user()->getAuthIdentifier();
        }

        return null;
    }

    /**
     * @inheritDoc
     */
    public function validate(array $credentials = [])
    {
        // Ne koristi se
        return false;
    }

    /**
     * @inheritDoc
     */
    public function setUser(Authenticatable $user)
    {
        session()->put($this->sessionKey, serialize($user));
    }

    public function login(Authenticatable $user)
    {
        $this->setUser($user);
    }

    public function logout()
    {
        // Obriši sve podatke iz sesije
        session()->flush();
    }

    public function authenticate()
    {
        if (! $this->check()) {
            throw new AuthenticationException('Unauthenticated.');
        }
    }
}