<?php


namespace App\Helpers\HrEdu;


use aosi\HrEduPerson;
use App\Helpers\AosiCoreHelper;
use Illuminate\Http\Request;

class HrEduPersonHelper extends AbstractHrEduHelper
{
    public function __construct()
    {
        parent::__construct();

        $this->attributeDefinitions = HrEduPerson::$attribute;
    }

    /**
     * @inheritDoc
     */
    public function getNonModifiableAttributes()
    {
        return [
            'userpassword', // Password ide kroz svoju formu.
            // TODO mivanci maknuti dolje hardkodirane atribute nakon usklađivanja sheme
            'postaladdress', // postaladdress se naslijeđuje iz org zapisa i ne može se samostalno mijenjati.
            'postalcode', // postalcode se naslijeđuje iz org zapisa i ne može se samostalno mijenjati.
            'street', // street se naslijeđuje iz org zapisa i ne može se samostalno mijenjati.
            'l', // l (lokalitet) se naslijeđuje iz org zapisa i ne može se samostalno mijenjati.
        ];
    }

    public function getAllowedAttributes($userType, $action, $must = false)
    {
        return HrEduPerson::get_allowed_attrs($userType, $action, $must);
    }

    public function validateLoginRequest(Request $request)
    {
        $paramsToValidate = [
            'uid' => 'required',
            'password' => 'required',
        ];

        // Ako nije single realm instance, onda realm treba biti naveden u requestu.
        if (! AosiCoreHelper::isSingleRealmInstance()) {
            $paramsToValidate['realm'] = 'required';
        }

        $credentials = $request->validate($paramsToValidate, [
            'uid.required' => 'Korisnička oznaka je obavezna',
            'password.required' => 'Zaporka je obavezna',
            'realm.required' => 'Domena je obavezna'
        ]);;

        // U komunikaciji s LDAPom, password mora biti base64 encoded.
        $credentials['password'] = base64_encode($credentials['password']);

        return $credentials;
    }
}