<?php

namespace App\Http\Controllers\Admin;

use aosi\HrEduPerson;
use App\Helpers\AosiCoreHelper;
use App\Helpers\HrEdu\HrEduPersonHelper;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;

class AdminListController extends Controller
{
    public function index()
    {
        $aosiCore = AosiCoreHelper::init();

        $user = Auth::user();

        $adminListResult = $aosiCore->ldapAdminList(
            $user->uid[0],
            $user->password[0],
            $user->basedn[0]
        );

        $admins = [];
        if (!$adminListResult) {
            flash('Ne postoji niti jedan administrator')->warning()->message();
        } else if ($aosiCore->result()) {
            $admins = $aosiCore->result();
        }

        return view('admin.popisAdministratora', compact('admins'));
    }

    public function edit($uid)
    {
        $uid = htmlspecialchars($uid);

        $aosiCore = AosiCoreHelper::init();
        $adminTypes = AosiCoreHelper::getAdminTypes();

        $user = Auth::user();

        $adminInfoResult = $aosiCore->ldapAdminInfo(
            $user->uid[0],
            $user->password[0],
            $user->basedn[0],
            $uid
        );

        if (!$adminInfoResult) {
            flash('Ne postoje informacije o administratoru '. $uid)->danger()->important();
            return redirect(route('admin.admin.index'));
        }

        $limits = [];
        $adminType = $adminTypes[$adminInfoResult];
        $oldHrEduPersonRoleValue = '';

        // Ako je $adminInfo == 1, to je administrator definiran u konfiguracijskoj datoteci.
        if ($adminInfoResult == 1) {
            flash('Nije moguće ažurirati administratora definiranog u konfiguracijskoj datoteci.')
                ->warning()
                ->important();
            return redirect(route('admin.admin.index'));
        } elseif ($adminInfoResult == 2) {
            // Ako je $adminInfo == 2, to je administrator imenika. Postavit ćemo da je to bila stara hrEduPerson role
            // vrijednost, a koju će onda biti potrebno brisati.
            $oldHrEduPersonRoleValue = AosiCoreHelper::ADMIN_VALUE_DESIGNATOR;
        } elseif ($adminInfoResult == 3) {
            // Ako je 3, to je administrator iz imenika s ograničenjem. ->result() sadrži array s ograničenjima.
            if ($aosiCore->result()) {
                // Trebam osigurati lower case za sve ključeve
                foreach ($aosiCore->result() as $key => $value) {
                    $limits[strtolower($key)] = $value;
                }
            }
            // Moram generirati URN koji će služiti kao old value kojeg će trebati brisati.
            $oldHrEduPersonRoleValue = AosiCoreHelper::generateUrn(AosiCoreHelper::ADMIN_VALUE_DESIGNATOR, $limits);
        }

        // Atributi nad kojima je moguće postavljati limite
        $adminLimitAttributes = AosiCoreHelper::ADMIN_LIMIT_ATTRIBUTES;
        $attributeDefinitions = HrEduPerson::$attribute;
//dd($adminLimitAttributes, $oldHrEduPersonRoleValue, $adminInfoResult, $aosiCore->result());
        return view('admin.azuriranjeAdministratora', compact(
            'uid',
            'adminType',
            'adminLimitAttributes',
            'limits',
            'oldHrEduPersonRoleValue',
            'attributeDefinitions'
        ));

    }

    public function update($uid, Request $request)
    {
        $uid = htmlspecialchars($uid);

        $aosiCore = AosiCoreHelper::init();
        $userHelper = new HrEduPersonHelper();

        $user = Auth::user();

        $adminInfoResult = $aosiCore->ldapAdminInfo(
            $user->uid[0],
            $user->password[0],
            $user->basedn[0],
            $uid
        );

        if (!$adminInfoResult) {
            flash('Ne postoje informacije o administratoru '. $uid)->danger()->important();
            return redirect(route('admin.admin.index'));
        }

        // Ako je $adminInfo == 1, to je administrator definiran u konfiguracijskoj datoteci.
        if ($adminInfoResult == 1) {
            flash('Nije moguće ažurirati administratora definiranog u konfiguracijskoj datoteci.')
                ->warning()
                ->important();
            return redirect(route('admin.admin.index'));
        }

        // Dohvati staru vrijednost koju će biti potrebno brisati (ako nije prazna)
        $oldHrEduPersonRoleValue = $request->get('oldhredupersonrolevalue');

        // Dohvati samo one vrijednosti koje se koriste za ograničavanje
        $limitAttributes = AosiCoreHelper::ADMIN_LIMIT_ATTRIBUTES;
        $limits = $request->only($limitAttributes);

        // Makni sve prazne vrijednosti
        $limits = $userHelper->filterEmptyRawRequestValues($limits, false);

        $urn = AosiCoreHelper::generateUrn(AosiCoreHelper::ADMIN_VALUE_DESIGNATOR, $limits);

        if ($oldHrEduPersonRoleValue == $urn) {
            flash('Nisu napravljene promjene u odnosu na staru vrijednost.')->warning();
            return back();
        }

        $userDn = AosiCoreHelper::generateDnForUid($uid);

        $attributeKey = 'hredupersonrole';

        $data = [
            $userDn => [$attributeKey => [$urn]]
        ];

        $addResult = $aosiCore->ldapAddAttribute(
            $user->uid[0],
            $user->password[0],
            $user->basedn[0],
            $data
        );

        if (!$addResult) {
            $message = head($aosiCore->result())['message'] ?? 'Greška';
            flash($message)->error()->important();
            return back();
        }

        // Dodavanje je prošlo, obriši staru vrijednost ako postoji.
        if ($oldHrEduPersonRoleValue) {
            $data = [
                $userDn => [$attributeKey => $oldHrEduPersonRoleValue]
            ];
            $deleteResult = $aosiCore->ldapDeleteAttribute(
                $user->uid[0],
                $user->password[0],
                $user->basedn[0],
                $data
            );

            if (!$deleteResult) {
                flash('Nije uspjelo brisanje stare vrijednosti atributa ' . $attributeKey)->error()->important();
                flash(head($aosiCore->result())['message'] ?? 'Greška')->error()->important();
            } else {
                flash('Administrator ažuriran')->success();
            }
        }

        return redirect(route('admin.admin.update', ['uid' => $uid]));
    }

    /**
     * Ukini administratorsko pravo korisniku
     * @param $uid
     * @return \Illuminate\Http\RedirectResponse|\Illuminate\Routing\Redirector
     */
    public function revoke($uid)
    {
        $uid = htmlspecialchars($uid);

        $aosiCore = AosiCoreHelper::init();

        $user = Auth::user();

        $attributeKey = 'hredupersonrole';

        $searchResult = $aosiCore->ldapSearch(
            $user->uid[0],
            $user->password[0],
            $user->basedn[0],
            '(uid=' . $uid . ')',
            [$attributeKey]
        );

        if (!$searchResult) {
            flash('Korisnik s oznakom ' . $uid . ' ne postoji.')->warning()->important();
            return redirect(route('admin.admin.index'));
        }

        // Potrebno je obrisati sve entries koje imaju ključ za definiranje admina, pa ću ih prikupiti
        $adminValueDesignator = AosiCoreHelper::ADMIN_VALUE_DESIGNATOR;

        $adminValues = [];
        if (isset(head($searchResult)[$attributeKey]) && is_array(head($searchResult)[$attributeKey])) {
            foreach (head($searchResult)[$attributeKey] as $value) {
                if (str_contains($value, $adminValueDesignator)) {
                    $adminValues[] = $value;
                }
            }
        }

        if (!$adminValues) {
            flash('Korisnik nema definirana administratorska prava nad imenikom.')->warning()->important();
            return redirect(route('admin.admin.index'));
        }

        $userDn = AosiCoreHelper::generateDnForUid($uid);

        $data = [
            $userDn => [$attributeKey => $adminValues]
        ];

        $deleteResult = $aosiCore->ldapDeleteAttribute(
            $user->uid[0],
            $user->password[0],
            $user->basedn[0],
            $data
        );

        if (!$deleteResult) {
            flash(head($aosiCore->result())['message'] ?? 'Greška kod brisanja atributa')->error()->important();
        } else {
            flash('Administratorska prava ukinuta za ' . $uid)->success();
        }

        return redirect(route('admin.admin.index'));
    }
}
