<?php

namespace App\Http\Controllers\Admin;

use aosi\HrEduOrg;
use App\Helpers\AosiCoreHelper;
use App\Helpers\HrEdu\HrEduOrgHelper;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;

class HrEduOrgController extends Controller
{
    public function edit()
    {
        $aosiCore = AosiCoreHelper::init();
        $orgHelper = new HrEduOrgHelper();

        // Atributi koji će biti prikazani na formi
        $viewableAttributes = HrEduOrg::get_allowed_attrs('admin', 'view');
        // Definicija svih postojećih atributa.
        $allAttributeDefinitions = $orgHelper->getAttributeDefinitions();
        $user = Auth::user();

        $orgResult = $aosiCore->ldapOrgInfo(
            $user->uid[0],
            $user->password[0],
            $user->basedn[0]
        );
//dd($orgResult);
        if (!$orgResult) {
            throw new \Exception('Nije moguće dohvatiti informacije o organizaciji.');
        }


        // Bind type zahardkodirano na admina
        $bindType = 'admin';
        // Vrijednosti atributa org zapisa
        $resource = (object) head($orgResult);
        $formPostUrl = route('admin.hreduorg.update');
        $info = 'Važno: kako su vrijednosti nekih od atributa (o, postalAddress, l) svih korinika u LDAP imeniku ' .
            'izvedene iz vrijednosti atributa hrEduOrg zapisa, ažuriranjem vrijednosti tih atributa, po ažuriranju ' .
            'org zapisa, svim će korisnicima u imeniku vrijednosti tih atributa biti ažurirane. Takva operacija ' .
            'promjene velikog broja zapisa može biti vremenski izuzetno zahtjevna. Zbog toga se može dogoditi da vaš ' .
            'web preglednik, nakon što pokrenete akciju ažuriranja, ne dočeka kraj izvršenja te akcije već ode u ' .
            'timeout. Važno je znati da takva greška ne utječe na izvršenje akcije (jer je ona već poslana AOSI web ' .
            'servisu na izvršenje) i ako se dogodi nemojte raditi reload stranice. Ukoliko promijenite vrijednost ' .
            'atributa o, uz već opisane promjene, promijenit će se i naziv ustanove prikazan pri vrhu ove stranice.';

//        dd($viewableAttributes, $allAttributeDefinitions, $org);
        return view('admin.azuriranjeHrEduOrgZapisa', compact(
                'resource',
                'viewableAttributes',
                'allAttributeDefinitions',
                'bindType',
                'formPostUrl',
                'info'
            )
        );
    }

    public function update(Request $request)
    {
        $aosiCore = AosiCoreHelper::init();
        $orgHelper = new HrEduOrgHelper();

        $bindType = 'admin';

        // Po potrebi validirati
//        $orgHelper->validateUpdateRequest($request, $bindType);

        $user = Auth::user();

        $data = $orgHelper->prepareModifiableAttributeValues($request, $bindType);
//dd($data);
        if ($data) {
            $modifyResult = $aosiCore->ldapModifyOrgAttribute(
                $user->uid[0],
                $user->password[0],
                $user->basedn[0],
                $data
            );

            if (!$modifyResult) {
                $errorMessages[] = array_pop(array_values($aosiCore->result()))['message'];
            }
        }

        // TODO mivanci javi pokušaj brisanja obaveznih atributa
        // TODO mivanci razmotri brisanje pomoću AJAXa

        // Atributi koji su prazni, a koji nisu obavezni, mogu se brisati bez davanja vrijednosti.
        $data = $orgHelper->prepareDeletableAttributeValues($request, $bindType);

        if ($data) {
            foreach ($data as $key => $value) {
                $data = [$key => $value];

                $deleteResult = $aosiCore->ldapDeleteOrgAttribute(
                    $user->uid[0],
                    $user->password[0],
                    $user->basedn[0],
                    $data
                );

                if (!$deleteResult) {
                    $ignoredErrorCodes = [
                        16 // LDAP_NO_SUCH_ATTRIBUTE - atribut je ne postoji. Za nas je to ok, pošto smo ga pokušali obrisati.
                    ];
                    if (!in_array(head($aosiCore->result())['code'], $ignoredErrorCodes)) {
                        $errorMessages[] = head($aosiCore->result())['message'];
                    }
                }
            }
        }

        // Prikaži poruke o greškama prilikom svih operacija.
        if ($errorMessages) {
            flash('Došlo je do greške prilikom ažuriranja podataka.')->error();
            foreach ($errorMessages as $errorMessage) {
                flash($errorMessage)->warning()->important();
            }
        } else {
            flash('Podaci su ažurirani.')->success();
        }

        return redirect(route('admin.hreduorg.edit'));
    }
}
