<?php

namespace App\Http\Controllers\Admin;

use Illuminate\Http\File;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Storage;

class LogoController extends Controller
{
    protected $logoFileName = 'logo';

    /**
     * Ključ za cache koji će se generirati prilikom uploada nove slike kako bi se generirao random url na sliku.
     * Pogledaj resources/views/_layouts/_partials/header.blade.php (src za logo)
     */
    const LOGO_CACHE_KEY = 'logo-cache-key';

    protected $cacheDuration = 86400;

    public function edit()
    {
        return view('admin.promjenaLogotipa');
    }

    public function update(Request $request)
    {
        $this->validate($request, [
            'logo' => 'required|file|image|max:1024'
        ], [
            'logo.required' => 'Datoteka je obavezna',
            'logo.file' => 'Datoteka nije ispravna',
            'logo.image' => 'Datoteka nije slikovna datoteka',
            'logo.max' => 'Datoteka je prevelika',
        ]);

        Storage::delete($this->logoFileName);

        $request->file('logo')->storeAs('', $this->logoFileName);

        cache()->put(static::LOGO_CACHE_KEY, str_random(), $this->cacheDuration);

        flash('Logotip je promijenjen')->success();

        return redirect(route('admin.logo.edit'));
    }

    public function show($logoCacheKey)
    {
        if (!Storage::has($this->logoFileName)) {
            return response('', 404);
        }

        $path = $this->getLogoStoragePath();

        return response()->file($path, [
            'Cache-Control' => 'max-age=86400, public'
        ]);
    }

    protected function getLogoStoragePath()
    {
        return storage_path('app' . DIRECTORY_SEPARATOR . $this->logoFileName);
    }
}
