<?php

namespace App\Http\Controllers\Admin;

use App\Helpers\AosiCoreHelper;
use App\Helpers\CsvHelper;
use foo\bar;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Validation\Rule;

/**
 * Dodavanje, ažuriranje i brisanje korisnika iz datoteke
 * Class UserFromFileController
 * @package App\Http\Controllers\Admin
 */
class UserFromFileController extends Controller
{
    protected $encodings;

    public function __construct()
    {
        $this->encodings = CsvHelper::getEncodings();
    }

    public function add()
    {
        return view('admin.dodavanjeKorisnikaIzDatoteke', [
            'encodings' => $this->encodings
        ]);
    }

    public function addStore(Request $request)
    {
        CsvHelper::validateCsvUploadRequest($request);

        $file = $request->file('csv_file');
        $delimiter = $request->get('delimiter');
        $encoding = $request->get('encoding');

        $aosiCore = AosiCoreHelper::init();
        $baseDn = AosiCoreHelper::getBaseDn();

        /**
         * Atributi koji moraju postojati u datoteci
         */
        $neededAtributes = ['uid', 'userpassword', 'hredupersonoib', 'givenname', 'sn', 'mail',
            'hredupersonprimaryaffiliation', 'hredupersonexpiredate'];

        $data = CsvHelper::getCsvDataFromUploadedFile($file, $delimiter, $encoding, $neededAtributes, $baseDn);

        $user = Auth::user();

        $aosiCore->ldapAddUserLE(
            $user->uid[0],
            $user->password[0],
            $user->basedn[0],
            $data
        );

        return view('admin.dodavanjeKorisnikaIzDatoteke', [
            'result' => $aosiCore->result()
        ]);
    }

    public function update()
    {
        return view('admin.azuriranjeKorisnikaIzDatoteke', [
            'encodings' => $this->encodings
        ]);
    }
    public function updateStore(Request $request)
    {
        CsvHelper::validateCsvUploadRequest($request);

        $file = $request->file('csv_file');
        $delimiter = $request->get('delimiter');
        $encoding = $request->get('encoding');

        $aosiCore = AosiCoreHelper::init();
        $baseDn = AosiCoreHelper::getBaseDn();

        /**
         * Atributi koji moraju postojati u datoteci
         */
        $neededAtributes = ['uid',];

        $data = CsvHelper::getCsvDataFromUploadedFile($file, $delimiter, $encoding, $neededAtributes, $baseDn, true);

        $user = Auth::user();

        $aosiCore->ldapModifyAttribute(
            $user->uid[0],
            $user->password[0],
            $user->basedn[0],
            $data
        );

        return view('admin.azuriranjeKorisnikaIzDatoteke', [
            'result' => $aosiCore->result()
        ]);
    }


    public function delete()
    {
        return view('admin.brisanjeKorisnikaIzDatoteke', [
            'encodings' => $this->encodings
        ]);
    }

    public function deleteStore(Request $request)
    {
        CsvHelper::validateCsvUploadRequest($request, true);

        $file = $request->file('csv_file');
        $encoding = $request->get('encoding');

        $aosiCore = AosiCoreHelper::init();

        $data = CsvHelper::getDataFromUploadedFile($file, $encoding);

        if ($data) {
            $data = array_map(function ($uid) {
                $uid = htmlspecialchars($uid);
                return AosiCoreHelper::generateDnForUid($uid);
            }, $data);
        } else {
            flash('Datoteka ne sadrži podatke.')->warning()->important();
            return back();
        }

        $user = Auth::user();

        $aosiCore->ldapDeleteUser(
            $user->uid[0],
            $user->password[0],
            $user->basedn[0],
            $data
        );

        return view('admin.brisanjeKorisnikaIzDatoteke', [
            'result' => $aosiCore->result()
        ]);
    }
}
