<?php

namespace App\Http\Controllers\Admin;

use aosi\HrEduPerson;
use App\Extensions\LdapUser;
use App\Helpers\AosiCoreHelper;
use App\Helpers\HrEdu\HrEduPersonHelper;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;

class UserListController extends Controller
{
    public function edit($uid)
    {
        $uid = htmlspecialchars($uid);

        $aosiCore = AosiCoreHelper::init();
        $userHelper = new HrEduPersonHelper();

        // Atributi koji će biti prikazani na formi
        $viewableAttributes = HrEduPerson::get_allowed_attrs('admin', 'view');
        // Definicija svih postojećih atributa.
        $allAttributeDefinitions = $userHelper->getAttributeDefinitions();

        $user = Auth::user();

        $bindResult = $aosiCore->ldapSearch(
            $user->uid[0],
            $user->password[0],
            $user->basedn[0],
            '(uid=' . $uid .')',
            $viewableAttributes
        );
//dd($viewableAttributes, $bindResult);
        if (!$bindResult) {
            flash('Korisnik s oznakom ' . $uid . ' ne postoji.')->warning()->important();
            return redirect(route('admin.user.list.show'));
        }

        $attributes = array_shift(array_values($bindResult));
        $ldapUser = new LdapUser($attributes);

        // Prikupi informacije o passwordPolicy .
        $userExistsResult = $aosiCore->ldapUserExists(
            $user->uid[0],
            $user->password[0],
            $user->basedn[0],
            $uid
        );

        $passwordPolicyMessage = '';
        if ($userExistsResult && $aosiCore->result()) {
            $passwordPolicyMessage = $this->generatePasswordPolicyMessage($aosiCore->result());
        }

        // Neka bude zahardkodirano da je view za admina
        $bindType = 'admin';
        $resource = $ldapUser;
        // Podesi rute, pošto se forma dijeli sa userom.
        $formPostUrl = route('admin.user.update', ['uid' => $uid]);
        $pictureShowUrl = route('admin.user.picture.show', ['uid' => $uid, 'hash' => '']);
        $pictureDeleteUrl = route('admin.user.picture.delete', ['uid' => $uid, 'hash' => '']);
        $showMandatoryOnlyAttributes = 1;

        return view('admin.azuriranjeKorisnika')
            ->with(compact(
                'resource',
                'viewableAttributes',
                'allAttributeDefinitions',
                'bindType',
                'formPostUrl',
                'pictureShowUrl',
                'pictureDeleteUrl',
                'showMandatoryOnlyAttributes',
                'passwordPolicyMessage'
            ));
    }

    public function update($uid, Request $request)
    {
        $uid = htmlspecialchars($uid);
        $aosiCore = AosiCoreHelper::init();
        $userHelper = new HrEduPersonHelper();

        $bindType = 'admin';

        $userHelper->validateUpdateRequest($request, $bindType);

        $user = Auth::user();

        $userDn = AosiCoreHelper::generateDnForUid($uid);

        $errorMessages = [];

        $normalizedModifiableAttributes = $userHelper->prepareModifiableAttributeValues($request, $bindType);

        if ($normalizedModifiableAttributes) {
            $data = [
                $userDn => $normalizedModifiableAttributes
            ];
//dd($data);
            $modifyResult = $aosiCore->ldapModifyAttribute(
                $user->uid[0],
                $user->password[0],
                $user->basedn[0],
                $data
            );

            if (!$modifyResult) {
                $errorMessages[] = array_pop(array_values($aosiCore->result(true)))['message'];
            }
        }

        // Hendlanje dodavanja binaries
        $normalizedModifiableBinaryAttributes = $userHelper->prepareModifiableBinaryAttributeValues($request, $bindType);
//dd($normalizedModifiableBinaryAttributes);
        if ($normalizedModifiableBinaryAttributes) {

            $data = [
                $userDn => $normalizedModifiableBinaryAttributes
            ];

            $addResult = $aosiCore->ldapAddAttribute(
                $user->uid[0],
                $user->password[0],
                $user->basedn[0],
                $data
            );

            if (!$addResult) {
                $errorMessages[] = head($aosiCore->result(true))['message'];
            }
        }

        // TODO mivanci javi pokušaj brisanja obaveznih atributa
        // TODO mivanci razmotri brisanje pomoću AJAXa

        // Atributi koji su prazni, a koji nisu obavezni, mogu se brisati bez davanja vrijednosti.
        $deletableAttributes = $userHelper->prepareDeletableAttributeValues($request, $bindType);
//dd($normalizedModifiableAttributes, $deletableAttributes);
        if ($deletableAttributes) {
            $data = [
                $userDn => $deletableAttributes
            ];

            $deleteResult = $aosiCore->ldapDeleteAttribute(
                $user->uid[0],
                $user->password[0],
                $user->basedn[0],
                $data
            );

            if (!$deleteResult) {
                $errorMessages[] = head($aosiCore->result())['message'] ?? 'Greška prilikom brisanja podataka';
            }
        }

        // Prikaži poruke o greškama prilikom svih operacija.
        if ($errorMessages) {
            flash('Došlo je do greške prilikom ažuriranja podataka.')->error();
            foreach ($errorMessages as $errorMessage) {
                flash($errorMessage)->warning()->important();
            }
        } else {
            flash('Podaci su ažurirani.')->success();
        }

        return redirect(route('admin.user.edit', ['uid' => $uid]));
    }

    public function pictureShow($uid, $hash)
    {
        $uid = htmlspecialchars($uid);
        $aosiCore = AosiCoreHelper::init();
        $user = Auth::user();

        $binSearchResult = $aosiCore->ldapBinSearch(
            $user->uid[0],
            $user->password[0],
            $user->basedn[0],
            '(uid=' . $uid .')',
            ['jpegphoto'],
            $hash
        );

        if (!$binSearchResult) {
            return response('', 404);
        }

        $image = head($binSearchResult)['jpegphoto'][0];
        return response(base64_decode($image), 200, ['Content-Type' => 'image/jpeg']);
    }

    public function pictureDelete($uid, $hash)
    {
        $uid = htmlspecialchars($uid);

        $aosiCore = AosiCoreHelper::init();

        $user = Auth::user();

        $userDn = AosiCoreHelper::generateDnForUid($uid);

        $data = [
            $userDn => ['jpegphoto' => [$hash]]
        ];

        $deleteResult = $aosiCore->ldapDeleteAttribute(
            $user->uid[0],
            $user->password[0],
            $user->basedn[0],
            $data
        );

        if (!$deleteResult) {
            return response()->json([
                'result' => 'error',
                'message' => $aosiCore->err_message(),
                'code' => $aosiCore->err_code(),
                'pictureHash' => $hash
            ], 404);
        }

        return response()->json(['result' => 'ok']);
    }

    /**
     * Resetiraj password za korisnika
     * @param $uid
     * @param Request $request
     * @return \Illuminate\Http\RedirectResponse|\Illuminate\Routing\Redirector
     * @throws \Exception
     */
    public function passwordReset($uid, Request $request)
    {
        $uid = htmlspecialchars($uid);

        $input = $request->validate([
            'userpassword' => 'required|string|max:128'
        ]);

        $aosiCore = AosiCoreHelper::init();

        $user = Auth::user();

        $userDn = AosiCoreHelper::generateDnForUid($uid);

        $data = [
            $userDn => [
                'userpassword' => [$input['userpassword']]
            ]
        ];

        $modifyResult = $aosiCore->ldapModifyAttribute(
            $user->uid[0],
            $user->password[0],
            $user->basedn[0],
            $data
        );

        if (!$modifyResult) {
            $message = head($aosiCore->result())['message'];
            flash($message)->error()->important();
            return redirect(route('admin.user.edit', ['uid' => $uid]));
        }

        flash('Zaporka promijenjena')->success();

        // Zapamti da je zaporka promijenjena za korisnika.
        AosiCoreHelper::cachePasswordResetInformation($uid, $input['userpassword']);

        return redirect(route('admin.user.edit', ['uid' => $uid]));
    }

    /**
     * Prikaži view za ispis obavijesti
     * @param $uid
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Http\RedirectResponse|\Illuminate\Routing\Redirector|\Illuminate\View\View
     * @throws \Exception
     */
    public function passwordResetNotice($uid)
    {
        $uid = htmlspecialchars($uid);

        $aosiCore = AosiCoreHelper::init();
        $cacheKey = AosiCoreHelper::generatePasswordResetCacheKey($uid);

        if (! cache()->has($cacheKey)) {
            flash('Ne postoji evidencija o promijenjenoj zaporci.')->warning()->important();
            return redirect(route('admin.user.update', ['uid' => $uid]));
        }

        $decryptedPassword = decrypt(cache($cacheKey));

        $user = Auth::user();

        $viewableAttributes = HrEduPerson::get_allowed_attrs('admin', 'view');

        $bindResult = $aosiCore->ldapSearch(
            $user->uid[0],
            $user->password[0],
            $user->basedn[0],
            '(uid=' . $uid .')',
            $viewableAttributes
        );

        if (!$bindResult) {
            flash('Korisnik s oznakom ' . $uid . ' ne postoji.')->warning()->important();
            return redirect(route('admin.user-list'));
        }

        $attributes = array_shift(array_values($bindResult));
        $ldapUser = new LdapUser($attributes);

        return view('admin.obavijestResetiranaZaporka', compact('ldapUser', 'decryptedPassword'));
    }

    /**
     * Prikaži formu za provjeru zaporke
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function passwordCheck()
    {
        // Potrebna inicijalizacija zbog učitavanja postavki o realmu.
        AosiCoreHelper::init();

        $isSingleRealmInstance = AosiCoreHelper::isSingleRealmInstance();
        $realms = AosiCoreHelper::getRealms();

        return view('admin.provjeraZaporke', compact('isSingleRealmInstance', 'realms'));
    }

    /**
     * Provjeri zaporku i flash-aj rezultat provjere
     * @param Request $request
     * @return \Illuminate\Http\RedirectResponse|\Illuminate\Routing\Redirector
     */
    public function passwordCheckResult(Request $request)
    {
        $aosiCore = AosiCoreHelper::init();

        $userHelper = new HrEduPersonHelper();

        $credentials =  $userHelper->validateLoginRequest($request);

        $baseDN = AosiCoreHelper::generateBaseDn($credentials['realm'] ?? null);

        // Radimo odmah userSearch() jer dobijemo i podatke o korisniku (a ne koristimo userBind()).
        $bindResult = $aosiCore->userSearch(
            $credentials['uid'],
            $credentials['password'],
            $baseDN,
            '(uid=' . $credentials['uid'] .')',
            ['uid']
        );

        // Ako bind nije uspješan, vrati se s greškom.
        if (!$bindResult) {
            flash('Neispravna korisnička oznaka ili zaporka.')->error()->important();
        } else {
            flash('Ispravna korisnička oznaka i zaporka.')->success()->important();

            // Ako user nije sam promijenio zaporku, metoda result() vraća array sa info o password policy, inače null.
            // Ovo se događa kada administrator korisniku postavi lozinku, pa ju korisnik mora sam promijeniti.
            if (AosiCoreHelper::isPasswordPolicyActive($bindResult)) {
                $passwordPolicy = head($bindResult)[AosiCoreHelper::PASSWORD_POLICY_INDICATOR_KEY];

                $message = $this->generatePasswordPolicyMessage($passwordPolicy);

                flash($message)->warning()->important();
            }
        }

        return redirect(route('admin.user.password.check'));
    }

    /**
     * Generiraj tekstualnu poruku u odnosu na podatke u $passwordPolicy
     * @param $passwordPolicy
     * @return string
     */
    protected function generatePasswordPolicyMessage($passwordPolicy)
    {
        $message = '';

        if (isset($passwordPolicy['is_expired']) && $passwordPolicy['is_expired']) {
            // Zaporka je istekla. Javi poruku ovisno o tome dali ima autentikacijskih pokušaja ili ne.
            if (isset($passwordPolicy['authn_left']) && $passwordPolicy['authn_left']) {
                $message = 'Zaporku je moguće koristiti još najviše ' . $passwordPolicy['authn_left'] .
                    ' puta! Ako se zaporka do tada ne promijeni, elektronički identitet bit će zaključan.';
            } else {
                $message = 'Zaporka je istekla. Administrator imenika ustanove mora otključati elektronički identitet.';
            }
        } elseif (isset($passwordPolicy['time_left']) && $passwordPolicy['time_left']) {
            // Zaporka još nije istekla...
            $message = 'Zaporka će isteći za ' . $passwordPolicy['time_left'] . '. Ako korisnik do tada ne ' .
                'promijeni zaporku, elektronički identitet bit će zaključan.';
        }

        return $message;
    }
}
