<?php

namespace App\Http\Controllers\Auth;

use aosi\HrEduPerson;
use App\Helpers\AosiCoreHelper;
use App\Helpers\HrEdu\HrEduPersonHelper;
use App\Http\Controllers\Controller;
use App\Extensions\LdapUser;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;

class LoginController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
    }

    public function userLoginView()
    {
        // Potrebna inicijalizacija zbog učitavanja postavki o realmu.
        AosiCoreHelper::init();

        $isSingleRealmInstance = AosiCoreHelper::isSingleRealmInstance();
        $realms = AosiCoreHelper::getRealms();

        return view('prijava', [
                'role' => 'user',
                'isSingleRealmInstance' => $isSingleRealmInstance,
                'realms' => $realms
            ]
        );
    }

    public function adminLoginView()
    {
        // Potrebna inicijalizacija zbog učitavanja postavki o realmu.
        AosiCoreHelper::init();

        $isSingleRealmInstance = AosiCoreHelper::isSingleRealmInstance();
        $realms = AosiCoreHelper::getRealms();

        return view('prijava', [
                'role' => 'admin',
                'isSingleRealmInstance' => $isSingleRealmInstance,
                'realms' => $realms
            ]
        );
    }

    protected function validateCredentials(Request $request)
    {
        $userHelper = new HrEduPersonHelper();
        return $userHelper->validateLoginRequest($request);
    }

    public function userLogin(Request $request)
    {
        $aosiCore = AosiCoreHelper::init();

        $credentials =  $this->validateCredentials($request);

        $baseDN = AosiCoreHelper::generateBaseDn($credentials['realm'] ?? null);

        // Napravi bind za user sučelje.
        // Bind type će biti naveden u atributima ulogiranog korisnika.
        // User koristi 'user*' metode.
        // Definiraj povratne atribute.
        $attributesToReturn = HrEduPerson::get_allowed_attrs('user', 'view');

        // Radimo odmah userSearch() jer dobijemo i podatke o korisniku (a ne koristimo userBind()).
        $bindResult = $aosiCore->userSearch(
            $credentials['uid'],
            $credentials['password'],
            $baseDN,
            '(uid=' . $credentials['uid'] .')',
            $attributesToReturn
        );

        // Ako bind nije uspješan, vrati se s greškom.
        if (!$bindResult) {
            flash('Prijava nije uspjela. Pokušajte ponovno.')->error()->important();
            return redirect(route('user.login.view'));
        }

        // Korisnik je uspješno bindan, napravi sjednicu.
        $attributes['uid'][0] = $credentials['uid'];
        // Password moramo sačuvati zbog budućih zahtjeva prema LDAPu
        $attributes['password'][0] = $credentials['password'];
        // Base DN moramo sačuvati zbog budućih zahtjeva prema LDAPu.
        $attributes['basedn'][0] = $baseDN;
        // Bind Type čuvamo zbog kasnije autorizacije korisnika.
        $attributes['bindtype'][0] = 'user';

        // Custom implementacija Authenticatable je app/Extensions/LdapUser.php
        $ldapUser = new LdapUser($attributes);

        // Custom implementacija Guard-a je app/Extensions/LdapGuard.php
        Auth::login($ldapUser);

        // Ako user nije sam promijenio zaporku, podaci će sadržavati podatke o aktivnoj password policy.
        // Ovo se događa kada administrator korisniku postavi lozinku, pa ju korisnik mora sam promijeniti.
        if (AosiCoreHelper::isPasswordPolicyActive($bindResult)) {
            AosiCoreHelper::setPasswordChangeRequirement(true);
            return redirect(route('user.password.edit'));
        }

        flash('Prijava je uspjela.')->success();
        return redirect(route('user.edit'));
    }

    public function adminLogin(Request $request)
    {
        $aosiCore = AosiCoreHelper::init();

        $credentials =  $this->validateCredentials($request);

        $baseDN = AosiCoreHelper::generateBaseDn($credentials['realm'] ?? null);

        // Napravi bind za admin sučelje.
        // Bind type će biti naveden u atributima ulogiranog korisnika.
        // Admin koristi 'ldap*' metode.
        $bindResult = $aosiCore->ldapBind($credentials['uid'], $credentials['password'], $baseDN);

        // Ako je false, bind je bio neuspješan. Ako je 0, nema pravo administracije imenika.
        if (! boolval($bindResult)) {
            if ($bindResult === false) {
                $message = 'Prijava nije uspjela. Pokušajte ponovno.';
            } elseif ($bindResult === 0) {
                $message = 'Prijava je uspjela, ali nemate pravo administracije imenika.';
            }

            flash($message)->error()->important();

            return redirect(route('admin.login.view'));
        }

        // Korisnik je uspješno bindan, napravi sjednicu.
        $attributes['uid'][0] = $credentials['uid'];
        $attributes['password'][0] = $credentials['password'];
        $attributes['basedn'][0] = $baseDN;
        $attributes['bindtype'][0] = 'admin';

        $attributes['adminlevel'][0] = $bindResult;
        // Ako je adminlevel 3, to je admin sa ograničenjima, pa ih zabilježi.
        if ($bindResult == 3) {
            $attributes['limitations'] = $aosiCore->result();
        }

        // Custom implementacija Authenticatable je app/Extensions/LdapUser.php
        $ldapUser = new LdapUser($attributes);

        // Custom implementacija Guard-a je app/Extensions/LdapGuard.php
        Auth::login($ldapUser);

        flash('Prijava je uspjela.')->success();
        return redirect(route('admin.user.list.show'));
    }

    public function logout()
    {
        Auth::logout();
        flash('Odjava je uspjela.')->success();
        return redirect(route('home'));
    }
}
