<?php

namespace App\Wrapper;

use Illuminate\Http\Request;
use Carbon\Carbon;
use Aosi\Aosi_core;

use aosi\Config;
use aosi\HrEduOrg;
use aosi\HrEduPerson;
use aosi\Logger;

class AosiWrapper
{

	private $core;

		function __construct() {
	
	
        		Config::load();
		        Logger::prepare();
		        ini_set('date.timezone',Config::get('timezone'));

		        HrEduPerson::l();
		        HrEduOrg::l();
	
	
	
			$this->core = new \aosi\Aosi_core();
		}




	/**
	 *
	 * Funkcija sluzi anonimnom dohvatu binarnih atributa. Vraca njihovu base64 enkodiranu vrijednost.
	 *
	 * @param string $base
	 * @param string $filter
	 * @param string $attributes
	 * @param string $md5_base64
	 * @return array $result
	 * */
	public function anonBinSearch($base,$filter, $attribute , $md5_base64 = FALSE )
	{

		$attributes = explode(',', $attribute);

		$anonBinSearch = $this->core->anonBinSearch($base,$filter,$attributes, $md5_base64);

		if(!$anonBinSearch){
			$code = $this->core->err_code();
			$message = $this->core->err_message();

		}else{
		$code = 0;
		}

		$r = $this->array_to_xml($anonBinSearch);
		

		return [ 'code' => $code, 'error' => $message, 'result' => $r];

	}



	/**
	 * Funkcija vraca zapise koji zadovoljavaju uvjete pretrage zadane ldap filterom. Za binarne atribute vraca md5 hash njihove vrijednosti.
	 *
	 * @param string $base
	 * @param string $filter
	 * @param string $attributes
	 * @return array $result
	 * */



	public function anonSearch($base,$filter, $attribute)
	{

		$attributes = explode(',', $attribute);

		$anonSearch = $this->core->anonSearch($base,$filter,$attributes);

		if(!$anonSearch){
			$code = $this->core->err_code();
			$message = $this->core->err_message();

		}else{
		$code = 0;
		}

		$r = $this->array_to_xml($anonSearch);
		
		return [ 'code' => $code, 'error' => $message, 'result' => $r];

	}




	/**
	 * Funkcija sluzi dodavanju atributa u korisnicki zapis u LDAP imeniku u ime administratora. Moze dodati jedan ili vise atributa za jednog ili vise korisnika.
	 *
	 * @param string $user
	 * @param string $password
	 * @param string $base
	 * @param string xml
	 * @return array $result
	 * */
	public function ldapAddAttribute($user,$password,$base,$xml)
	{

		$data = $this->xml_to_array($xml);

		$ldapAddAttribute = $this->core->ldapAddAttribute($user,$password,$base,$data);
		//$r=$ldapAddAttribute;

		if(!$ldapAddAttribute){
			$code = "1"; //$this->core->err_code();
			$message = "Dodavanje barem jednog atributa nije uspjelo"; //$this->core->err_message();

			$r =  $this->array_to_xml($this->core->result());	
		}else{
		$code = 0;
		}

//		$r = $this->array_to_xml($ldapAddAttribute);
		
		$r =  $this->array_to_xml($this->core->result());	

		return [ 'code' => $code, 'error' => $message, 'result' => $r];

	}



	/**
	 * Funkcija služi dodavanju korisnika u LDAP imenik. Moze dodati jednog ili vise korisnika ovisno o ulaznim podacima. Ulazni podaci moraju sadrzavati sve (obavezne)  atribute.
	 *
	 * @param string $user
	 * @param string $password
	 * @param string $base
	 * @param string xml
	 * @return array $result
	 * */
	public function ldapAddUser($user,$password,$base,$xml)
	{

		$data = $this->xml_to_array($xml);

		$ldapAddUser = $this->core->ldapAddUser($user,$password,$base,$data);

		if(!$ldapAddUser){

			//$code = $this->core->err_code();
			//$message = $this->core->err_message();
			$code = "1";
			$message = "Dodavanje barem jednog korisnika nije uspjelo.";

		}else{
		$code = 0;
		}

		//$r = $this->array_to_xml($ldapAddUser);
		$r =  $this->array_to_xml($this->core->result());	
		
		return [ 'code' => $code, 'error' => $message, 'result' => $r];

	}



	/**
	 * Funkcija služi dodavanju korisnika u LDAP imenik. Moze dodati jednog ili vise korisnika ovisno o ulaznim podacima. Ova funkcija ne zahtjeva sve obavezne atribute, nego samo njihov dio.
	 * @param string $user
	 * @param string $password
	 * @param string $base
	 * @param string xml
	 * @return array $result
	 * */
	public function ldapAddUserLE($user,$password,$base,$xml)
	{

		$data = $this->xml_to_array($xml);

		$ldapAddUserLE = $this->core->ldapAddUserLE($user,$password,$base,$data);

		if(!$ldapAddUserLE){
			//provjeriti zasto core ne vraca code i message
			//$code = $this->core->err_code();
			//$message = $this->core->err_message();
			$code = -1;
			$message = "dodavanje barem jednog korisnika nije uspjelo.";
			$ldapAddUserLE =  $this->core->result();


		}else{
		$code = 0;
		}

		$r = $this->array_to_xml($ldapAddUserLE);
		
		return [ 'code' => $code, 'error' => $message, 'result' => $r];

	}




	/**
	 * Funkcija služi dohvatu informacija o administratoru imenika
	 *
	 * @param string $user
	 * @param string $password
	 * @param string $base
	 * @param string uid
	 * @return array $result
	 * */
	public function ldapAdminInfo($user,$password,$base,$uid)
	{


		$ldapAdminInfo = $this->core->ldapAdminInfo($user,$password,$base,$uid);

		if(!$ldapAdminInfo){
			$code = $this->core->err_code();
			$message = $this->core->err_message();

		}else{
		$code = 0;
		}

//		$r = $this->array_to_xml($ldapAdminInfo);
		$r = $ldapAdminInfo;		
	//dodati obradu rezultata ako core vraca 3, koji format rezultata?!	
		return [ 'code' => $code, 'error' => $message, 'result' => $r];

	}




	/**
	 * Funkcija primarno služi autentikaciji / autorizaciji administratora imenika.
	 *
	 * @param string $user
	 * @param string $password
	 * @param string $base
	 * @return array $result
	 * */
	public function ldapBind($user,$password,$base)
	{


		$ldapBind = $this->core->ldapBind($user,$password,$base);

		if(!$ldapBind){
			$code = $this->core->err_code();
			$message = $this->core->err_message();

		}else{
		$code = 0;
		}

//		$r = $this->array_to_xml($ldapBind);
		$r = $ldapBind;

		return [ 'code' => $code, 'error' => $message, 'result' => $r];

	}



	/**
	 * Funkcija sluzi dohvatu binarnih atributa. Vraca njihovu base64 enkodiranu vrijednost.
	 *
	 * @param string $user
	 * @param string $password
	 * @param string $base
	 * @param string $filter
	 * @param string $attributes
	 * @param string $md5_base64
	 * @return array $result
	 * */
	public function ldapBinSearch($user,$password,$base,$filter, $attribute , $md5_base64=FALSE )
	{

		$attributes = explode(',', $attribute);

		$ldapBinSearch = $this->core->ldapBinSearch($user,$password,$base,$filter,$attributes, $md5_base64);
		if(!$ldapBinSearch){
			$code = $this->core->err_code();
			$message = $this->core->err_message();

		}else{
		$code = 0;
		}

		$r = $this->array_to_xml($ldapBinSearch);

		return [ 'code' => $code, 'error' => $message, 'result' => $r];

	}



	/**
	 * Funkcija služi brisanju neke vrijednosti nekog atributa korisnika iz LDAP imenika u ime administratora. Moze dodati izbrisati vise vrijednosti vise atributa za vise korisnika.
	 *
	 * @param string $user
	 * @param string $password
	 * @param string $base
	 * @param string $xml
	 * @return array $result
	 * */
	public function ldapDeleteAttribute($user,$password,$base,$xml )
	{

		$data = $this->xml_to_array($xml);

		$ldapDeleteAttribute = $this->core->ldapDeleteAttribute($user,$password,$base,$data);
		if(!$ldapDeleteAttribute){
			$code = 1; //$this->core->err_code();
			$message = "Brisanje barem jednog atributa nije uspjelo."; //$this->core->err_message();

		}else{
		$code = 0;
		}

		//$r = $this->array_to_xml($ldapDeleteAttribute);
		$r = $this->core->result();
		$r = $this->array_to_xml($r);
		return [ 'code' => $code, 'error' => $message, 'result' => $r];

	}




	/**
	 * Funkcija služi brisanju korisnika iz LDAP imenika u ime administratora. Moze dodati izbrisati više korisnika.
	 *
	 * @param string $user
	 * @param string $password
	 * @param string $base
	 * @param string $xml
	 * @return array $result
	 * */
	public function ldapDeleteUser($user,$password,$base,$xml )
	{

		$data = $this->xml_to_array($xml);
		
		//$data = 
//		return $data;

		$data = explode(':', $xml);
		$ldapDeleteUser = $this->core->ldapDeleteUser($user,$password,$base,$data);
		$r = $ldapDeleteUser;
		if(!$ldapDeleteUser){
			$code = "1"; //$this->core->err_code();
			$message = "Brisanje barem jednog korisnika nije uspjelo."; //;$this->core->err_message();

		$r = $this->array_to_xml($this->core->result());
		}else{
		$code = 0;
		}


		return [ 'code' => $code, 'error' => $message, 'result' => $r];

	}



	/**
	 * Funkcija vraca zapise koji zadovoljavaju uvjete pretrage zadane ldap filterom i omogucuje paginaciju.
	 *
	 * @param string $user
	 * @param string $password
	 * @param string $base
	 * @param string $attribute
	 * @param string $from
	 * @param string $size
	 * @return array $result
	 * */
	public function ldapList($user,$password,$base,$filter,$attribute,$from,$size )
	{

		$attributes = explode(',', $attribute);

		$ldapList = $this->core->ldapList($user,$password,$base,$filter,$attributes,$from,$size);
		if(!$ldapList){
			$code = $this->core->err_code();
			$message = $this->core->err_message();

		}else{
		$code = 0;
		}

		$r = $this->array_to_xml($ldapList);
		$count = $this->core->result()['count'];

//dodati code, from, size iz corea
		return [ 'code' => $code, 'count' => $count, 'error' => $message, 'from' => $from,  'result' => $r, 'size' => $size ];

	}



	/**
	 * Funkcija služi promjeni vrijednosti atributa korisnika iz LDAP imenika u ime administratora. Moze promijeniti vrijednost vise atributa za vise korisnika.
	 *
	 * @param string $user
	 * @param string $password
	 * @param string $base
	 * @param string $xml
	 * @return array $result
	 * */
	public function ldapModifyAttribute($user,$password,$base,$xml )
	{

		$data = $this->xml_to_array($xml);

		$ldapModifyAttribute = $this->core->ldapModifyAttribute($user,$password,$base,$data);
		if(!$ldapModifyAttribute){
			$code = $this->core->err_code();
			$message = $this->core->err_message();

		}else{
		$code = 0;
		}

		//$r = $this->array_to_xml($ldapModifyAttribute);
		$r = $this->array_to_xml($this->core->result());

		return [ 'code' => $code, 'error' => $message, 'result' => $r];

	}



	/**
	 * Funkcija služi dohvatu org zapisa u ime administratora.
	 *
	 * @param string $user
	 * @param string $password
	 * @param string $base
	 * @return array $result
	 * */
	public function ldapOrgInfo($user,$password,$base)
	{


		$ldapOrgInfo = $this->core->ldapOrgInfo($user,$password,$base);

		if(!$ldapOrgInfo){
			$code = $this->core->err_code();
			$message = $this->core->err_message();

		}else{
		$code = 0;
		}

		$r = $this->array_to_xml($ldapOrgInfo);

		return [ 'code' => $code, 'error' => $message, 'result' => $r];

	}



	/**
	 * Funkcija vraca zapise koji zadovoljavaju uvjete pretrage zadane ldap filterom. Za binarne atribute vraca md5 hash njihove vrijednosti.
	 * @param string $user
	 * @param string $password
	 * @param string $base
	 * @param string $filter
	 * @param string $attributes
	 * @return array $result
	 * */
	public function ldapSearch($user,$password,$base,$filter,$attributes)
	{
		
		$attributes = explode(',', $attributes);

		$ldapSearch = $this->core->ldapSearch($user,$password,$base,$filter,$attributes);

		if(!$ldapSearch){
			$code = $this->core->err_code();
			$message = $this->core->err_message();

		}else{
		$code = 0;
		}

		$r = $this->array_to_xml($ldapSearch);
		
		

		return [ 'code' => $code, 'error' => $message, 'result' => $r];

	}



	/**
	 * Funkcija sluzi provjeri postoji li korisnik u imeniku u ime administratora.
	 *
	 * @param string $user
	 * @param string $password
	 * @param string $base
	 * @param string uid
	 * @return array $result
	 * */
	public function ldapUserExists($user,$password,$base,$uid)
	{


		$ldapUserExists = $this->core->ldapUserExists($user,$password,$base,$uid);

		if(!$ldapUserExists){
			$code = $this->core->err_code();
			$message = $this->core->err_message();

		}else{
		$code = 0;
		}

		$r = $this->array_to_xml($ldapUserExists);
		return [ 'code' => $code, 'error' => $message, 'result' => $r];

	}




	/**
	 * Funkcija služi dodavanju atributa u korisnicki zapis u LDAP imeniku u ime korisnika.
	 *
	 * @param string $user
	 * @param string $password
	 * @param string $base
	 * @param string xml
	 * @return array $result
	 * */
	public function userAddAttribute($user,$password,$base,$xml)
	{

		$data = $this->xml_to_array($xml);

		$userAddAttribute = $this->core->userAddAttribute($user,$password,$base,$data);
		//$r=$ldapAddAttribute;

		if(!$userAddAttribute){
			$code = "1"; //$this->core->err_code();
			$message = "Dodavanje barem jednog atributa nije uspjelo"; //$this->core->err_message();

			$r =  $this->array_to_xml($this->core->result());	
		}else{
		$code = 0;
		}

//		$r = $this->array_to_xml($ldapAddAttribute);
		
		$r =  $this->array_to_xml($this->core->result());	

		return [ 'code' => $code, 'error' => $message, 'result' => $r];

	}




	/**
	 * Funkcija primarno služi autentikaciji korisnika.
	 *
	 * @param string $user
	 * @param string $password
	 * @param string $base
	 * @return array $result
	 * */
	public function userBind($user,$password,$base)
	{


		$userBind = $this->core->userBind($user,$password,$base);

		if(!$userBind){
			$code = $this->core->err_code();
			$message = $this->core->err_message();

		}else{
		$code = 0;
		}

		$r = $this->array_to_xml($userBind);
		
		
		//ako korisnik nije promijenio password
		if($this->core->result()){

			$rezultat = $this->core->result();
		
			if($rezultat['is_exipred'] == 0){ $poruka = 'Lozinka istice za '.$rezultat['time_left'];	}
			if($rezultat['is_exipred'] == 1){ $poruka = 'Lozinka je istekla! Preostaje jos '.$rezultat['authn_left'].' autentikacija ovom lozinkom.';	}
			
			$poruka = ['pwdPolicySubEntry' => $poruka];	
			$r = $this->array_to_xml( [$uid.','.$base =>$poruka]);
		 

		}

		return [ 'code' => $code, 'error' => $message, 'result' => $r];



	}



	/**
	 *
	 * Funkcija sluzi dohvatu binarnih atributa. Vraca njihovu base64 enkodiranu vrijednost. 
	 * @param string $user
	 * @param string $password
	 * @param string $base
	 * @param string $filter
	 * @param string $attributes
	 * @param string $md5_base64
	 * @return array $result
	 * */
	public function userBinSearch($user,$password,$base,$filter, $attribute , $md5_base64=FALSE )
	{

		$attributes = explode(',', $attribute);

		$userBinSearch = $this->core->userBinSearch($user,$password,$base,$filter,$attributes, $md5_base64);
		if(!$userBinSearch){
			$code = $this->core->err_code();
			$message = $this->core->err_message();

		}else{
		$code = 0;
		}

		$r = $this->array_to_xml($userBinSearch);

		return [ 'code' => $code, 'error' => $message, 'result' => $r];

	}



	/**
	 *
	 * Funkcija služi brisanju neke vrijednosti nekog atributa korisnika iz LDAP imenika u ime korisnika. Moze dodati izbrisati vise vrijednosti vise atributa za danog korisnika.
	 *
	 * @param string $user
	 * @param string $password
	 * @param string $base
	 * @param string $xml
	 * @return array $result
	 * */
	public function userDeleteAttribute($user,$password,$base,$xml )
	{

		$data = $this->xml_to_array($xml);

		$userDeleteAttribute = $this->core->userDeleteAttribute($user,$password,$base,$data);
		if(!$userDeleteAttribute){
			$code = 1; //$this->core->err_code();
			$message = "Brisanje barem jednog atributa nije uspjelo."; //$this->core->err_message();

		}else{
		$code = 0;
		}

		//$r = $this->array_to_xml($ldapDeleteAttribute);
		$r = $this->core->result();
		$r = $this->array_to_xml($r);
		return [ 'code' => $code, 'error' => $message, 'result' => $r];

	}



	/**
	 *
	 * Funkcija služi promjeni vrijednosti atributa korisnika. Moze promijeniti vrijednost vise atributa. Promjena se vrši metodom replace sto znaci da se za neki atribut izbrisu sve stare vrijednosti. Ako atribut ima vise vrijednosti potrebno je poslati SVE nove vrijednosti.
	 *
	 * @param string $user
	 * @param string $password
	 * @param string $base
	 * @param string $xml
	 * @return array $result
	 * */
	public function userModifyAttribute($user,$password,$base,$xml )
	{

		$data = $this->xml_to_array($xml);

		$userModifyAttribute = $this->core->userModifyAttribute($user,$password,$base,$data);
		if(!$userModifyAttribute){
			$code = 1; //$this->core->err_code();
			$message = "Izmjena barem jednog atributa nije uspjela."; //$this->core->err_message();

		}else{
		$code = 0;
		}

		$r = $this->core->result();
		$r = $this->array_to_xml($r);
		return [ 'code' => $code, 'error' => $message, 'result' => $r];


	}




	/**
	 *
	 * Funkcija služi dohvatu org zapisa u ime korisnika.
	 *
	 * @param string $user
	 * @param string $password
	 * @param string $base
	 * @return array $result
	 * */
	public function userOrgInfo($user,$password,$base )
	{


		$userOrgInfo = $this->core->userOrgInfo($user,$password,$base);
		if(!$userOrgInfo){
			$code = $this->core->err_code();
			$message = $this->core->err_message();

		}else{
		$code = 0;
		}


		$r = $this->array_to_xml($userOrgInfo);

		return [ 'code' => $code, 'error' => $message, 'result' => $r];

	}





	/**
	 * Funkcija vraca zapise koji zadovoljavaju uvjete pretrage zadane ldap filterom. Za binarne atribute vraca md5 hash njihove vrijednosti.
	 * @param string $user
	 * @param string $password
	 * @param string $base
	 * @param string $filter
	 * @param string $attributes
	 * @return array $result
	 * */
	public function userSearch($user,$password,$base,$filter,$attributes)
	{
		
		$attributes = explode(',', $attributes);

		$userSearch = $this->core->userSearch($user,$password,$base,$filter,$attributes);

		if(!$userSearch){
			$code = $this->core->err_code();
			$message = $this->core->err_message();

		}else{
		$code = 0;
		}

		$r = $this->array_to_xml($userSearch);
		
		
		//ako korisnik nije promijenio password
		if($this->core->result()){

			$rezultat = $this->core->result();
		
			if($rezultat['is_exipred'] == 0){ $poruka = 'Lozinka istice za '.$rezultat['time_left'];	}
			if($rezultat['is_exipred'] == 1){ $poruka = 'Lozinka je istekla! Preostaje jos '.$rezultat['authn_left'].' autentikacija ovom lozinkom.';	}
			
			$poruka = ['pwdPolicySubEntry' => $poruka];	
			$r = $this->array_to_xml( [$uid.','.$base =>$poruka]);
		 

		}

		return [ 'code' => $code, 'error' => $message, 'result' => $r];

	}























	//---------------------------------------------------
	

	private function array_to_xml($attrarray){
        $xml='<ldap>';
        foreach ($attrarray as $dn=>$entry){
            $xml.='<entry dn="' . $dn .'">';
           
            foreach ($entry as $ldapname=>$value){
                $xml.='<attribute ldapname="' . $ldapname .'">';
                if(is_array($value)){
                        foreach ($value as $key=>$val){
                            $val=htmlspecialchars ($val,ENT_QUOTES,'UTF-8');
                            $xml.="<value>$val</value>";
                        }
                } else {
                    $value=htmlspecialchars ($value,ENT_QUOTES,'UTF-8');
                    $xml.="<value>" . $value . "</value>";
                }
                $xml.="</attribute>";
            }
            $xml.='</entry>';
        }
        $xml.='</ldap>';
        return $xml;
	}

    private function xml_to_array($xml,$utf8_decode_to=false){
       
        $p = xml_parser_create('UTF-8');
        xml_parser_set_option($p, XML_OPTION_CASE_FOLDING, 0);
        xml_parser_set_option($p, XML_OPTION_SKIP_WHITE, 1);
        xml_parse_into_struct($p, $xml, $vals);
        xml_parser_free($p);
        //print_r($vals);
        foreach($vals as $val){
            switch ($val['tag']){
                case 'entry':
                    switch ($val['type']){
                        case 'open':
                            $dn=$val['attributes']['dn'];
                            $admintype=$val['attributes']['admintype'];
                            $attributes=array();
                        break;
                        case 'complete':
                            if(isset($val['attributes']['dn'])){
                                $usrlist[$val['attributes']['dn']]=array();
                                if(isset($val['attributes']['admintype'])){
                                 $usrlist[$val['attributes']['dn']]['aosi_admintype']=$val['attributes']['admintype'];
                                }
                            }
                        break;
                        case 'close':
                            $usrlist[$dn]=$attributes;
                            if($admintype){
                                $usrlist[$dn]['aosi_admintype']=$admintype;
                            }
                            unset($admintype);
                        break;
                    }
                    break;
                case 'urn':
                    if($val['type']=='open' || $val['type']=='complete'){
                        $urnpair=array();
                    } elseif ($val['type']=='close'){
                        array_push($attributes,$urnpair);
                    }
                    break;
                case 'urn_value':
                    if(!is_array($urnpair[$val['attributes']['ldapname']])){
                        $urnpair[$val['attributes']['ldapname']]=array();
                    }
                    if($utf8_decode_to && gettype($val['value'])=='string'){
                     array_push($urnpair[$val['attributes']['ldapname']],iconv('UTF-8',$utf8_decode_to,trim($val['value'])));
                    } else {
                     array_push($urnpair[$val['attributes']['ldapname']],trim($val['value']));
                    }
                    break;
                case 'attribute':
                    if($val['type']=='open' || $val['type']=='complete'){
                        $ldapname=$val['attributes']['ldapname'];
                        unset($attrvalue);
                    } elseif ($val['type']=='close'){
                        $attributes[$ldapname]=$attrvalue;
                    }
                    break;
                case 'value':
                    if($utf8_decode_to && gettype($val['value'])=='string'){
                     $attrvalue[]=iconv('UTF-8',$utf8_decode_to,htmlspecialchars_decode(trim($val['value']), ENT_QUOTES));
                    } else {
                        $attrvalue[]=htmlspecialchars_decode(trim($val['value']), ENT_QUOTES);
                    }
                    break;
            }   
        }
        return $usrlist;
    } 
    } 
