<?php

/**
 * Formatiraj string datuma
 *
 * @param string $dateTimeString Datum .
 * @param string $fromFormat Trenutni format datuma.
 * @param string $toFormat Željeni format datuma.
 * @return null|string
 * @throws Exception
 */
function formatDateString($dateTimeString, $fromFormat = 'ldap', $toFormat = 'hr')
{
    // Dostupni formati, ažurirati po potrebi.
    $formats = [
        'hr' => 'd.m.Y.',
        'ldap' => 'Ymd'
    ];

    // Osnovna validacija parametra.
    if (!$dateTimeString || !array_key_exists($fromFormat, $formats) || !array_key_exists($toFormat, $formats)) {
        throw new Exception('Dani formati za konverziju datuma nisu ispravni.');
    }

    // Ako je vrijednost 'NONE' nema konverzije
    if (strcasecmp($dateTimeString,'NONE') == 0) {
        return 'NONE';
    }

    // Napravi instancu u danom formatu. Create datetime for given $dateTimeString which is in given $fromFormat format.
    $dateTime = \DateTime::createFromFormat($formats[$fromFormat], $dateTimeString);

    // Ako kreiranje instance nije uspjelo, vrati prazan string.
    if (!$dateTime) {
        return '';
    }

    // Konvertiraj u željeni format.
    return $dateTime->format($formats[$toFormat]);
}

/**
 * Generiraj URL do logotipa obzirom na postojanje ključa u cache-u. Ovo je napravljeno tako kako bi se napravio
 * cache bust nakon uploada nove slike (generira se novi ključ, a posljedično novi URL), pa preglednik uvijek
 * dohvati novu sliku.
 * @return string
 * @throws Exception
 */
function getLogoUrl()
{
    return route('logo', ['cacheKey' => cache(\App\Http\Controllers\Admin\LogoController::LOGO_CACHE_KEY) ?? 'initial']);
}