/**
 * Created by PhpStorm.
 * User: Matija Lovrić
 * Date: 15.3.2018.
 */

/*
 * Default-postavke za svaku instancu DataTable
 */
$.extend( $.fn.dataTable.defaults, {
    language: {
        emptyTable:      "Nema podataka",
        info:            "Prikazano _START_ do _END_ od _TOTAL_ redaka",
        infoEmpty:       "Prikazano 0 do 0 od 0 redaka",
        infoFiltered:    "(filtrirano iz _MAX_ ukupnih redaka)",
        infoPostFix:     "",
        infoThousands:   ",",
        lengthMenu:      "Prikaži _MENU_ redaka po stranici",
        loadingRecords:  "Dohvaćam...",
        processing:      "Obrađujem...",
        search:          "Filtriraj:",
        zeroRecords:     "Ništa nije pronađeno",
        paginate: {
            first:       "Prva",
            previous:    "Nazad",
            next:        "Naprijed",
            last:        "Zadnja"
        },
        aria: {
            sortAscending:  ": aktiviraj za rastući poredak",
            sortDescending: ": aktiviraj za padajući poredak"
        },
        buttons: {
            copy: "Kopiraj",
            colvis: 'Odabir stupaca'
        }
    }
});

/*
 * Inicijalizira DataTable instancu s ekstenzijama sučelja te standardnim dodatno biranim postavkama.
 *
 * Ekstenzije sučelja:
 *  - Dodatni JSON parametri za pozicioniranje elemenata sučelja lijevo ili desno
 *  - Gumb za pražnjenje polja za filtriranje
 *  - Skrivanje paginacije ako postoji samo jedna stranica
 *  - Pretvaranje cijelog retka u klikabilnu poveznicu na URL sadržan u tom retku
 */
$.fn.dataTableExtended = function (options) {
    this.each(function () {
        var table = $(this);
        var data = {
            initComplete: function () {
                // If option is set, wrap in table-responsive wrapper to make table overflow horizontally,
                // otherwise, make it shrink on smaller screens
                if (options.overflow) {
                    table.wrap('<div class="table-responsive"></div>');
                    table.css('width', table.width() - 2); // Width fix for a responsive table
                }
                else {
                    table.css('width', 'auto');
                }

                var id = table.prop('id');
                var id_index = id.split('_')[2];
                var tab_wrapper = table.closest('.dataTables_wrapper');
                // Get all filter elements belonging to this DataTable instance
                var tab_filter = tab_wrapper.find('.dataTables_filter');
                // Get all length menu elements belonging to this DataTable instance
                var tab_length = tab_wrapper.find('.dataTables_length');
                // Get all info elements belonging to this DataTable instance
                var tab_info = tab_wrapper.find('.dataTables_info');
                // Get all pagination elements belonging to this DataTable instance
                var tab_pager = tab_wrapper.find('.dataTables_paginate');
                // Get all button groups belonging to this DataTable instance
                var tab_buttons = tab_wrapper.find('.dt-buttons');

                if (options.filterPosition) {
                    tab_filter.css('float', options.filterPosition);
                }

                // Limit search action to words of preset minumum length
                if (options.minFilterLength) {
                    var filterInput = tab_filter.find('input');
                    filterInput.off().on('keyup', function () {
                        var searchValue = this.value.trim();
                        if (searchValue.length >= options.minFilterLength)
                            table.DataTable().search(searchValue, false, false).draw();
                    });
                }

                if (options.lengthPosition) {
                    tab_length.css('float', options.lengthPosition);
                }

                if (options.infoPosition) {
                    tab_info.css('float', options.infoPosition);
                    tab_info.css('margin-bottom', '14px');
                }

                if (options.pagerPosition) {

                    tab_pager.css('float', options.pagerPosition);
                }

                tab_buttons.attr('id', 'dt-buttons_' + id_index);
                if (options.buttonsPosition)
                    tab_buttons.css('float', options.buttonsPosition);

                // Add FontAwesome icons to buttons
                if (options.faButtons) {
                    tab_buttons.find('.buttons-copy span').prepend('<i class="fas fa-copy"></i>&nbsp;');
                    tab_buttons.find('.buttons-pdf span').prepend('<i class="fas fa-file-pdf"></i>&nbsp;');
                }

                // If table is responsive, reserve space for the colvis dropdown if it's positioned below the table
                if (table.parents('.table-responsive').length && table.nextAll('#dt-buttons_' + id_index).find('.buttons-colvis').length) {
                    var height = tab_buttons.height() * (options.columns.length + 1) + 5;
                    tab_buttons.wrap('<div style="height: ' + height + 'px;"></div>');
                }

                if (options.smallButtons)
                    tab_buttons.find('button').addClass('btn-sm');

                // Add a tooltip and a "Clear" button to DataTable search field
                var search_field = tab_filter.find('input[type=search]');
                search_field.prop('title', 'Utipkajte dijelove riječi po kojima želite filtrirati prikaz');
                search_field.wrap('<div class="input-group" style="display: inline-flex;"></div>');
                search_field.after(
                    '<div class="input-group-append" style="width: auto;">' +
                    '<button id="' + id + '_clear_search" class="btn btn-sm btn-outline-secondary input-sm" type="button" title="Obriši uneseni pojam">' +
                    '<i class="fas fa-times"></i>' +
                    '</button>' +
                    '</div>'
                );

                // Make the "Clear" button do it's job
                search_field.next().find('button#' + id + '_clear_search').click(function () {
                    search_field.val('');
                    table.DataTable().search('', false, false).draw();
                    search_field.focus();
                });
            },
            fnDrawCallback: function(oSettings) {
                // Get all pagination elements belonging to this DataTable instance
                var pager = table.closest('.dataTables_wrapper').find('.dataTables_paginate');

                // If there is only one page, remove the pager
                if ( oSettings._iDisplayLength > 0 && oSettings._iDisplayLength < oSettings.fnRecordsDisplay() ) {
                    pager.show();
                }
                else {
                    pager.hide();
                }
                $('.data-table-wrapper').show();
            }
        };

        // Create a DataTable instance
        table.dataTable($.extend(data, options));
    });
};

//
// Funkcija za cacheiranje stranica za paginaciju pri server-side obradi. Radi samo s pagerom;
// sortiranje i filtriranje rezultiraju novim setovima podataka, pa se šalju novi upiti na server.
//
// Paging caching pipeline function for DataTables. To be used to the `ajax` option of DataTables
//
$.fn.dataTable.pageCachingPipeline = function ( opts ) {
    // Default configuration options
    var conf = $.extend( {
        pages: 5,     // number of pages to cache
        url: '',      // script url
        data: null,   // function or object with parameters to send to the server
                      // matching how `ajax.data` works in DataTables
        type: 'GET' // Ajax HTTP method
    }, opts );

    // Private variables for storing the cache
    var cacheLower = -1;
    var cacheUpper = null;
    var cacheLastRequest = null;
    var cacheLastJson = null;

    return function ( request, drawCallback, settings ) {
        var ajax          = false;
        var requestStart  = request.start;
        var drawStart     = request.start;
        var requestLength = request.length;
        var requestEnd    = requestStart + requestLength;

        if ( settings.clearCache ) {
            // API requested that the cache be cleared
            ajax = true;
            settings.clearCache = false;
        }
        else if ( cacheLower < 0 || requestStart < cacheLower || requestEnd > cacheUpper ) {
            // outside cached data - need to make a request
            ajax = true;
        }
        else if ( JSON.stringify( request.order )   !== JSON.stringify( cacheLastRequest.order ) ||
            JSON.stringify( request.columns ) !== JSON.stringify( cacheLastRequest.columns ) ||
            JSON.stringify( request.search )  !== JSON.stringify( cacheLastRequest.search )
        ) {
            // properties changed (ordering, columns, searching)
            ajax = true;
        }

        // Store the request for checking next time around
        cacheLastRequest = $.extend( true, {}, request );

        if ( ajax ) {
            // Need data from the server
            if ( requestStart < cacheLower ) {
                requestStart = requestStart - (requestLength*(conf.pages-1));

                if ( requestStart < 0 ) {
                    requestStart = 0;
                }
            }

            cacheLower = requestStart;
            cacheUpper = requestStart + (requestLength * conf.pages);

            request.start = requestStart;
            request.length = requestLength*conf.pages;

            // Provide the same `data` options as DataTables.
            if ( typeof conf.data === 'function' ) {
                // As a function it is executed with the data object as an arg
                // for manipulation. If an object is returned, it is used as the
                // data object to submit
                var d = conf.data( request );
                if ( d ) {
                    $.extend( request, d );
                }
            }
            else if ( $.isPlainObject( conf.data ) ) {
                // As an object, the data given extends the default
                $.extend( request, conf.data );
            }

            settings.jqXHR = $.ajax( {
                "type":     conf.type,
                "url":      conf.url,
                "data":     request,
                "dataType": "json",
                "cache":    false,
                "success":  function ( json ) {
                    cacheLastJson = $.extend(true, {}, json);

                    if ( cacheLower != drawStart ) {
                        json.data.splice( 0, drawStart-cacheLower );
                    }
                    if ( requestLength >= -1 ) {
                        json.data.splice( requestLength, json.data.length );
                    }

                    drawCallback( json );
                }
            } );
        }
        else {
            json = $.extend( true, {}, cacheLastJson );
            json.draw = request.draw; // Update the echo for each response
            json.data.splice( 0, requestStart-cacheLower );
            json.data.splice( requestLength, json.data.length );

            drawCallback(json);
        }
    }
};

// Register an API method that will empty the pipelined data, forcing an Ajax
// fetch on the next draw (i.e. `table.clearPipeline().draw()`)
$.fn.dataTable.Api.register( 'clearPipeline()', function () {
    return this.iterator( 'table', function ( settings ) {
        settings.clearCache = true;
    } );
} );
