<?php

/*
|--------------------------------------------------------------------------
| Web Routes
|--------------------------------------------------------------------------
|
| Here is where you can register web routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| contains the "web" middleware group. Now create something great!
|
*/

// TODO počisti sve viewove od nepotrebnih JS-a i CSS-a

// Logo ustanove
Route::get('logo/{cacheKey}', 'Admin\LogoController@show')->name('logo');

// Rute kojma korisnik ne može pristupiti ako je ulogiran (prvo se mora odjaviti).
Route::group(['middleware' => ['guest']], function () {
    Route::get('/', function () {
        return view('naslovnica');
    })->name('home');

    Route::get('/admin/prijava', 'Auth\LoginController@adminLoginView')->name('admin.login.view');
    Route::post('/admin/prijava', 'Auth\LoginController@adminLogin')->name('admin.login');

    Route::get('/korisnik/prijava', 'Auth\LoginController@userLoginView')->name('user.login.view');
    Route::post('/korisnik/prijava', 'Auth\LoginController@userLogin')->name('user.login');
});

// Sljedeće rute su dostupne samo autenticiranim korisnicima.
Route::group(['middleware' => ['auth']], function () {

    Route::post('/odjava', 'Auth\LoginController@logout')->name('logout');

    // TODO makni aosi demo
    Route::get('/aosi-demo', 'AOSIDemoController@index');

    // Rute za obične korisnike. Samom autentikacijom korisnik ima pravo na korisničke rute.
    Route::group([
        'prefix' => 'user',
    ], function () {
        Route::get('/uredi', 'User\UserController@edit')->name('user.edit');
        Route::put('/', 'User\UserController@update')->name('user.update');
        Route::get('slika/{hash}', 'User\UserController@pictureShow')->name('user.picture.show');
        Route::delete('slika/{hash}', 'User\UserController@pictureDelete')->name('user.picture.delete');
        Route::get('zaporka/uredi', 'User\UserController@passwordEdit')->name('user.password.edit');
        Route::put('zaporka', 'User\UserController@passwordUpdate')->name('user.password.update');
    });

    // Rute za administratore.
    Route::group([
        'prefix' => 'admin',
        'middleware' => 'can:admin'
    ], function () {
        // TODO uniformiti rute
        Route::get('korisnici/popis', 'AdminController@userListShow')->name('admin.user.list.show');
        Route::get('korisnici/upravljanje', 'AdminController@bulkManagementShow')->name('admin.user.bulk-management.show');
        Route::post('korisnici/getUserList', 'AdminController@getUserList');  // Putanja za AJAX dohvat
        Route::post('korisnici/export', 'AdminController@export')->name('admin.user.list.export');
        Route::post('korisnici/deleteUser', 'AdminController@deleteUser');  // Putanja za AJAX dohvat
        Route::post('korisnici/changeExpireDate', 'AdminController@changeExpireDate');  // Putanja za AJAX dohvat

        Route::get('korisnici/azuriraj/{uid}', 'Admin\UserListController@edit')->name('admin.user.edit');
        Route::put('korisnici/azuriraj/{uid}', 'Admin\UserListController@update')->name('admin.user.update');
        Route::get('korisnici/slika/{uid}/{hash}', 'Admin\UserListController@pictureShow')->name('admin.user.picture.show');
        Route::delete('korisnici/slika/{uid}/{hash}', 'Admin\UserListController@pictureDelete')->name('admin.user.picture.delete');
        Route::patch('korisnici/zaporka/resetiraj/{uid}', 'Admin\UserListController@passwordReset')->name('admin.user.password.reset');
        Route::get('korisnici/zaporka/obavijest/{uid}', 'Admin\UserListController@passwordResetNotice')->name('admin.user.password.reset.notice');
        Route::get('korisnici/zaporka/provjeri', 'Admin\UserListController@passwordCheck')->name('admin.user.password.check');
        Route::post('korisnici/zaporka/provjeri', 'Admin\UserListController@passwordCheckResult')->name('admin.user.password.check.result');

        Route::get('korisnici/dodaj','Admin\UserAddController@create')->name('admin.user.create');
        Route::post('korisnici','Admin\UserAddController@store')->name('admin.user.store');

        Route::get('korisnici/iz-datoteke/dodaj','Admin\UserFromFileController@add')->name('admin.user.from-file.add');
        Route::post('korisnici/iz-datoteke/dodaj','Admin\UserFromFileController@addStore')->name('admin.user.from-file.addStore');
        Route::get('korisnici/iz-datoteke/brisi','Admin\UserFromFileController@delete')->name('admin.user.from-file.delete');
        Route::post('korisnici/iz-datoteke/brisi','Admin\UserFromFileController@deleteStore')->name('admin.user.from-file.deleteStore');
        Route::get('korisnici/iz-datoteke/azuriraj','Admin\UserFromFileController@update')->name('admin.user.from-file.update');
        Route::post('korisnici/iz-datoteke/azuriraj','Admin\UserFromFileController@updateStore')->name('admin.user.from-file.updateStore');

        Route::get('korisnici/zakljucani', 'Admin\LockedUsersController@index')->name('admin.user.locked.index');

        Route::get('administratori', 'Admin\AdminListController@index')->name('admin.admin.index');
        Route::get('administratori/azuriraj/{uid}', 'Admin\AdminListController@edit')->name('admin.admin.edit');
        Route::put('administratori/azuriraj/{uid}', 'Admin\AdminListController@update')->name('admin.admin.update');
        Route::delete('administratori/azuriraj/{uid}', 'Admin\AdminListController@revoke')->name('admin.admin.revoke');

        Route::get('hreduorg', 'Admin\HrEduOrgController@edit')->name('admin.hreduorg.edit');
        Route::put('hreduorg', 'Admin\HrEduOrgController@update')->name('admin.hreduorg.update');

        Route::get('logo/uredi', 'Admin\LogoController@edit')->name('admin.logo.edit');
        Route::put('logo', 'Admin\LogoController@update')->name('admin.logo.update');

        Route::get('pregled-loga-akcija', 'Admin\ActionLogController@show')->name('admin.actionLog.show');

    });
});

//Route::get('/aosi-demo', 'AOSIDemoController@index');
Route::any('/AOSI', 'AosiSoapServerController@handle');
Route::any('/AOSI-RPC', 'AosiSoapRpcServerController@handle');
//Route::any('/TEST', 'TestniServer@handle');
//Route::any('/client/{funkcija}', 'AosiSoapClientController@go');
//Route::any('/klijent', 'TestniClient@go');
